@@ -122,7 +122,7 @@
 
 (defun mew-addrbook-clean-up ()
   (remove-hook 'kill-emacs-hook 'mew-addrbook-clean-up)
-  (when mew-alias-auto-alist
+  (when (and mew-alias-auto-file mew-alias-auto-alist)
     (mew-lisp-save mew-alias-auto-file mew-alias-auto-alist))
   (setq mew-alias-auto-alist nil))
 
@@ -420,7 +420,7 @@ it will add an alias."
   (interactive)
   (setq major-mode 'mew-addrbook-mode)
   (setq mode-name mname)
-  (setq mode-line-buffer-identification mew-mode-line-id)
+  (setq mode-line-buffer-identification (mew-mode-line-id))
   (use-local-map mew-addrbook-mode-map)
   (run-hooks 'mew-addrbook-mode-hook)
   (setq buffer-undo-list nil))
@@ -545,7 +545,7 @@ reflected."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2009 Mew developing team.
+;; Copyright (C) 1999-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
