@@ -39,7 +39,7 @@ To compose a multipart message, you should execute this command first."
 	(setq buffer-undo-list nil)))))
 
 (defun mew-attach-expand-path (syntax nums)
-  "Ignore the first dir"
+  ;; Ignore the first dir
   (let ((path ""))
     (while (cdr nums)
       (setq syntax (aref syntax (+ mew-syntax-magic (1- (car nums)))))
@@ -48,9 +48,9 @@ To compose a multipart message, you should execute this command first."
     path))
 
 (defun mew-attach-line ()
-  "Return the line number for the cursor from the beginning of the 
-attachments region. 
-0 if on the beginning boundary. 
+  "Return the line number for the cursor from the beginning of the
+attachments region.
+0 if on the beginning boundary.
 -1 if on the ending boundary."
   (let (ret max)
     (save-excursion
@@ -151,9 +151,8 @@ attachments region.
 	  (mew-attach-move-onto-file))
       (setq numreg (number-to-string (car nums)))
       (setq nums (cdr nums))
-      (while nums
-	(setq numreg (concat (number-to-string (car nums)) "." numreg))
-	(setq nums (cdr nums)))
+      (dolist (num nums)
+	(setq numreg (concat (number-to-string num) "." numreg)))
       (setq numreg (regexp-quote numreg))
       (goto-char (1+ (mew-attach-begin)))
       (re-search-forward (concat "^....." numreg " ") nil t)
@@ -188,7 +187,7 @@ attachments region.
 
 (defun mew-attach-forward ()
   "Go to the first subdirectory in attachments."
-  (interactive) 
+  (interactive)
   (if (mew-attach-line0-1)
       (mew-attach-goto-number 'here nil)
     (mew-attach-move-onto-file)
@@ -250,9 +249,6 @@ attachments region.
 	  (mew-delete-directory-recursively (mew-attachdir))
 	  (setq mew-encode-syntax nil)
 	  (mew-attach-clear)
-	  (if (and mew-xemacs-p mew-icon-p)
-	      (set-specifier default-toolbar
-			     (cons (current-buffer) mew-draft-toolbar)))
 	  (message "Attachments were deleted"))
       (let* ((nums (mew-syntax-nums))
 	     (subdir (mew-attach-expand-path mew-encode-syntax nums))
@@ -302,7 +298,7 @@ attachments region.
 	(mew-make-directory dirpath)
 	(setq mew-encode-syntax
 	      (mew-syntax-insert-entry
-	       mew-encode-syntax 
+	       mew-encode-syntax
 	       nums
 	       (mew-encode-syntax-multi dir mew-type-mlm)))
 	(mew-encode-syntax-print mew-encode-syntax)))))
@@ -319,12 +315,13 @@ attachments region.
 	   (tofile to)
 	   ct msg FILE)
       (while (or (null frompath)
-		 (file-directory-p (file-chase-links frompath)))
+		 (file-directory-p (file-chase-links frompath))
+		 (not (file-exists-p (file-chase-links frompath))))
 	(setq frompath (mew-draft-input-file-name
 			(concat (mew-capitalize action) " from: ") default)))
       (while (or (null efile)
 		 (file-exists-p (expand-file-name efile attachdir)))
-	(setq tofile (mew-input-string 
+	(setq tofile (mew-input-string
 		      (concat (mew-capitalize action) " to %s(%s): ")
 		      subdir
 		      (file-name-nondirectory frompath)))
@@ -344,7 +341,7 @@ attachments region.
 		     "Attached as binary. Type '\\[mew-attach-toggle]' to toggle binary to text")))))
       (setq mew-encode-syntax
 	    (mew-syntax-insert-entry
-	     mew-encode-syntax 
+	     mew-encode-syntax
 	     nums
 	     (mew-encode-syntax-single tofile (list ct) nil nil nil 'cdp)))
       (mew-encode-syntax-print mew-encode-syntax)
@@ -419,7 +416,7 @@ on '.' in attachments."
 	(setq file (file-name-nondirectory filepath))
 	(setq mew-encode-syntax
 	      (mew-syntax-insert-entry
-	       mew-encode-syntax 
+	       mew-encode-syntax
 	       nums
 	       (mew-encode-syntax-single file (list mew-ct-txt) ;; for charset
 					 nil nil nil nil)))
@@ -487,7 +484,7 @@ on '.' in attachments."
 	   (syntax (mew-syntax-get-entry mew-encode-syntax nums)))
       (mew-syntax-set-privacy syntax nil)
       (mew-syntax-set-decrypters syntax nil)
-      ;; 
+      ;;
       (mew-syntax-set-cte syntax
 			  (mew-ctdb-cte
 			   (mew-ctdb-by-ct
@@ -527,33 +524,31 @@ on '.' in attachments."
   (mew-attach-charset-base "icharset"))
 
 (defun mew-attach-charset-base (param)
-  (if (not mew-mule-p)
-      (message "This command cannot be used on this Emacs")
-    (if (not (mew-attach-not-line0-1-dot))
-	(message "Cannot specify character set here")
-      (let* ((nums (mew-syntax-nums))
-	     (syntax (mew-syntax-get-entry mew-encode-syntax nums))
-	     (file (mew-syntax-get-file syntax))
-	     (ctl (mew-syntax-get-ct syntax))
-	     (ct (mew-syntax-get-value ctl 'cap))
-	     (params (mew-syntax-get-params ctl))
-	     (char (mew-syntax-get-param ctl param))
-	     charset)
-	(if (not (mew-ct-textp ct))
-	    (message "Cannot use character set %s" ct)
-	  (unless char (setq char "guess"))
-	  (setq charset (mew-input-type
-			 (if (string= param "charset")
-			     "Output message charset for %s (%s): "
-			   "Input file charset for %s (%s): ")
-			 file char
-			 (cons "guess" mew-charset-list)))
-	  (setq params (mew-delete param params))
-	  (if (string= charset "guess")
-	      (setq ctl (cons ct params))
-	    (setq ctl (cons ct (cons (list param charset) params))))
-	  (mew-syntax-set-ct syntax ctl)
-	  (mew-encode-syntax-print mew-encode-syntax))))))
+  (if (not (mew-attach-not-line0-1-dot))
+      (message "Cannot specify character set here")
+    (let* ((nums (mew-syntax-nums))
+	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))
+	   (file (mew-syntax-get-file syntax))
+	   (ctl (mew-syntax-get-ct syntax))
+	   (ct (mew-syntax-get-value ctl 'cap))
+	   (params (mew-syntax-get-params ctl))
+	   (char (mew-syntax-get-param ctl param))
+	   charset)
+      (if (not (mew-ct-textp ct))
+	  (message "Cannot use character set %s" ct)
+	(unless char (setq char "guess"))
+	(setq charset (mew-input-type
+		       (if (string= param "charset")
+			   "Output message charset for %s (%s): "
+			 "Input file charset for %s (%s): ")
+		       file char
+		       (cons "guess" mew-charset-list)))
+	(setq params (mew-delete param params))
+	(if (string= charset "guess")
+	    (setq ctl (cons ct params))
+	  (setq ctl (cons ct (cons (list param charset) params))))
+	(mew-syntax-set-ct syntax ctl)
+	(mew-encode-syntax-print mew-encode-syntax)))))
 
 (defun mew-attach-description ()
   "Input a description(Content-Description:) in attachments."
@@ -616,8 +611,8 @@ on '.' in attachments."
 	(mew-encode-syntax-print mew-encode-syntax)))))
 
 (defun mew-attach-gzip64 ()
-  "Put the 'G' mark to encode with Gzip64 in attachments. This is 
-applicable only to Text/Plain and Application/Postscript since compression 
+  "Put the 'G' mark to encode with Gzip64 in attachments. This is
+applicable only to Text/Plain and Application/Postscript since compression
 is not effective other objects. For example, JPEG is already compressed."
   (interactive)
   (if (not (mew-attach-not-line0-1-dot))
@@ -629,7 +624,7 @@ is not effective other objects. For example, JPEG is already compressed."
       (if (or (mew-ct-messagep ct)
 	      (mew-ct-multipartp ct)
 	      (not (mew-ct-linebasep ct)))
-	  ;; never use compress to types other than text/plain or 
+	  ;; never use compress to types other than text/plain or
 	  ;; application/postscript. Indeed, compression is not
 	  ;; effective to compressed data such as jpeg.
 	  (message "Cannot apply gzip to %s" ct)
@@ -655,13 +650,13 @@ is not effective other objects. For example, JPEG is already compressed."
        ((member (car pgp-sign) privacy) ;; xxx hard coding
 	(message "Already marked"))
        (t
-	(mew-syntax-set-privacy 
+	(mew-syntax-set-privacy
 	 syntax (append privacy pgp-sign))
 	(mew-encode-syntax-print mew-encode-syntax)))))))
 
 (defun mew-attach-pgp-enc ()
-  "Put the 'PE' mark to encrypt with PGP in attachment. 
-Input decryptors' addresses."
+  "Put the 'PE' mark to encrypt with PGP in attachment.
+Input decrypters' addresses."
   (interactive)
   (cond
    ((not (mew-attach-not-line0-1-dot))
@@ -685,7 +680,7 @@ Input decryptors' addresses."
 	    (setq towhom (mew-input-address "To (%s): " (mew-join "," towhom)))
 	  (setq towhom (mew-input-address "To: ")))
 	;; towhom is already canonicalized.
-	(mew-syntax-set-privacy 
+	(mew-syntax-set-privacy
 	 syntax (append privacy pgp-enc))
 	(mew-syntax-set-decrypters syntax towhom)
 	(mew-encode-syntax-print mew-encode-syntax)))))))
@@ -709,13 +704,13 @@ Input decryptors' addresses."
        ((member (car smime-sign) privacy) ;; xxx hard coding
 	(message "Already marked"))
        (t
-	(mew-syntax-set-privacy 
+	(mew-syntax-set-privacy
 	 syntax (append privacy smime-sign))
 	(mew-encode-syntax-print mew-encode-syntax)))))))
 
 (defun mew-attach-smime-enc ()
-  "Put the 'SE' mark to encrypt with S/MIME in attachment. 
-Input decryptors' addresses."
+  "Put the 'SE' mark to encrypt with S/MIME in attachment.
+Input decrypters' addresses."
   (interactive)
   (cond
    ((not (mew-attach-not-line0-1-dot))
@@ -739,7 +734,7 @@ Input decryptors' addresses."
 	    (setq towhom (mew-input-address "To (%s): " (mew-join "," towhom)))
 	  (setq towhom (mew-input-address "To: ")))
 	;; towhom is already canonicalized.
-	(mew-syntax-set-privacy 
+	(mew-syntax-set-privacy
 	 syntax (append privacy smime-enc))
 	(mew-syntax-set-decrypters syntax towhom)
 	(mew-encode-syntax-print mew-encode-syntax)))))))
@@ -755,7 +750,7 @@ Input decryptors' addresses."
     (let* ((nums (mew-syntax-nums))
 	   (subdir (mew-attach-expand-path mew-encode-syntax nums))
 	   (attachdir (mew-attachdir))
-	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))	   
+	   (syntax (mew-syntax-get-entry mew-encode-syntax nums))	
 	   (file (mew-syntax-get-file syntax))
 	   efile FILE)
       (setq efile (if (string= subdir "") file (concat subdir file)))
@@ -771,12 +766,6 @@ Input decryptors' addresses."
 ;; Drag & Drop
 ;;
 
-(defun mew-draft-dnd-test-function (window action types)
-  (let ((type (x-dnd-choose-type types)))
-    (if type
-	(cons action type)
-      nil)))
-
 (defun mew-draft-dnd-handle-local-file (uri action)
   (let* ((from (dnd-get-local-file-name uri t))
 	 (to (file-name-nondirectory from)))
@@ -795,7 +784,7 @@ Input decryptors' addresses."
 	(mew-attach-copy from to))
        ((eq action 'link)
 	(mew-attach-link from to))
-       (t 
+       (t
 	(message "Nothing happened"))))))
 
 (defun mew-draft-dnd-handle-file (uri action)
@@ -808,13 +797,13 @@ Input decryptors' addresses."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2006 Mew developing team.
+;; Copyright (C) 1996-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -823,7 +812,7 @@ Input decryptors' addresses."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
