@@ -22,7 +22,7 @@
 ;;; Readability
 ;;;
 
-(defconst mew-sp ?\ ) ;; 32 does not work on XEmacs
+(defconst mew-sp ?\ )
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -141,7 +141,7 @@
 (defconst mew-bin  "binary")
 (defconst mew-xuu  "x-uuencode")
 
-(defconst mew-decode-value 
+(defconst mew-decode-value
   `(,mew-b64 ,mew-qp ,mew-xg ,mew-7bit ,mew-8bit ,mew-bin ,mew-xuu))
 (defconst mew-decode-composite-value
   `(,mew-7bit ,mew-8bit ,mew-bin))
@@ -164,16 +164,11 @@
   (+ (length [key beg end pri]) (length mew-mime-fields)))
 
 (defconst mew-x-mailer
-  (concat mew-version " on "
-          (if (string-match "XEmacs" emacs-version) "XEmacs" "Emacs")
-          (if (boundp 'emacs-program-version)
-              (format " %s " emacs-program-version)
-	    (string-match "^\\([.0-9]+\\)\\.[0-9]+$" emacs-version)
-	    (concat " " (match-string 1 emacs-version) " "))
-          (and (boundp 'xemacs-codename)
-	       (string-match "^[a-zA-Z0-9 ]+$" xemacs-codename)
-	       (concat "(" xemacs-codename ")"))
-          (and (boundp 'mule-version) (concat "/ Mule " mule-version)))
+  (format "%s on Emacs %s / Mule %s"
+	  mew-version
+	  (when (string-match "^\\([.0-9]+\\)\\.[0-9]+$" emacs-version)
+	    (match-string 1 emacs-version))
+	  mule-version)
   "*A value inserted into X-Mailer: field in Draft mode if *non-nil*.")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -223,13 +218,13 @@
 (defconst mew-regex-mark "^\\([^ \n]\\)")
 (defun mew-sumsyn-mark () (string-to-char (mew-match-string 1)))
 
-;; fld msg 
+;; fld msg
 (defconst mew-regex-sumsyn-short   "\r \\([^ \n]*\\) \\([0-9]+\\)")
 
 ;; fld msg my-id par-id uid siz
 (defconst mew-regex-sumsyn-long    "\r \\([^ \n]*\\) \\([0-9]+\\) \\([^ \n]*\\) \\([^ \n]*\\) \\([^ \n]*\\) \\([0-9]*\\)")
 
-;; fld msg 
+;; fld msg
 (defun mew-regex-sumsyn-msg (msg)
                            (format "\r \\([^ \n]*\\) \\(%s\\) " msg))
 
@@ -328,7 +323,8 @@
 (defconst mew-inherit-7bit nil)
 (defconst mew-inherit-offline nil)
 (defconst mew-inherit-decode-signer nil)
-(defconst mew-inherit-encode-signer nil)
+(defconst mew-inherit-encode-pgp-signer nil)
+(defconst mew-inherit-encode-smime-signer nil)
 (defconst mew-inherit-prefetching nil)
 (defconst mew-inherit-refile-case nil)
 (defconst mew-inherit-refile-proto nil)
@@ -340,18 +336,19 @@
 (defconst mew-inherit-pick-omit-and nil)
 (defconst mew-inherit-pick-omit-and2 nil)
 (defconst mew-inherit-submission nil)
+(defconst mew-inherit-complete-folder nil)
 
 (provide 'mew-const)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2007 Mew developing team.
+;; Copyright (C) 2000-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -360,7 +357,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
