@@ -15,6 +15,7 @@
 (defvar mew-prog-plain    'mew-mime-text/plain)
 (defvar mew-prog-html     '(mew-mime-text/html mew-mime-text/html-ext))
 (defvar mew-prog-xml      '(mew-mime-text/xml  mew-mime-text/xml-ext))
+(defvar mew-prog-patch    '(mew-mime-text/plain mew-mime-text/patch-ext))
 (defvar mew-prog-enriched 'mew-mime-text/enriched)
 (defvar mew-prog-text     'mew-mime-text/plain)
 (defvar mew-prog-audio           mew-darwin-exec)
@@ -41,6 +42,7 @@
 (defvar mew-prog-visio           mew-darwin-exec)
 (defvar mew-prog-ooffice         mew-darwin-exec)
 (defvar mew-prog-rtf             mew-darwin-exec)
+(defvar mew-prog-unzip           mew-darwin-exec)
 
 ;;;
 ;;; Text/Html, Application/Xml, Image
@@ -68,20 +70,33 @@
 
 ;;
 
-(setq mew-delete-temp-file  nil)
+(setq mew-delete-temp-file nil)
 (defvar mew-dir-list-function 'mew-dir-list-with-link-count)
 
+;;
+
+(defun mew-set-file-type (file)
+  (unless mew-use-suffix
+    (mew-mac-set-file-type file mew-file-type)))
+
+(defvar mew-cs-est 'utf-8)
+
+(defun mew-focus-frame (frame)
+  (when focus-follows-mouse
+    (set-mouse-position
+     (selected-frame) (1- (frame-width)) 0)))
+
 (provide 'mew-darwin)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2005 Mew developing team.
+;; Copyright (C) 2002-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -90,7 +105,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
