@@ -17,7 +17,7 @@
     (,mew-qp  "-q")
     (,mew-xg  "-g""-t")))
 
-(defsubst mew-prog-mime-encode-get-opt (cte switch)
+(defun mew-prog-mime-encode-get-opt (cte switch)
   (cdr (mew-assoc-case-equal cte switch 0)))
 
 (defvar mew-encode-multipart-encrypted-switch
@@ -29,10 +29,10 @@
 
 ;;
 
-(defsubst mew-encode-get-security-func (proto switch)
+(defun mew-encode-get-security-func (proto switch)
   (nth 1 (mew-assoc-case-equal proto switch 0)))
 
-(defsubst mew-encode-get-canonicalize-func (proto switch)
+(defun mew-encode-get-canonicalize-func (proto switch)
   (nth 2 (mew-assoc-case-equal proto switch 0)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -40,7 +40,7 @@
 ;;; MIME encoder
 ;;;
 
-(defsubst mew-encode-error (error-msg)
+(defun mew-encode-error (error-msg)
   (mew-tinfo-set-encode-err error-msg)
   (error ""))
 
@@ -73,7 +73,7 @@
   (list mew-subj: mew-in-reply-to: mew-references:))
 
 (defconst mew-multiple-field-combine-list
-  (list mew-from: mew-reply-to: mew-to: mew-cc:)) 
+  (list mew-from: mew-reply-to: mew-to: mew-cc:))
 
 (defun mew-header-count-field (field)
   (let ((case-fold-search t)
@@ -116,11 +116,9 @@
     (save-excursion
       (save-restriction
 	(narrow-to-region (point-min) (mew-header-end))
-	(let ((fields mew-multiple-field-error-list))
-	  (while fields
-	    (if (> (mew-header-count-field (car fields)) 1)
-		(mew-encode-error (format "Multiple %s!" (car fields))))
-	    (setq fields (cdr fields))))
+	(dolist (field mew-multiple-field-error-list)
+	  (if (> (mew-header-count-field field) 1)
+	      (mew-encode-error (format "Multiple %s!" field))))
 	(mapcar 'mew-header-combine-field mew-multiple-field-combine-list)))))
 
 (defun mew-encode-ask-subject ()
@@ -129,23 +127,23 @@
     (let ((subj (read-string (concat mew-subj: " "))))
       (mew-header-replace-value mew-subj: subj))))
 
+(defun mew-encode-fcc-folder-check (fld)
+  (let ((folder (mew-canonicalize-folder fld)))
+    (when (and mew-ask-fcc
+	       (mew-folder-localp folder)
+	       (not (file-directory-p (mew-expand-folder folder))))
+      (if (y-or-n-p (format "%s does not exist. Create it? " folder))
+	  (mew-local-folder-check folder)
+	(mew-encode-error "Edit Fcc:")))
+    folder))
+
 (defun mew-encode-ask-fcc (resentp)
   (let* ((target (if resentp mew-resent-fcc: mew-fcc:))
-	 (folders (mew-header-get-value target))
-	 folder ret)
+	 (folders (mew-header-get-value target)))
     (when folders
-      (setq folders (mew-addrstr-parse-value-list2 folders))
-      (while folders
-	(setq folder (mew-canonicalize-folder (car folders)))
-	(setq folders (cdr folders))
-	(when (and mew-ask-fcc
-		   (mew-folder-localp folder)
-		   (not (file-directory-p (mew-expand-folder folder))))
-	  (if (y-or-n-p (format "%s does not exist. Create it? " folder))
-	      (mew-local-folder-check folder)
-	    (mew-encode-error "Edit Fcc:")))
-	(setq ret (cons folder ret))))
-    (nreverse ret)))
+      (mapcar
+       'mew-encode-fcc-folder-check
+       (mew-addrstr-parse-value-list2 folders)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -183,9 +181,7 @@
 	    (backward-char 1)
 	    ;; single quote is allowed for local-part
 	    (setq vals (mapcar 'mew-chop (mew-split-quoted val ?, ?: ?\; 'no-single)))
-	    (while vals
-	      (setq val (car vals))
-	      (setq vals (cdr vals))
+	    (dolist (val vals)
 	      (setq ins nil addrs nil)
 	      (cond
 	       ;; phrase:addr1,addr2;
@@ -213,7 +209,7 @@
 		    (setq ins (copy-sequence addrs))))))
 	      (setq insl (nconc insl ins))
 	      (unless (string-match "bcc" key) ;; Removing Bcc:
-		(setq ret (nconc ret addrs)))) ;; end of while
+		(setq ret (nconc ret addrs)))) ;; end of dolist
 	    (insert " " (mapconcat 'identity insl ", "))
 	    (setq insl nil)
 	    (forward-line))))
@@ -302,7 +298,7 @@
 ;;; Misc
 ;;;
 
-(defun mew-encode-set-privacy (pnm privacy)
+(defun mew-encode-set-privacy (pnm privacy case)
   (unless (or (mew-syntax-get-privacy mew-encode-syntax) ;; specified
 	      ;; encryption previously failed, so bypass.
 	      (and (null privacy) (mew-tinfo-get-privacy-err)))
@@ -312,10 +308,10 @@
 	(setq type privacy))
        ((mew-tinfo-get-privacy-type)
 	(setq type (mew-tinfo-get-privacy-type)))
-       ((and (mew-tinfo-get-encrypted-p) mew-protect-privacy-encrypted)
-	(setq type mew-protect-privacy-encrypted-type))
-       (mew-protect-privacy-always
-	(setq type mew-protect-privacy-always-type)))
+       ((and (mew-tinfo-get-encrypted-p) (mew-protect-privacy-encrypted case))
+	(setq type (mew-protect-privacy-encrypted-type case)))
+       ((mew-protect-privacy-always case)
+	(setq type (mew-protect-privacy-always-type case))))
       (mew-syntax-set-privacy
        mew-encode-syntax
        (mew-pcdb-ct (mew-pcdb-by-service type)))
@@ -349,12 +345,11 @@
   (mew-highlight-header-region (point-min) (point)))
 
 (defun mew-encode-save-draft ()
-  (mew-frwlet
-   mew-cs-dummy mew-cs-text-for-write
-   (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)
-   (clear-visited-file-modtime)
-   (mew-delete-file buffer-auto-save-file-name)
-   (set-buffer-modified-p nil)))
+  (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+    (write-region (point-min) (point-max) (buffer-file-name) nil 'no-msg)
+    (clear-visited-file-modtime)
+    (mew-delete-file buffer-auto-save-file-name)
+    (set-buffer-modified-p nil)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -382,7 +377,7 @@
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
-  (let (multip recipients msgid-logtime)
+  (let (multip recipients msgid logtime)
     (mew-smtp-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
     (unless headerp
@@ -417,9 +412,9 @@
     (mew-encode-check-sender resentp)
     (mew-encode-from case resentp)
     ;;
-    (setq msgid-logtime (mew-encode-id-date pnm (mew-smtp-message-id case) resentp))
-    (mew-smtp-set-msgid pnm (nth 0 msgid-logtime))
-    (mew-smtp-set-logtime pnm (nth 1 msgid-logtime))
+    (mew-set '(msgid logtime) (mew-encode-id-date pnm (mew-smtp-message-id case) resentp))
+    (mew-smtp-set-msgid pnm msgid)
+    (mew-smtp-set-logtime pnm logtime)
     ;;
     (goto-char (mew-header-end))
     (forward-line) ;; necessary for PGP
@@ -427,8 +422,17 @@
     (message "Making a message...")
     ;; save syntax before setting privacy
     (unless headerp
-      (mew-encode-set-privacy pnm privacy)
-      (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
+      (mew-encode-set-privacy pnm privacy case)
+      (let (mew-inherit-encode-pgp-signer
+	    mew-inherit-encode-smime-signer)
+	(setq mew-inherit-encode-pgp-signer
+	      (or signer
+		  (mew-pgp-signer (mew-tinfo-get-case))
+		  (mew-get-my-address)))
+	(setq mew-inherit-encode-smime-signer
+	      (or signer
+		  (mew-smime-signer (mew-tinfo-get-case))
+		  (mew-get-my-address)))
 	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
 	(if multip
@@ -462,7 +466,7 @@
   (mew-set-buffer-multibyte t)
   (if (buffer-modified-p) (save-buffer)) ;; to make backup
   (widen)
-  (let (multip newsgroups msgid-logtime)
+  (let (multip newsgroups msgid logtime)
     (mew-nntp2-set-raw-header
      pnm (mew-buffer-substring (point-min) (mew-header-end)))
     ;; Let's backup
@@ -487,9 +491,9 @@
     ;;
     (mew-encode-from case nil)
     ;;
-    (setq msgid-logtime (mew-encode-id-date pnm (mew-nntp-message-id case)))
-    (mew-nntp2-set-msgid pnm (nth 0 msgid-logtime))
-    (mew-nntp2-set-logtime pnm (nth 1 msgid-logtime))
+    (mew-set '(msgid logtime) (mew-encode-id-date pnm (mew-nntp-message-id case)))
+    (mew-nntp2-set-msgid pnm msgid)
+    (mew-nntp2-set-logtime pnm logtime)
     ;;
     (goto-char (mew-header-end))
     (forward-line) ;; necessary for PGP
@@ -497,8 +501,17 @@
     (message "Making a message...")
     ;; save syntax before setting privacy
     (unless headerp
-      (mew-encode-set-privacy pnm privacy)
-      (let ((mew-inherit-encode-signer (or signer (mew-get-my-address))))
+      (mew-encode-set-privacy pnm privacy case)
+      (let (mew-inherit-encode-pgp-signer
+	    mew-inherit-encode-smime-signer)
+	(setq mew-inherit-encode-pgp-signer
+	      (or signer
+		  (mew-pgp-signer (mew-tinfo-get-case))
+		  (mew-get-my-address)))
+	(setq mew-inherit-encode-smime-signer
+	      (or signer
+		  (mew-smime-signer (mew-tinfo-get-case))
+		  (mew-get-my-address)))
 	(goto-char (mew-header-end)) ;; due to invalid null lines in the header
 	(forward-line)
 	(if multip
@@ -509,7 +522,7 @@
     (mew-overlay-delete-buffer)
     (message "Making a message...done")))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;  
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Encoding singlepart
 ;;;
@@ -533,7 +546,7 @@
   (if (and (mew-which-exec mew-prog-8bit) (file-readable-p file))
       (eq (call-process mew-prog-8bit file nil nil "-8") 1)))
 
-(defsubst mew-encode-limit-7bitp (privacy)
+(defun mew-encode-limit-7bitp (privacy)
   ;; Only parts to be signed FIRST are limited to 7bit.
   (and privacy (mew-case-equal mew-ct-mls (car (car privacy)))))
 
@@ -557,12 +570,11 @@
                      mew-prog-mime-encode-text-switch
                    mew-prog-mime-encode-switch))
          (beg (point))
-	 buffer cs opt last)
+	 buffer cs opt last flowed delsp flowed-delsp)
     (if (and (stringp file)
 	     (setq buffer (get-file-buffer file))
 	     (buffer-modified-p buffer))
-	(save-excursion
-	  (set-buffer buffer)
+	(with-current-buffer buffer
 	  (save-buffer)))
     (when textp
       (when (and (stringp file) (file-readable-p file))
@@ -572,9 +584,8 @@
 	    (mew-encode-error
 	     (format "Unknown coding system %s in the body"
 		     (symbol-name cs)))))
-	(mew-frwlet
-	 (or cs mew-cs-autoconv) mew-cs-dummy
-	 (mew-insert-file-contents file)))
+	(mew-frwlet (or cs mew-cs-autoconv) mew-cs-dummy
+	  (mew-insert-file-contents file)))
       ;; A user may specify charset to convey JIS X 0201 Katakana.
       ;; So, we need to avoid the sanity check.
       (unless charset
@@ -582,6 +593,15 @@
 	    (mew-charset-sanity-check beg (point-max)))
 	(setq charset (mew-charset-guess-region beg (point-max))))
       (setq cs (mew-charset-to-cs charset))
+      (when (string= mew-ct-txt ct)
+	(cond
+	 ((mew-tinfo-get-flowed)
+	  (setq flowed t)
+	  (if (string= (mew-tinfo-get-flowed) "yes")
+	      (setq delsp t)))
+	 ((mew-tinfo-get-use-flowed)
+	  (setq flowed-delsp (mew-encode-flowed beg (point-max) charset))
+	  (mew-set '(flowed delsp) flowed-delsp))))
       (unless (mew-coding-system-p cs)
 	(mew-encode-error
 	 (format "Unknown coding system %s in the body" (symbol-name cs))))
@@ -635,21 +655,19 @@
      (t
       (if (and no-encoding cte (mew-case-equal cte mew-b64))
 	  (setq cte mew-bin))
-      ;; There are 7bit ascii bodies such as 
+      ;; There are 7bit ascii bodies such as
       ;; application/pgp-encrypted and message/external-body.
       ;; If 7bit or 8bit, it should be linebase.
       (if (null cte) (setq cte mew-7bit))))
     (cond
      ((mew-case-equal cte mew-bin)
-      (mew-frwlet
-       mew-cs-binary mew-cs-dummy
-       (mew-insert-file-contents file)))
+      (mew-frwlet mew-cs-binary mew-cs-dummy
+	(mew-insert-file-contents file)))
      ((or (mew-case-equal cte mew-7bit) (mew-case-equal cte mew-8bit))
       ;; Certainly linebase here.
       (unless textp
-	(mew-frwlet
-	 mew-cs-text-for-read mew-cs-dummy
-	 (mew-insert-file-contents file)))
+	(mew-frwlet mew-cs-text-for-read mew-cs-dummy
+	  (mew-insert-file-contents file)))
       (when (and (string= ct mew-ct-msg)
 		 (or mew-inherit-7bit (not mew-use-8bit))
 		 (mew-case-equal cte mew-8bit))
@@ -659,10 +677,8 @@
 	  (mew-convert-message))))
      ((and (mew-case-equal cte mew-b64) (fboundp 'base64-encode-region))
       (unless textp
-	(mew-frwlet
-	 (if linebasep mew-cs-text-for-read mew-cs-binary)
-	 mew-cs-dummy
-	 (mew-insert-file-contents file)))
+	(mew-frwlet (if linebasep mew-cs-text-for-read mew-cs-binary) mew-cs-dummy
+	  (mew-insert-file-contents file)))
       (when linebasep
 	(goto-char beg)
 	(mew-lf-to-crlf))
@@ -675,24 +691,103 @@
 	  (mew-encode-error (concat "Unknown CTE: " cte))
 	(when textp
 	  (setq file (mew-make-temp-name))
-	  (mew-frwlet
-	   mew-cs-dummy mew-cs-text-for-write
-	   ;; NEVER use call-process-region for privacy reasons
-	   (write-region beg (point-max) file nil 'no-msg))
+	  (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	    ;; NEVER use call-process-region for privacy reasons
+	    (write-region beg (point-max) file nil 'no-msg))
 	  (delete-region beg (point-max)))
-	(mew-piolet
-	 mew-cs-text-for-read mew-cs-dummy
-	 (apply 'call-process mew-prog-mime-encode file t nil opt))
+	(mew-piolet mew-cs-text-for-read mew-cs-dummy
+	  (apply 'call-process mew-prog-mime-encode file t nil opt))
 	(if textp (mew-delete-file file))))
      (t
       (mew-encode-error (concat mew-prog-mime-encode " does not exist"))))
-    (list (if charset (list "charset" charset)) cte)))
+    (list cte charset flowed delsp)))
 
 (defun mew-encode-no-mime-encoding (privacy)
   (let ((first (car privacy)))
     (and (listp first)
-	 (or (string-equal (car first) mew-ct-mle)
-	     (string-equal (car first) mew-ct-smm)))))
+	 (or (string= (car first) mew-ct-mle)
+	     (string= (car first) mew-ct-smm)))))
+
+(defun mew-broken-name (cdpl)
+  (let ((mew-encode-word-max-length 1000) ;; xxx
+	(file (nth 1 (assoc "filename" cdpl)))
+	name)
+    (when (and file (string-match mew-regex-nonascii file))
+      (setq name (car (mew-header-encode-string file)))
+      ;; name must not be double-quoted here.
+      ;; mew-heaer-insert will do this later
+      (list "name" name))))
+
+(defun mew-encode-flowed (beg end charset)
+  "Encoding lines with RFC 3676"
+  (let (flowed delsp column)
+    (save-excursion
+      (goto-char beg)
+      (save-restriction
+	(narrow-to-region beg end)
+	(when (mew-encode-flowed-check)
+	  (setq flowed t)
+	  (setq delsp (mew-charset-to-delsp charset))
+	  (setq column mew-flowed-fold-length)
+	  (if delsp (setq column (1- column)))
+	  (while (not (eobp))
+	    (mew-encode-flowed-line column delsp)
+	    (forward-line)))))
+    (list flowed delsp)))
+
+(defun mew-encode-flowed-remove-trailing-sp ()
+  (while (and (not (bobp)) (= (char-before) mew-sp))
+    (delete-backward-char 1)))
+
+(defun mew-encode-flowed-line (column delsp)
+  (let (prefix beg)
+    (looking-at "^>*")
+    (setq prefix (mew-match-string 0))
+    (goto-char (match-end 0))
+    (if (looking-at " ")
+	(if (string= prefix "")
+	    (insert mew-flowed-stuffed)
+	  (progn
+	    (setq prefix (format "%s%c" prefix mew-flowed-stuffed))
+	    (forward-char)))
+      (when (looking-at "From ")
+	(insert mew-flowed-stuffed)
+	(setq prefix (format "%s%c" prefix mew-flowed-stuffed))))
+    (setq beg (point))
+    (save-excursion
+      (end-of-line)
+      (unless (and (= (- (point) beg) 3)
+		   (string= (mew-buffer-substring beg (point)) "-- "))
+	(mew-encode-flowed-remove-trailing-sp)))
+    (move-to-column column)
+    (catch 'loop
+      (while (not (eolp))
+	(cond
+	 (delsp
+	  (while (> (current-column) column)
+	    (backward-char 1))
+	  (insert " \n"))
+	 (t
+	  (if (search-backward " " beg t)
+	      (forward-char)
+	    (unless (search-forward " " (save-excursion (end-of-line) (point)) t)
+	      (throw 'loop (end-of-line))))
+	  (insert "\n")))
+	(insert prefix)
+	(setq beg (point))
+	(move-to-column column)))))
+
+(defun mew-encode-flowed-check ()
+  (catch 'loop
+    (save-excursion
+      (save-restriction
+	(narrow-to-region (point-min) (min (point-max) mew-file-max-size))
+	(goto-char (point-min))
+	(while (not (eobp))
+	  (end-of-line)
+	  (if (> (current-column) mew-flowed-fold-threshold)
+	      (throw 'loop t))
+	  (forward-line))))))
 
 (defun mew-encode-singlepart (syntax &optional path depth buffered coverp)
   ;; path is nil if called make-single or security multipart
@@ -709,15 +804,23 @@
 	 (no-encoding (mew-encode-no-mime-encoding privacy))
 	 (mew-inherit-7bit (mew-encode-limit-7bitp privacy))
 	 (beg (point))
-	 charset-cte charset bodybeg cst ask-cst)
-    (setq charset-cte (mew-encode-mime-body ctl cte (or buffered file) no-encoding))
+	 mret charset bodybeg cst ask-cst broken-name flowed delsp)
+    (setq mret (mew-encode-mime-body ctl cte (or buffered file) no-encoding))
     (goto-char beg)
-    (mew-set '(charset cte) charset-cte)
+    (mew-set '(cte charset flowed delsp) mret)
     (setq ctl (mew-delete "icharset" ctl))
     (when charset
       (setq ctl (mew-syntax-get-params ctl))
       (setq ctl (mew-delete "charset" ctl))
-      (setq ctl (cons ct (cons charset ctl))))
+      (setq ctl (cons ct (cons (list "charset" charset) ctl))))
+    (when flowed
+      (setq ctl (nconc ctl (list (list "format" "flowed"))))
+      (if delsp (setq ctl (nconc ctl (list (list "delsp" "yes"))))))
+    ;;
+    (setq ctl (mew-delete "name" ctl))
+    (setq broken-name (mew-broken-name cdpl))
+    (if broken-name (setq ctl (nconc ctl (list broken-name))))
+    ;;
     (mew-header-insert mew-ct: ctl)
     (mew-header-insert mew-cte: cte)
     (and cd (mew-header-insert mew-cd: cd))
@@ -735,7 +838,7 @@
       (mew-encode-security-multipart
        beg privacy depth (mew-syntax-get-decrypters syntax) cte))
     (goto-char (point-max))
-    (when (and coverp (setq cst (nth 1 charset)))
+    (when (and coverp (setq cst charset))
       (cond
        ((null mew-ask-charset)
 	;; not ask
@@ -788,7 +891,7 @@
 	  (setq first (mew-syntax-get-part mew-encode-syntax))
 	  (setq ct (mew-syntax-get-value (mew-syntax-get-ct first) 'cap))
 	  (if (not (mew-ct-textp ct))
-	      (setq multip t)		   
+	      (setq multip t)		
 	    (setq path (expand-file-name
 			(mew-syntax-get-file mew-encode-syntax) path))
 	    (setq privacy (mew-syntax-get-privacy mew-encode-syntax))
@@ -832,7 +935,7 @@
     (while (< cnt len)
       (insert "\n--" boundary "\n")
       (if (mew-syntax-multipart-p (aref syntax cnt))
-	  (setq 8bitp (mew-encode-multipart 
+	  (setq 8bitp (mew-encode-multipart
 		       (aref syntax cnt)
 		       fullname
 		       (1+ depth)))
@@ -851,7 +954,7 @@
     (save-excursion
       (goto-char cte-pos)
       (mew-header-insert mew-cte: (if (> 8bit-cnt 0) mew-8bit mew-7bit)))
-    (when privacy 
+    (when privacy
       (mew-encode-security-multipart
        beg privacy depth (mew-syntax-get-decrypters syntax) cte))
     (goto-char (point-max))
@@ -866,12 +969,11 @@
   (save-restriction
     (narrow-to-region beg (point-max))
     (let (proto ct)
-      (while privacy
-	(goto-char (point-min)) 
-	(mew-set '(ct proto) (car privacy))
+      (dolist (ent privacy)
+	(goto-char (point-min))
+	(mew-set '(ct proto) ent)
 	(setq ct (mew-capitalize ct))
-	(setq privacy (cdr privacy))
-	(cond 
+	(cond
 	 ((string= ct mew-ct-mle)
 	  (mew-encode-multipart-encrypted ct proto depth decrypters cte))
 	 ((string= ct mew-ct-mls)
@@ -911,9 +1013,8 @@
 	(mew-lf-to-crlf)))
     (setq send (point-max))
     (setq file (mew-make-temp-name))
-    (mew-frwlet
-     mew-cs-dummy ocs
-     (write-region sbeg send file nil 'no-msg))
+    (mew-frwlet mew-cs-dummy ocs
+      (write-region sbeg send file nil 'no-msg))
     (if retain
 	(mew-remove-buffer tmpbuf)
       (delete-region sbeg send))
@@ -926,9 +1027,6 @@
 	 (switch mew-encode-multipart-encrypted-switch) ;; save length
 	 (func (mew-encode-get-security-func proto switch))
 	 file1 file2 file3 cte2 cte3 fc errmsg)
-    (if (and mew-encrypt-to-myself
-	     (not (member mew-inherit-encode-signer decrypters)))
-	(setq decrypters (cons mew-inherit-encode-signer decrypters)))
     ;; Write the part converting line breaks.
     (setq file1 (mew-save-transfer-form (point-min) (point-max) nil cte))
     ;; The narrowed region stores nothing
@@ -938,7 +1036,7 @@
       (error
        (mew-delete-file file1)
        (mew-encode-error
-	(format "unknown error for %s. Check %s, anyway" 
+	(format "unknown error for %s. Check %s, anyway"
 		mew-ct-mle mew-temp-dir))))
     (mew-set '(file2 cte2 file3 cte3 errmsg) fc)
     (if errmsg
@@ -957,14 +1055,14 @@
       (mew-header-insert mew-cte: mew-7bit)
       (insert (format "\n--%s\n" boundary))
       ;; Insert control keys
-      (mew-encode-singlepart 
+      (mew-encode-singlepart
        (mew-encode-syntax-single file2 (list proto) cte2))
       (insert (format "\n--%s\n" boundary))
       ;; Insert encrypted body
-      (mew-encode-singlepart 
+      (mew-encode-singlepart
        (mew-encode-syntax-single file3 mew-type-apo cte3))
       (insert (format "\n--%s--\n" boundary))
-      ;; Throw away the garbage 
+      ;; Throw away the garbage
       (mew-delete-file file1)
       (mew-delete-file file2)
       (mew-delete-file file3))))
@@ -985,7 +1083,7 @@
       (error
        (mew-delete-file file1)
        (mew-encode-error
-	(format "unknown error for %s. Check %s, anyway" 
+	(format "unknown error for %s. Check %s, anyway"
 		mew-ct-mls mew-temp-dir))))
     (mew-set '(file2 cte2 micalg errmsg ct2 cdp2) fmc)
     (if errmsg
@@ -1008,10 +1106,10 @@
       (goto-char (point-max))
       ;; After the signed part
       (insert (format "\n--%s\n" boundary))
-      (mew-encode-singlepart 
+      (mew-encode-singlepart
        (mew-encode-syntax-single file2 (or ct2 (list proto)) cte2 nil nil cdp2))
       (insert (format "\n--%s--\n" boundary))
-      ;; Throw away the garbage 
+      ;; Throw away the garbage
       (mew-delete-file file1)
       (mew-delete-file file2))))
 
@@ -1040,15 +1138,12 @@
 	(if (null opt)
 	    (mew-encode-error (concat "Unknown CTE: " cte))
 	  (setq file (mew-make-temp-name))
-	  (mew-frwlet
-	   mew-cs-dummy
-	   (if linebasep mew-cs-text-for-write mew-cs-binary)
-	   ;; NEVER use call-process-region for privacy reasons
-	   (write-region (point-min) (point-max) file nil 'no-msg)
-	   (delete-region (point-min) (point-max)))
-	  (mew-piolet
-	   mew-cs-text-for-read mew-cs-dummy
-	   (apply 'call-process mew-prog-mime-encode file t nil opt))
+	  (mew-frwlet mew-cs-dummy (if linebasep mew-cs-text-for-write mew-cs-binary)
+	    ;; NEVER use call-process-region for privacy reasons
+	    (write-region (point-min) (point-max) file nil 'no-msg)
+	    (delete-region (point-min) (point-max)))
+	  (mew-piolet mew-cs-text-for-read mew-cs-dummy
+	    (apply 'call-process mew-prog-mime-encode file t nil opt))
 	  (mew-delete-file file)))
        (t
 	(mew-encode-error (concat mew-prog-mime-encode " does not exist")))))))
@@ -1095,7 +1190,7 @@
   ;; called on the beginning of a content-header
   (let ((case-fold-search t)
 	cthd-end ctbody-beg
-	ct-val cte-val 
+	ct-val cte-val
 	ctl ct cte charset)
     (unless (re-search-forward mew-eoh nil t)
       (goto-char (point-max)))
@@ -1197,14 +1292,15 @@
   (let* ((file (buffer-file-name))
 	 (back (mew-prepend-prefix file mew-backup-prefix))
 	 (info (concat file mew-draft-info-suffix)))
-    (mew-frwlet
-     mew-cs-dummy mew-cs-m17n
-     (write-region (point-min) (point-max) back nil 'no-msg))
+    (mew-frwlet mew-cs-dummy mew-cs-m17n
+      (write-region (point-min) (point-max) back nil 'no-msg))
     (mew-lisp-save
      info
      (list
       (cons "Syntax:" mew-encode-syntax)
       (cons "Case:" (mew-tinfo-get-case))
+      (cons "Flowed:" (mew-tinfo-get-flowed))
+      (cons "Use-Flowed:" (mew-tinfo-get-use-flowed))
       (cons "Message:" (mew-tinfo-get-hdr-file))) ;; Header mode
      'nobackup)
     nil)) ;; to save
@@ -1220,6 +1316,8 @@
 	       (mew-syntax-singlepart-p mew-encode-syntax))
 	  (setq mew-encode-syntax nil))
       (mew-tinfo-set-case (cdr (assoc "Case:" syntax)))
+      (mew-tinfo-set-flowed (cdr (assoc "Flowed:" syntax)))
+      (mew-tinfo-set-flowed (cdr (assoc "Use-Flowed:" syntax)))
       (mew-tinfo-set-hdr-file (cdr (assoc "Message:" syntax))) ;; Header mode
       t)))
 
@@ -1227,9 +1325,8 @@
   (let* ((file (buffer-file-name))
 	 (back (mew-prepend-prefix file mew-backup-prefix)))
     (when (file-exists-p back)
-      (mew-frwlet
-       mew-cs-m17n mew-cs-dummy
-       (mew-insert-file-contents back)))))
+      (mew-frwlet mew-cs-m17n mew-cs-dummy
+	(mew-insert-file-contents back)))))
 
 (defun mew-encode-undo ()
   "Get back to the draft before making MIME message."
@@ -1244,20 +1341,19 @@
        (mew-header-prepared)
        (if mew-encode-syntax (mew-draft-prepare-attachments))
        (mew-draft-toolbar-update)
-       (setq mew-encode-syntax nil)
        (setq buffer-undo-list nil)))))
 
 (provide 'mew-encode)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2006 Mew developing team.
+;; Copyright (C) 1996-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1266,7 +1362,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
