@@ -14,11 +14,16 @@
 (defvar mew-imap-friend-folder-list nil)
 
 (defconst mew-imap-inbox-string "inbox")
-(defsubst mew-imap-inbox-str-p (mailbox)
+(defun mew-imap-inbox-str-p (mailbox)
   (mew-case-equal mailbox mew-imap-inbox-string))
 
 (defvar mew-imap-skip-uidl t)
 
+(defvar mew-imap-marks-to-clean
+  (list mew-mark-read mew-mark-unread mew-mark-escape mew-mark-review))
+(defvar mew-imap-marks-to-sync
+  (list mew-mark-read mew-mark-unread mew-mark-review))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; IMAP info
@@ -36,9 +41,9 @@
     "delete"
     "case" "mailbox" "msgid" "max" "tag"
     "wrk" "jobs" "namespace" "authl"
-    "spam" "spam-field" "spam-word" "spam-pattern" 
+    "spam" "spam-field" "spam-word" "spam-pattern"
     "new-mailbox" "my-prefix" "specials" "bytes"
-    "thread-info" "disp-info" "status-buf"))
+    "virtual-info" "disp-info" "status-buf" "mark-hist"))
 
 (mew-info-defun "mew-imap-" mew-imap-info-list)
 
@@ -56,12 +61,12 @@
     ("user-login"    ("OK" . "pwd-login") ("NO" . "wpwd"))
     ("pwd-login"     ("OK" . "next") ("NO" . "wpwd"))
     ("login"         ("OK" . "next") ("NO" . "wpwd"))
-    ("select"        ("OK" . "uid"))
-    ("select2"       ("OK" . "search"))
+    ("select"        ("OK" . "post-select"))
+    ("flags"         ("OK" . "flags")) ;; xxx NG but loop
     ("uid"           ("OK" . "umsg"))
     ("copy"          ("OK" . "copy") ("NO \\[TRYCREATE\\]" . "create") ("NO" . "wmbx"))
     ("create"        ("OK" . "copy") ("NO" . "wmbx"))
-    ("dels"	     ("OK" . "dels"))
+    ("dels"	     ("OK" . "dels")) ;; xxx NG but loop
     ("expunge"       ("OK" . "post-expunge"))
     ;;
     ("search"        ("OK" . "post-search"))
@@ -78,12 +83,20 @@
     ("logout2"       ("OK" . "noop"))
     ("logout3"       ("OK" . "noop"))))
 
-(defsubst mew-imap-fsm-by-status (status)
+(defun mew-imap-fsm-by-status (status)
   (assoc status mew-imap-fsm))
 
-(defsubst mew-imap-fsm-next (status code)
+(defun mew-imap-fsm-next (status code)
   (cdr (mew-assoc-match2 code (nthcdr 1 (mew-imap-fsm-by-status status)) 0)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Structure
+;;;
+
+;; 'get, 'exec and 'jobs
+(mew-defstruct copyinfo dst uid) ;; uid = "uid1,uid2"
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Filters
@@ -91,7 +104,7 @@
 
 (defun mew-imap-secure-p (pnm)
   (or (mew-imap-get-ssh-process pnm) (mew-imap-get-ssl-process pnm)))
- 
+
 (defun mew-imap-command-capability (pro pnm)
   (mew-net-status (mew-imap-get-status-buf pnm)
 		  "Auth'ing"
@@ -155,21 +168,22 @@
 	     (work (concat file mew-queue-work-suffix))
 	     (data (concat file mew-imapq-info-suffix))
 	     (data-work (concat data mew-queue-work-suffix))
-	     info mailbox)
+	     jobinfo execinfo mailbox)
 	(rename-file file work 'override)
-	(setq info (mew-lisp-load data))
+	(setq jobinfo (mew-lisp-load data))
 	(rename-file data data-work 'override)
-	(setq mailbox (car info))
+	(setq mailbox (mew-jobinfo-get-mailbox jobinfo))
 	(setq mailbox (mew-imap-utf-7-encode-string mailbox))
 	(setq mailbox (mew-imap-expand-mailbox case mailbox))
 	(mew-imap-set-mailbox pnm mailbox)
-	(setq info (cdr info))
-	(mew-imap-set-rmvs pnm (nth 0 info))
-	(mew-imap-set-kils pnm (nth 1 info))
-	(mew-imap-set-refs pnm (nth 2 info))
-	(mew-imap-set-movs pnm (nth 3 info))
-	(mew-imap-set-rttl pnm (nth 4 info))
-	(mew-imap-set-dttl pnm (nth 5 info))
+	(setq execinfo (mew-jobinfo-get-execinfo jobinfo))
+	(mew-imap-set-rmvs pnm (mew-execinfo-get-remove execinfo))
+	(mew-imap-set-kils pnm (mew-execinfo-get-kill   execinfo))
+	(mew-imap-set-refs pnm (mew-execinfo-get-refile execinfo))
+	(mew-imap-set-movs pnm (mew-execinfo-get-move   execinfo))
+	(mew-imap-set-rttl pnm (mew-execinfo-get-rttl   execinfo))
+	(mew-imap-set-dttl pnm (mew-execinfo-get-dttl   execinfo))
+	(mew-imap-set-mark-hist pnm (mew-imap-make-mark-hist (mew-jobinfo-get-mark-hist jobinfo)))
 	(mew-imap-set-rcnt pnm 1)
 	(mew-imap-set-rgcnt pnm 1)
 	(mew-imap-set-dgcnt pnm 1)
@@ -192,51 +206,101 @@
       (mew-imap-set-status pnm "rename")
       (mew-imap-command-rename pro pnm))
      (t
-      (if (mew-imap-get-spam pnm)
-	  (mew-imap-set-status pnm "select2"))
       (mew-imap-process-send-string pro pnm "SELECT \"%s\"" mailbox)))))
 
+(defun mew-imap-command-post-select (pro pnm)
+  (let ((mark-hist (mew-imap-get-mark-hist pnm)))
+    (cond
+     (mark-hist
+      (mew-imap-set-status pnm "flags") ;; come back
+      (mew-imap-command-flags pro pnm))
+     ((mew-imap-get-spam pnm)
+      (mew-imap-set-status pnm "search") ;; goto "uid"
+      (mew-imap-command-search pro pnm))
+     (t
+      (mew-imap-set-status pnm "uid")
+      (mew-imap-command-uid pro pnm)))))
+
+(defun mew-imap-command-flags (pro pnm)
+  (let ((mark-hist (mew-imap-get-mark-hist pnm))
+	ent msg mark flag)
+    (cond
+     (mark-hist
+      (setq ent (car mark-hist))
+      (setq msg  (mew-mark-hist-get-msg ent))
+      (setq mark (mew-mark-hist-get-mark ent))
+      (mew-imap-set-mark-hist pnm (cdr mark-hist))
+      (cond
+       ((char-equal mark mew-mark-review)
+	(setq flag "+FLAGS (\\Seen \\Flagged)"))
+       ((char-equal mark mew-mark-read)
+	(mew-imap-set-mark-hist
+	 pnm (cons (mew-make-mark-hist :msg msg :mark mew-mark-tmp) (cdr mark-hist)))
+	(setq flag "+FLAGS (\\Seen)"))
+       ((char-equal mark mew-mark-unread)
+	(setq flag "-FLAGS (\\Seen)"))
+       ((char-equal mark mew-mark-tmp)
+	(setq flag "-FLAGS (\\Flagged)")))
+      (mew-imap-process-send-string
+       pro pnm "UID STORE %s %s" msg flag))
+     (t
+      (mew-imap-set-status pnm "post-select")
+      (mew-imap-command-post-select pro pnm)))))
+
 (defun mew-imap-command-uid (pro pnm)
   (let ((directive (mew-imap-get-directive pnm))
-	(refs (mew-imap-get-refs pnm)) ;; (uid siz del (+fld msg))
+	(refs (mew-imap-get-refs pnm))
 	(rmvs (mew-imap-get-rmvs pnm))
 	(bnm (mew-imap-get-bnm pnm))
 	(range (mew-imap-get-range pnm))
 	max)
     (if (and mew-imap-skip-uidl
-	     (or (eq directive 'exec) (eq directive 'jobs) (eq directive 'get)))
+	     (or (eq directive 'exec) (eq directive 'jobs)))
 	(mew-imap-command-dispatch pro pnm directive refs rmvs nil)
       (mew-net-status (mew-imap-get-status-buf pnm)
 		      "Checking" nil (mew-imap-secure-p pnm))
-      (cond
-       ((eq directive 'biff)
-	(setq max (mew-lisp-load (mew-expand-file bnm mew-imap-msgid-file))))
-       ((eq directive 'scan)
-	(if (eq range nil) ;; update
-	    (setq max (mew-lisp-load
-		       (mew-expand-file bnm mew-imap-msgid-file))))))
-      (if max
-	  (setq max (string-to-number max))
-	(setq max 0))
-      (mew-imap-set-max pnm max)
-      (mew-imap-process-send-string
-       pro pnm "UID FETCH %d:* (UID RFC822.SIZE)" (1+ max)))))
+      (if (eq directive 'get)
+	  (mew-imap-process-send-string
+	   pro pnm "UID FETCH %s:%s (UID RFC822.SIZE FLAGS)"
+	   (mew-refileinfo-get-uid (car refs))
+	   (mew-refileinfo-get-uid (nth (1- (length refs)) refs)))
+	(cond
+	 ((eq directive 'biff)
+	  (setq max (mew-lisp-load (mew-expand-file bnm mew-imap-msgid-file))))
+	 ((eq directive 'scan)
+	  (if (eq range nil) ;; update
+	      (setq max (mew-lisp-load
+			 (mew-expand-file bnm mew-imap-msgid-file))))))
+	(if max
+	    (setq max (string-to-number max))
+	  (setq max 0))
+	(mew-imap-set-max pnm max)
+	(mew-imap-process-send-string
+	 pro pnm "UID FETCH %d:* (UID RFC822.SIZE FLAGS)" (1+ max))))))
 
 (defun mew-imap-command-umsg (pro pnm)
   (let* ((directive (mew-imap-get-directive pnm))
 	 (max (mew-imap-get-max pnm))
-	 (refs (mew-imap-get-refs pnm)) ;; (uid siz del (+fld msg))
+	 (refs (mew-imap-get-refs pnm))
 	 (rmvs (mew-imap-get-rmvs pnm))
 	 (del-time (mew-imap-get-delete pnm))
 	 (range (mew-imap-get-range pnm))
 	 (ctime (current-time))
-	 rtr rtrs dels uid siz uidl old-uidl uid-time hlds)
+	 rtr rtrs dels uid siz uidl old-uidl uid-time hlds flags mdb)
     (if (eq directive 'inc)
 	(setq old-uidl (mew-net-uidl-db-get (mew-imap-passtag pnm))))
     (goto-char (point-min))
-    (while (re-search-forward "^\\* [0-9]+ FETCH (UID \\([0-9]+\\) RFC822.SIZE \\([0-9]+\\))" nil t)
-      (setq uid (mew-match-string 1))
-      (setq siz (mew-match-string 2))
+    (while (re-search-forward "^\\* [0-9]+ FETCH (" nil t)
+      (when (looking-at ".*UID \\([0-9]+\\)")
+	(setq uid (mew-match-string 1)))
+      (when (looking-at ".*RFC822.SIZE \\([0-9]+\\)")
+	(setq siz (mew-match-string 1)))
+      (when (looking-at ".*FLAGS (\\([^)]*\\)") ;; MUST be a list of flags
+	(setq flags (mew-match-string 1))
+	(if (string-match "Flagged" flags) ;; \\\\ is urgly
+	    (setq mdb (cons (list uid mew-mark-review) mdb))
+	  (if (string-match "Seen" flags)
+	      (setq mdb (cons (list uid mew-mark-read) mdb)))))
       (cond
        ((or (eq directive 'exec) (eq directive 'jobs) (eq directive 'get))
 	(cond
@@ -246,11 +310,11 @@
 	  (setq dels (cons uid dels)))))
        ((eq directive 'biff)
 	(if (and uid (> (string-to-number uid) max))
-	    (setq rtrs (cons (list uid siz) rtrs))))
+	    (setq rtrs (cons (mew-make-refileinfo :uid uid :size siz) rtrs))))
        ((eq directive 'scan)
 	(if (and uid (or range ;; all, last:n
 			 (> (string-to-number uid) max))) ;; update
-	    (setq rtrs (cons (list uid siz) rtrs))))
+	    (setq rtrs (cons (mew-make-refileinfo :uid uid :size siz) rtrs))))
        ((eq directive 'sync)
 	(if uid (setq hlds (cons uid hlds))))
        ((eq directive 'inc)
@@ -264,26 +328,26 @@
 	  (setq uidl (cons (cons uid ctime) uidl))
 	  (if (eq del-time t)
 	      (setq dels (cons uid dels)))
-	  (setq rtr (list uid siz))
+	  (setq rtr (mew-make-refileinfo :uid uid :size siz))
 	  (setq rtrs (cons rtr rtrs)))))))
-    (mew-imap-set-msgid pnm (or (nth 0 (car rtrs)) max)) ;; 'scan
+    (mew-imap-set-msgid pnm (or (mew-refileinfo-get-uid (car rtrs)) max)) ;; 'scan
     (mew-imap-set-uidl pnm uidl) ;; 'inc
     ;; last:n
-    (when (and (eq directive 'scan) (numberp range))
-      (if (> (length rtrs) range)
-	  (setcdr (nthcdr (1- range) rtrs) nil)))
+    (when (and (eq directive 'scan) (integerp range))
+      (mew-ntake range rtrs))
     (setq rtrs (nreverse rtrs))
     (setq dels (nreverse dels))
     (setq hlds (nreverse hlds))
+    (mew-imap-set-mdb pnm (nreverse mdb))
     (mew-imap-command-dispatch pro pnm directive rtrs dels hlds)))
 
 (defun mew-imap-command-dispatch (pro pnm directive rtrs dels hlds)
   (cond
    ((or (eq directive 'exec) (eq directive 'jobs))
-    ;; '((uid1 siz1 t   dst1 dst2)
-    ;;   (uid2 siz2 t   dst1)
-    ;;   (uid3 siz3 nil dst1 dst2)
-    ;;   (uid4 siz4 nil dst3))
+    ;; '((uid1 siz1 t   (dst1 dst2))
+    ;;   (uid2 siz2 t   (dst1))
+    ;;   (uid3 siz3 nil (dst1 dst2))
+    ;;   (uid4 siz4 nil (dst3)))
     ;; =>
     ;; '((dst1 uid1 uid2)
     ;;   (dst2 uid1 uid2 uid3)
@@ -294,32 +358,28 @@
     ;;   (dst2 uid1,uid2)
     ;;   (dst2 uid3)
     ;;   (dst3 uid4))
-    (let (uid del fld flds ent ents dels2 fld-uids uids)
+    (let (uid del fld flds ents dels2 fld-uids uids)
       (setq ents rtrs)
       (setq rtrs nil)
-      (while ents
-	(setq ent  (car ents))
-	(setq ents (cdr ents))
-	(mew-set '(uid nil del) ent)
-	(setq flds (nthcdr 3 ent))
+      (dolist (ent ents)
+	(setq uid (mew-refileinfo-get-uid ent))
+	(setq del (mew-refileinfo-get-delete ent))
+	(setq flds (mew-refileinfo-get-folders ent))
 	(if del (setq dels2 (cons uid dels2)))
-	(while flds
-	  (setq fld  (car flds))
-	  (setq flds (cdr flds))
+	(dolist (fld flds)
 	  (if (setq fld-uids (assoc fld rtrs))
 	      (nconc fld-uids (list uid))
 	    (setq rtrs (cons (list fld uid) rtrs)))))
       ;;
       (setq ents rtrs)
       (setq rtrs nil)
-      (while ents
-	(setq ent  (car ents))
-	(setq ents (cdr ents))
+      (dolist (ent ents)
 	(setq fld  (car ent))
 	(setq uids (cdr ent))
 	(setq uids (mew-net-msg-group uids))
-	(setq uids (mapcar (lambda (arg) (list fld arg)) uids))
+	(setq uids (mapcar (lambda (arg) (mew-make-copyinfo :dst fld :uid arg)) uids))
 	(setq rtrs (nconc rtrs uids)))
+      ;;
       (mew-imap-set-rtrs pnm rtrs)
       (mew-imap-set-rgttl pnm (length rtrs))
       ;;
@@ -389,8 +449,8 @@
 	 (spam (mew-imap-get-spam pnm))
 	 (rtrs (mew-imap-get-rtrs pnm))
 	 (rtr (car rtrs))
-	 (dst (nth 0 rtr))
-	 (uid (nth 1 rtr)))
+	 (dst (mew-copyinfo-get-dst rtr))
+	 (uid (mew-copyinfo-get-uid rtr)))
     (if (null rtr)
 	(cond
 	 ((mew-imap-get-dels pnm)
@@ -412,17 +472,22 @@
   (let* ((case (mew-imap-get-case pnm))
 	 (rtrs (mew-imap-get-rtrs pnm))
 	 (rtr (mew-imap-get-rfl pnm))
-	 (dst (nth 0 rtr))) ;; 'exec or 'jobs
+	 (dst (mew-copyinfo-get-dst rtr))) ;; 'exec or 'jobs
     (mew-imap-set-rgcnt pnm (1- (mew-imap-get-rgcnt pnm)))
     (mew-imap-set-rtrs pnm (cons rtr rtrs))
     (setq dst (mew-imap-expand-mailbox
 	       case (mew-imap-utf-7-encode-string dst)))
     (mew-imap-process-send-string pro pnm "CREATE \"%s\"" dst)))
 
+(defun mew-imap-escape-format (str)
+  (if (mew-folder-imapp str) (concat "%" str) str))
+
 (defun mew-imap-command-wmbx (pro pnm)
-  (mew-imap-set-error pnm "Mailbox name is wrong!")
-  (mew-imap-set-status pnm "logout2")
-  (mew-imap-command-logout2 pro pnm))
+  (let* ((rtr (mew-imap-get-rfl pnm))
+	 (dst (mew-copyinfo-get-dst rtr)))
+    (mew-imap-set-error pnm (format "Cannot create '%s' or the messages do not exist" (mew-imap-escape-format dst)))
+    (mew-imap-set-status pnm "logout2")
+    (mew-imap-command-logout2 pro pnm)))
 
 (defun mew-imap-command-pre-dels (pro pnm)
   (let ((dtt (length (mew-imap-get-dels pnm)))
@@ -500,7 +565,7 @@
 	(setq rtrs (copy-sequence dels))
 	(mew-imap-set-rttl pnm (length rtrs))
 	(setq rtrs (mew-net-msg-group rtrs))
-	(setq rtrs (mapcar (lambda (arg) (list trash arg)) rtrs))
+	(setq rtrs (mapcar (lambda (arg) (mew-make-copyinfo :dst trash :uid arg)) rtrs))
 	(mew-imap-set-rtrs pnm rtrs)
 	(mew-imap-set-rgttl pnm (length rtrs)))
       (mew-imap-set-dttl pnm (length dels))
@@ -523,7 +588,6 @@
     (cond
      ((= rtt 0)
       (mew-imap-message pnm "No new messages")
-      (mew-imap-set-status pnm "logout")
       (mew-imap-command-logout pro pnm))
      ((= rtt 1)
       (mew-imap-message pnm "Retrieving 1 message in background...")
@@ -538,15 +602,15 @@
   (mew-net-status2 (mew-imap-get-status-buf pnm)
 		   (mew-imap-get-rttl pnm)
 		   (mew-imap-get-rcnt pnm)
-		   (nth 1 (car (mew-imap-get-rtrs pnm)))
+		   (mew-refileinfo-get-size (car (mew-imap-get-rtrs pnm)))
 		   'zero
 		   (mew-imap-secure-p pnm))
   (let* ((directive (mew-imap-get-directive pnm))
 	 (rtrs (mew-imap-get-rtrs pnm))
 	 (dels (mew-imap-get-dels pnm))
 	 (rtr (car rtrs))
-	 (uid (nth 0 rtr))
-	 (siz (nth 1 rtr))
+	 (uid (mew-refileinfo-get-uid rtr))
+	 (siz (mew-refileinfo-get-size rtr))
 	 (lim (mew-imap-get-size pnm))
 	 (get-body (mew-imap-get-get-body pnm)))
     (cond
@@ -561,14 +625,14 @@
      ((or (eq directive 'exec) (eq directive 'jobs) (eq directive 'get))
       (mew-imap-set-truncated pnm nil)
       ;; RFC822 turns the Seen flag "on"
-      (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822" uid))
+      ;; BODY.PEEK[] keeps the Seen flag "off"
+      (mew-imap-process-send-string pro pnm "UID FETCH %s BODY.PEEK[]" uid))
      ((and (eq directive 'scan) (not get-body))
       (mew-imap-set-truncated pnm t)
-      ;; RFC822 keeps the Seen flag "off"
       (mew-imap-process-send-string pro pnm "UID FETCH %s BODY.PEEK[HEADER]" uid))
      ((or (= lim 0) (< (string-to-number siz) lim))
       (mew-imap-set-truncated pnm nil)
-      (mew-imap-process-send-string pro pnm "UID FETCH %s RFC822" uid))
+      (mew-imap-process-send-string pro pnm "UID FETCH %s BODY.PEEK[]" uid))
      (t
       (mew-imap-set-truncated pnm t)
       (mew-imap-process-send-string
@@ -579,28 +643,28 @@
 	 (width (1- (mew-scan-width)))
 	 (rtrs (mew-imap-get-rtrs pnm))
 	 (rtr (car rtrs))
-	 (uid (nth 0 rtr))
-	 (siz (nth 1 rtr))
-	 (del (nth 2 rtr))
-	 (fld-msg (nth 3 rtr)) ;; 'exec, 'jobs, 'scan, ...
+	 (uid (mew-refileinfo-get-uid rtr))
+	 (siz (mew-refileinfo-get-size rtr))
+	 (del (mew-refileinfo-get-delete rtr))
+	 (union (mew-refileinfo-get-folders rtr)) ;; 'exec, 'jobs, 'scan, ...
 	 (truncated (mew-imap-get-truncated pnm))
 	 (case (mew-imap-get-case pnm))
 	 fld msg vec file msg-file lmsg mark folder)
     (cond
-     ((null fld-msg)
-      (setq fld (mew-imap-get-bnm pnm)))
-     ((stringp fld-msg)
-      (setq fld fld-msg))
-     ((listp fld-msg)
-      (mew-set '(fld msg) fld-msg)
-      (setq lmsg msg)))
+     ((or (eq directive 'exec) (eq directive 'jobs))
+      (setq fld (car union)))
+     ((eq directive 'get)
+      (mew-set '(fld msg) union)
+      (setq lmsg msg))
+     (t ;; 'biff, 'scan, 'sync
+      (setq fld (mew-imap-get-bnm pnm))))
     (setq folder (mew-case:folder-folder fld))
     ;; line delimiters
     (goto-char (point-min))
     (mew-crlf-to-lf)
     (cond
      ((eq directive 'scan)
-      (setq msg uid) 
+      (setq msg uid)
       (setq file (mew-expand-new-msg fld msg)))
      (t
       (setq msg-file (mew-net-get-new-message
@@ -618,9 +682,9 @@
       (mew-header-insert-xmu uid siz nil)))
     (catch 'write-error
       (condition-case nil
-	  (mew-frwlet 
-	   mew-cs-dummy mew-cs-text-for-write
-	   (write-region (point-min) (point-max) file nil 'no-msg))
+	  (let ((write-region-inhibit-fsync mew-use-async-write))
+	    (mew-frwlet mew-cs-dummy mew-cs-text-for-write
+	      (write-region (point-min) (point-max) file nil 'no-msg)))
 	(error
 	 (mew-imap-set-status pnm "logout")
 	 (mew-imap-command-logout pro pnm)
@@ -693,9 +757,7 @@
 	  (setq my-prefix (nth 0 ent))
 	  (setq sep (nth 1 ent)))
 	 (t
-	  (while sexp
-	    (setq ent (car sexp))
-	    (setq sexp (cdr sexp))
+	  (dolist (ent sexp)
 	    (mew-set '(my sp) ent)
 	    (when (member my prefix-list)
 	      (cond
@@ -874,7 +936,7 @@
   '(("CRAM-MD5" mew-imap-command-auth-cram-md5)
     ("LOGIN"    mew-imap-command-auth-login)))
 
-(defsubst mew-imap-auth-get-func (auth)
+(defun mew-imap-auth-get-func (auth)
   (nth 1 (mew-assoc-case-equal auth mew-imap-auth-alist 0)))
 
 (defun mew-imap-command-auth-cram-md5 (pro pnm)
@@ -917,19 +979,19 @@
 
 (defconst mew-imap-info-prefix "mew-imap-info-")
 
-(defsubst mew-imap-info-name (case mailbox)
+(defun mew-imap-info-name (case mailbox)
   (let ((server (mew-imap-server case))
-	(port (mew-imap-port case))
+	(port (mew-*-to-string (mew-imap-port case)))
 	(sshsrv (mew-imap-ssh-server case))
 	(name mew-imap-info-prefix))
     (setq name (concat name server "/" mailbox))
-    (unless (string= port mew-imap-port)
+    (unless (mew-port-equal port mew-imap-port)
       (setq name (concat name ":" port)))
     (if sshsrv
-	(concat name "%" sshsrv)
+        (concat name "%" sshsrv)
       name)))
 
-(defsubst mew-imap-buffer-name (pnm)
+(defun mew-imap-buffer-name (pnm)
   (concat mew-buffer-prefix pnm))
 
 (defun mew-imap-process-send-string (pro pnm &rest args)
@@ -947,7 +1009,7 @@
 	(process-send-string pro (concat str mew-cs-eol))
       (message "IMAP time out"))))
 
-(defsubst mew-imap-message (pnm &rest args)
+(defun mew-imap-message (pnm &rest args)
   (or (mew-imap-get-no-msg pnm) (apply 'message args)))
 
 (defun mew-imap-tag ()
@@ -993,21 +1055,21 @@
 	 (namespace (assoc mew-imap-inbox-folder alist)))
     (if (= (length namespace) mew-imap-namespace-length)
 	namespace
-      (error (mew-substitute-for-summary "Type '3\\[mew-status-update]' to collect IMAP folders!")))))
+      (error (mew-substitute-for-summary "Type '\\[universal-argument]\\[mew-status-update]' to collect IMAP folders!")))))
 
-(defsubst mew-imap-namespace-sep (namespace)
+(defun mew-imap-namespace-sep (namespace)
   (nth 1 namespace))
 
-(defsubst mew-imap-namespace-change-sep (namespace sep)
+(defun mew-imap-namespace-change-sep (namespace sep)
   (setcar (nthcdr 1 namespace) sep))
 
-(defsubst mew-imap-namespace-my-prefix (namespace)
+(defun mew-imap-namespace-my-prefix (namespace)
   (nth 2 namespace))
 
-(defsubst mew-imap-namespace-user-prefix (namespace)
+(defun mew-imap-namespace-user-prefix (namespace)
   (nth 3 namespace))
 
-(defsubst mew-imap-namespace-sharp (namespace)
+(defun mew-imap-namespace-sharp (namespace)
   (nth 4 namespace))
 
 (defun mew-imap-separator (case)
@@ -1021,7 +1083,7 @@
     (if (char-equal sep-char (aref dir len))
 	(substring dir 0 len)
       dir)))
-	   
+	
 (defun mew-imap-file-name-as-directory (dir case)
   ;; inbox.foo => inbox.foo.
   (let* ((sep (mew-imap-separator case))
@@ -1033,7 +1095,7 @@
 
 ;;;
 
-(defsubst mew-imap-bnm-to-mailbox (bnm)
+(defun mew-imap-bnm-to-mailbox (bnm)
   "
    case:%inbox -> inbox
    case:%foo   -> inbox.foo
@@ -1082,19 +1144,19 @@
 
 ;;		Mew		cache		IMAP
 ;;		folder		directory	mailbox
-;; 
+;;
 ;; WU
 ;;		%foo		foo		foo
 ;;		%foo/bar	foo/bar		foo/bar
 ;;		%~alice/foo	#~alice/foo	~alice/foo   "~" -> "~"
 ;;		%#shared.foo	#shared.foo	#shared.foo
-;; 
+;;
 ;; Cyrus
 ;;		%foo		foo		inbox.foo
 ;;		%foo.bar	foo.bar		inbox.foo.bar
 ;;		%~alice.foo	#user.alice.foo	user.alice.foo
 ;;		%#shared.foo	#shared.foo	shared.foo
-;; 
+;;
 ;; Courier
 ;;		%foo		foo		inbox.foo
 ;;		%foo.bar	foo.bar		inbox.foo.bar
@@ -1128,14 +1190,14 @@
 ;;;
 
 (defun mew-imap-open (pnm server port no-msg)
-  (let ((sprt (mew-port-sanity-check port))
+  (let ((sprt (mew-*-to-port port))
 	pro tm)
     (condition-case emsg
 	(progn
 	  (setq tm (run-at-time mew-imap-timeout-time nil 'mew-imap-timeout))
 	  (or no-msg (message "Connecting to the IMAP server..."))
 	  (setq pro (open-network-stream pnm nil server sprt))
-	  (process-kill-without-query pro)
+	  (mew-process-silent-exit pro)
 	  (mew-set-process-cs pro mew-cs-binary mew-cs-text-for-net)
 	  (or no-msg (message "Connecting to the IMAP server...done")))
       (quit
@@ -1155,10 +1217,25 @@
 ;;; Launcher
 ;;;
 
+(defun mew-imap-make-mark-hist (ents)
+  (let (ret)
+    (dolist (ent ents)
+      (let ((mark (mew-mark-hist-get-mark ent))
+	    (msg (mew-mark-hist-get-msg ent)))
+	(cond
+	 ((memq mark mew-imap-marks-to-sync)
+	  (setq ret (cons ent ret)))
+	 ;; xxx the refile mark overrides the read mark.
+	 ;; If we refile a message, we cannot distinguish
+	 ;; read or unread...
+	 ((char-equal mark mew-mark-refile)
+	  (setq ret (cons (mew-make-mark-hist :msg msg :mark mew-mark-read) ret))))))
+    (nreverse ret)))
+
 (defun mew-imap-retrieve (case directive bnm &rest args)
   (let* ((server (mew-imap-server case))
          (user (mew-imap-user case))
-	 (port (mew-imap-port case))
+	 (port (mew-*-to-string (mew-imap-port case)))
 	 (sshsrv (mew-imap-ssh-server case))
 	 (sslp (mew-imap-ssl case))
 	 (sslport (mew-imap-ssl-port case))
@@ -1172,7 +1249,7 @@
 	 (buf (get-buffer-create (mew-imap-buffer-name pnm)))
 	 (no-msg (eq directive 'biff))
 	 process sshname sshpro sslname sslpro lport info jobs tls
-	 thread-info disp-info virtual)
+	 virtual-info disp-info virtual)
     (if (mew-imap-get-process pnm)
 	(message "Another IMAP process is running. Try later")
       (cond
@@ -1184,7 +1261,7 @@
 	  (when lport
 	    (setq process (mew-imap-open pnm "localhost" lport no-msg)))))
        (sslp
-	(if (string= port sslport) (setq tls mew-tls-imap))
+	(if (mew-port-equal port sslport) (setq tls mew-tls-imap))
  	(setq sslpro (mew-open-ssl-stream case server sslport tls))
  	(when sslpro
  	  (setq sslname (process-name sslpro))
@@ -1200,6 +1277,7 @@
 	    (mew-imap-exec-recover bnm))
 	(mew-summary-lock process "IMAPing" (or sshpro sslpro))
 	(mew-sinfo-set-summary-form (mew-get-summary-form bnm))
+	(mew-sinfo-set-summary-column (mew-get-summary-column bnm))
 	(mew-sinfo-set-unread-mark nil)
 	(mew-sinfo-set-scan-id nil)
 	(mew-sinfo-set-scan-md5 nil)
@@ -1229,6 +1307,7 @@
 	(mew-imap-set-mailbox pnm mailbox)
 	(mew-imap-set-case pnm case)
 	(mew-imap-set-bytes pnm nil)
+	(mew-imap-set-mark-hist pnm (mew-imap-make-mark-hist (mew-sinfo-get-mark-hist)))
 	(cond
 	 ((eq directive 'biff)
 	  (mew-imap-set-passwd pnm (nth 0 args))) ;; password
@@ -1239,25 +1318,24 @@
 	 ((eq directive 'get)
 	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (mew-imap-set-refs pnm (nth 0 args))
-	  (setq thread-info (nth 1 args))
-	  (mew-imap-set-thread-info pnm thread-info)
+	  (setq virtual-info (nth 1 args))
+	  (mew-imap-set-virtual-info pnm virtual-info)
 	  (setq disp-info (nth 2 args))
 	  (mew-imap-set-disp-info pnm disp-info)
-	  (setq virtual (mew-net-thread-info-get-virtual thread-info))
+	  (setq virtual (mew-net-virtual-info-get-virtual virtual-info))
 	  (when virtual
 	    (mew-imap-set-status-buf pnm virtual)
-	    (save-excursion
-	      (set-buffer virtual)
+	    (with-current-buffer virtual
 	      (mew-summary-lock process "IMAPing" (or sshpro sslpro)))))
 	 ((eq directive 'scan)
 	  (mew-imap-set-range pnm (nth 0 args))
 	  (mew-imap-set-get-body pnm (nth 1 args))
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
 	  (if (mew-imap-get-range pnm)
 	      (progn
-		(mew-imap-set-mdb pnm (mew-summary-mark-collect4))
+;;		(mew-imap-set-mdb pnm (mew-summary-mark-collect4))
 		(mew-net-folder-clean))
-	    (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm))
-	    (mew-imap-set-mdb pnm (mew-summary-mark-collect5))
+;;	    (mew-imap-set-mdb pnm (mew-summary-mark-collect5))
 	    (mew-net-invalid-cache-invisible))
 	  (when (and (string= mailbox mew-imap-inbox-string)
 		     (or (mew-imap-spam-pattern case)
@@ -1268,15 +1346,15 @@
 	    (mew-imap-set-spam-word pnm (mew-imap-spam-word case))
 	    (mew-imap-set-spam-pattern pnm (mew-imap-spam-pattern case))))
 	 ((eq directive 'sync)
-	  )
+	  (mew-sinfo-set-unread-mark (mew-get-unread-mark bnm)))
 	 ((eq directive 'exec)
 	  (setq info (nth 0 args))
-	  (mew-imap-set-rmvs pnm (nth 0 info)) ;; (uid siz del dst1 dst2 ...)
-	  (mew-imap-set-kils pnm (nth 1 info))
-	  (mew-imap-set-refs pnm (nth 2 info))
-	  (mew-imap-set-movs pnm (nth 3 info))
-	  (mew-imap-set-rttl pnm (nth 4 info))
-	  (mew-imap-set-dttl pnm (nth 5 info)))
+	  (mew-imap-set-rmvs pnm (mew-execinfo-get-remove info))
+	  (mew-imap-set-kils pnm (mew-execinfo-get-kill   info))
+	  (mew-imap-set-refs pnm (mew-execinfo-get-refile info))
+	  (mew-imap-set-movs pnm (mew-execinfo-get-move   info))
+	  (mew-imap-set-rttl pnm (mew-execinfo-get-rttl   info))
+	  (mew-imap-set-dttl pnm (mew-execinfo-get-dttl   info)))
 	 ((eq directive 'jobs)
 	  (mew-imap-set-jcnt pnm 0)
 	  (setq jobs (nth 0 args))
@@ -1305,8 +1383,7 @@
 
 (defun mew-imap-debug (label string)
   (when (mew-debug 'net)
-    (save-excursion
-      (set-buffer (get-buffer-create mew-buffer-debug))
+    (with-current-buffer (get-buffer-create mew-buffer-debug)
       (goto-char (point-max))
       (let ((start (point)))
 	(insert (format "\n<%s>\n%s\n" label string))
@@ -1332,7 +1409,7 @@
        (mew-net-status2 (mew-imap-get-status-buf pnm)
 			(mew-imap-get-rttl pnm)
 			(mew-imap-get-rcnt pnm)
-			(nth 1 (car (mew-imap-get-rtrs pnm)))
+			(mew-refileinfo-get-size (car (mew-imap-get-rtrs pnm)))
 			nil
 			(mew-imap-secure-p pnm))
        (goto-char (point-min))
@@ -1383,6 +1460,31 @@
        (if (and process (equal (process-buffer process) (current-buffer)))
 	   (mew-erase-buffer))))))
 
+(defun mew-imap-mark-clean (mark)
+  (let* ((regex (mew-mark-list-regex mew-imap-marks-to-clean)))
+    (save-excursion
+      (goto-char (point-min))
+      (while (re-search-forward regex nil t)
+	(mew-mark-put mark)
+	(forward-line)))))
+
+;; see also mew-summary-mark-recover
+(defun mew-imap-mark-recover (mdb)
+  (let ((opos (point))
+	(case-fold-search nil)
+	msg mrk omrk)
+    (goto-char (point-min))
+    (dolist (ent mdb)
+      (mew-set '(msg mrk) ent)
+      (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	(setq omrk (mew-summary-get-mark))
+	(unless (and omrk (or (char-equal omrk mew-mark-refile)
+			      (char-equal omrk mew-mark-delete)
+			      (char-equal omrk mew-mark-unlink)))
+	  (mew-mark-put mrk)) ;; hist will be reset
+	(forward-line)))
+    (goto-char opos)))
+
 (defun mew-imap-sentinel (process event)
   (let* ((pnm (process-name process))
 	 (directive (mew-imap-get-directive pnm))
@@ -1404,136 +1506,146 @@
 	 (error (mew-imap-get-error pnm))
 	 (file (mew-expand-file bnm mew-imap-msgid-file))
 	 (buf (process-buffer process))
-	 (thread-info (mew-imap-get-thread-info pnm))
+	 (virtual-info (mew-imap-get-virtual-info pnm))
 	 (disp-info (mew-imap-get-disp-info pnm)))
-    (mew-imap-debug "IMAP SENTINEL" event)
-    (set-process-buffer process nil)
-    (set-buffer bnm)
-    (mew-summary-mark-recover mdb)
-    (mew-remove-buffer buf)
-    (if (not done)
-	(let* ((rtrs (mew-imap-get-rtrs pnm))
-	       (lefts (length rtrs))
-	       (msgid (nth 0 (car rtrs)))
-	       leftp)
-	  (if error
-	      (mew-imap-message pnm error)
-	    (mew-imap-message pnm "IMAP connection is lost"))
-	  (cond
-	   ((eq directive 'exec)
-	    (mew-imap-exec-recover bnm))
-	   ((eq directive 'scan)
-	    (mew-summary-visible-buffer bnm))
-	   ((or (eq directive 'delete) (eq directive 'rename))
-	    (setq mew-summary-buffer-process-error t)))
-	  (when (mew-imap-get-dispatched pnm)
+    (save-excursion
+      (mew-imap-debug "IMAP SENTINEL" event)
+      (set-process-buffer process nil)
+      (set-buffer bnm)
+      (when (eq directive 'sync)
+	(if (mew-sinfo-get-unread-mark)
+	    (mew-imap-mark-clean mew-mark-unread)
+	  (mew-imap-mark-clean mew-mark-read)))
+      (mew-imap-mark-recover mdb)
+      (mew-sinfo-set-mark-hist nil) ;; xxx
+      (mew-remove-buffer buf)
+      (if (not done)
+	  (let* ((rtrs (mew-imap-get-rtrs pnm))
+		 (lefts (length rtrs))
+		 (msgid (mew-refileinfo-get-uid (car rtrs)))
+		 leftp)
+	    (if error
+		(mew-imap-message pnm error)
+	      (mew-imap-message pnm "IMAP connection is lost"))
 	    (cond
+	     ((eq directive 'exec)
+	      (mew-imap-exec-recover bnm))
 	     ((eq directive 'scan)
-	      (when msgid
-		(setq msgid (number-to-string (1- (string-to-number msgid))))
-		(mew-lisp-save file msgid nil 'unlimit))
-	      (setq leftp t))
-	     ((eq directive 'inc)
-	      ;; uidl is reversed.
-	      (setq msgid (assoc msgid uidl))
-	      (setq uidl (cdr (member msgid uidl)))
-	      (mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
-	      (setq leftp t)))
-	    (when leftp
-	      (mew-imap-message
-	       pnm
-	       "%d message retrieved. %d messages are left due to an error"
-	       (- rttl lefts) lefts)
-	      (mew-summary-folder-cache-save))))
-      (if thread-info (mew-summary-retrieve-message-for-thread thread-info))
-      (cond
-       ((eq directive 'list)
-	(mew-imap-message pnm "Collecting mailbox list...done"))
-       ((eq directive 'delete)
-	(mew-imap-message pnm "Deleting this folder...done"))
-       ((eq directive 'rename)
-	(mew-imap-message pnm "Renaming this folder...done"))
-       ((eq directive 'biff)
-	(funcall mew-biff-function rttl))
-       ((eq directive 'sync)
-	(mew-imap-message pnm "Synchronizing messages...")
-	(mew-net-folder-sync bnm hlds)
-	(mew-imap-message pnm "Synchronizing messages...done")
-	(mew-summary-folder-cache-save))
-       ((eq directive 'inc)
-	(mew-biff-clear)
-	(mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
+	      (mew-summary-visible-buffer bnm))
+	     ((or (eq directive 'delete) (eq directive 'rename))
+	      (setq mew-summary-buffer-process-error t)))
+	    (when (mew-imap-get-dispatched pnm)
+	      (cond
+	       ((eq directive 'scan)
+		(when msgid
+		  (setq msgid (number-to-string (1- (string-to-number msgid))))
+		  (mew-lisp-save file msgid nil 'unlimit))
+		(setq leftp t))
+	       ((eq directive 'inc)
+		;; uidl is reversed.
+		(setq msgid (assoc msgid uidl))
+		(setq uidl (cdr (member msgid uidl)))
+		(mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
+		(setq leftp t)))
+	      (when leftp
+		(mew-imap-message
+		 pnm
+		 "%d message retrieved. %d messages are left due to an error"
+		 (- rttl lefts) lefts)
+		(mew-summary-folder-cache-save))))
+	(if virtual-info (mew-summary-retrieve-message-for-virtual virtual-info))
 	(cond
-	 ((= rttl 0)
-	  (mew-imap-message pnm "No new messages"))
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message retrieved")
+	 ((eq directive 'list)
+	  (mew-imap-message pnm "Collecting mailbox list...done"))
+	 ((eq directive 'delete)
+	  (mew-imap-message pnm "Deleting this folder...done"))
+	 ((eq directive 'rename)
+	  (mew-imap-message pnm "Renaming this folder...done"))
+	 ((eq directive 'biff)
+	  (and (functionp mew-biff-function)
+	       (funcall mew-biff-function rttl)))
+	 ((eq directive 'sync)
+	  (mew-imap-message pnm "Synchronizing messages...")
+	  (mew-net-folder-sync bnm hlds)
+	  (mew-imap-message pnm "Synchronizing messages...done")
 	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-imap-message pnm "%s messages retrieved" rttl)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'get)
-	(cond
-	 ((= rttl 0)
-	  (mew-imap-message pnm "No new messages"))
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message retrieved")
-	  (mew-summary-folder-cache-save))
-	 (t
-	  (mew-imap-message pnm "%s messages retrieved" rttl)
-	  (mew-summary-folder-cache-save))))
-       ((eq directive 'scan)
-	(mew-biff-clear)
-	(cond
-	 ((= rttl 0)
-	  (mew-imap-message pnm "No new messages"))
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message retrieved")
-	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-net-invalid-cache-clean))
-	 (t
-	  (mew-imap-message pnm "%s messages retrieved" rttl)
-	  (mew-lisp-save file msgid nil 'unlimit)
-	  (mew-net-invalid-cache-clean))))
-       ((eq directive 'exec)
-	(if kils (mew-mark-exec-unlink bnm kils))
-	(when movs
-	  (let ((mew-inherit-exec-case case)
-		(mew-inherit-offline t))
-	    (mew-mark-exec-refile bnm movs)))
-	(when (or kils movs)
-	  (mew-mark-kill-invisible)
-	  (mew-summary-folder-cache-save))
-	(cond
-	 ((= rttl 1)
-	  (mew-imap-message pnm "1 message refiled"))
-	 ((> rttl 1)
-	  (mew-imap-message pnm "%d messages refiled" rttl))
-	 ((= dttl -1)
-	  (mew-imap-message pnm "All messages deleted"))
-	 ((= dttl 1)
-	  (mew-imap-message pnm "1 message deleted"))
-	 (t
-	  (mew-imap-message pnm "%d messages deleted" dttl))))
-       ((eq directive 'jobs)
-	(if (= jcnt 1)
-	    (mew-imap-message pnm "1 job processed")
-	  (mew-imap-message pnm "%d jobs processed" jcnt)))))
-    ;;
-    (mew-net-status-clear (mew-imap-get-status-buf pnm))
-    (mew-info-clean-up pnm)
-    (set-buffer-modified-p nil)
-    (mew-summary-unlock)
-    (if (and (processp sshpro) (not mew-ssh-keep-connection))
-	(process-send-string sshpro "exit\n"))
-    (if (and (processp sslpro) (not mew-ssl-keep-connection))
- 	 (delete-process sslpro))
-    (mew-net-disp-info-display disp-info)
-    (unless (eq directive 'biff)
-      (run-hooks 'mew-imap-sentinel-non-biff-hook))
-    (run-hooks 'mew-imap-sentinel-hook)
-    (when (and mew-auto-flush-queue flush)
-      (mew-smtp-flush-queue mew-case))))
+	 ((eq directive 'inc)
+	  (mew-biff-clear)
+	  (mew-net-uidl-db-set (mew-imap-passtag pnm) uidl)
+	  (cond
+	   ((= rttl 0)
+	    (mew-imap-message pnm "No new messages"))
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message retrieved")
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-imap-message pnm "%s messages retrieved" rttl)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'get)
+	  (cond
+	   ((= rttl 0)
+	    (mew-imap-message pnm "The message does not exist"))
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message retrieved")
+	    (mew-summary-folder-cache-save))
+	   (t
+	    (mew-imap-message pnm "%s messages retrieved" rttl)
+	    (mew-summary-folder-cache-save))))
+	 ((eq directive 'scan)
+	  (mew-biff-clear)
+	  (cond
+	   ((= rttl 0)
+	    (mew-imap-message pnm "No new messages"))
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message retrieved")
+	    (mew-lisp-save file msgid nil 'unlimit)
+	    ;; xxx
+	    (mew-net-invalid-cache-clean))
+	   (t
+	    (mew-imap-message pnm "%s messages retrieved" rttl)
+	    (mew-lisp-save file msgid nil 'unlimit)
+	    ;; xxx
+	    (mew-net-invalid-cache-clean))))
+	 ((eq directive 'exec)
+	  (if kils (mew-mark-exec-unlink bnm kils))
+	  (when movs
+	    (let ((mew-inherit-exec-case case)
+		  (mew-inherit-offline t))
+	      (mew-mark-exec-refile bnm movs)))
+	  (when (or kils movs)
+	    (mew-mark-kill-invisible)
+	    (mew-summary-folder-cache-save))
+	  (cond
+	   ((= rttl 1)
+	    (mew-imap-message pnm "1 message refiled"))
+	   ((> rttl 1)
+	    (mew-imap-message pnm "%d messages refiled" rttl))
+	   ((= dttl -1)
+	    (mew-imap-message pnm "All messages deleted"))
+	   ((= dttl 1)
+	    (mew-imap-message pnm "1 message deleted"))
+	   (t
+	    (mew-imap-message pnm "%d messages deleted" dttl))))
+	 ((eq directive 'jobs)
+	  (if (= jcnt 1)
+	      (mew-imap-message pnm "1 job processed")
+	    (mew-imap-message pnm "%d jobs processed" jcnt)))))
+      ;;
+      (and mew-use-async-write (mew-unix-sync))
+      (mew-net-status-clear (mew-imap-get-status-buf pnm))
+      (mew-info-clean-up pnm)
+      (set-buffer-modified-p nil)
+      (mew-summary-unlock)
+      (if (and (processp sshpro) (not mew-ssh-keep-connection))
+	  (process-send-string sshpro "exit\n"))
+      (if (and (processp sslpro) (not mew-ssl-keep-connection))
+	  (delete-process sslpro))
+      (mew-net-disp-info-display disp-info)
+      (unless (eq directive 'biff)
+	(run-hooks 'mew-imap-sentinel-non-biff-hook))
+      (run-hooks 'mew-imap-sentinel-hook)
+      (when (and mew-auto-flush-queue flush)
+	(mew-smtp-flush-queue mew-case)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -1547,8 +1659,7 @@
 	 (tag (mew-imap-passtag2 case))
 	 passwd)
     (when (get-buffer case:inbox)
-      (save-excursion
-	(set-buffer case:inbox)
+      (with-current-buffer case:inbox
 	(when (and (mew-summary-exclusive-p 'no-msg)
 		   (and (or mew-use-cached-passwd mew-use-master-passwd)
 			(setq passwd (mew-passwd-get-passwd tag))))
@@ -1561,8 +1672,7 @@
 	 (inbox (mew-proto-inbox-folder nil case))
 	 (case:inbox (mew-case-folder case inbox)))
     (when (get-buffer case:inbox)
-      (save-excursion
-	(set-buffer case:inbox)
+      (with-current-buffer case:inbox
 	(when (mew-summary-exclusive-p)
 	  (mew-imap-retrieve case 'biff case:inbox))))))
 
@@ -1637,12 +1747,8 @@
 	  (ent (assoc case mew-imap-folder-alist)))
       (setcar (nthcdr 1 ent) t1))))
 
-(defun mew-imap-update (&optional case)
+(defun mew-imap-update (case)
   (let ((bnm (mew-summary-folder-name 'ext)))
-    (unless case 
-      (setq case (mew-sinfo-get-case))
-      (if mew-config-cases
-	  (setq case (mew-input-case case "IMAP"))))
     (mew-imap-retrieve case 'list bnm)))
 
 (defun mew-imap-folder-insert (case folder)
@@ -1743,13 +1849,13 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2006 Mew developing team.
+;; Copyright (C) 2002-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1758,7 +1864,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
