@@ -22,26 +22,13 @@
 ;;;
 
 (defun mew-message-mode ()
-  "\\<mew-message-mode-map>
-Mew Message mode:: major mode for display a message.
-The keys that are defined for this mode are:
+  "A  major mode for display a message.
 
-\\[mew-message-next-page]	Scroll up this message.
-\\[mew-message-prev-page]	Back-scroll this message.
-\\[mew-message-next-msg]	Display a message or a part below.
-\\[mew-message-prev-msg]	Display a message or a part above.
-\\[mew-message-goto-summary]	Get back to Summary mode.
-\\[mew-message-reply]	Answer to this message. A new draft is prepared in Draft mode. 
-	Mew automatically decides To: and Cc:.
-\\[mew-message-reply-with-citation]	Answer to this message. A new draft is prepared in Draft mode. 
-	Mew automatically decides To: and Cc: and cites the body.
-\\[mew-message-forward]	Forward this message to a third person. A new draft is prepared in 
-	Draft mode and this message is automatically attached.
-"
+\\{mew-message-mode-map}"
   (interactive)
   (setq major-mode 'mew-message-mode)
   (setq mode-name mew-mode-name-message)
-  (setq mode-line-buffer-identification mew-mode-line-id)
+  (setq mode-line-buffer-identification (mew-mode-line-id))
   (use-local-map mew-message-mode-map)
   (setq buffer-read-only t)
   (make-local-variable 'page-delimiter)
@@ -50,22 +37,24 @@ The keys that are defined for this mode are:
   (mew-message-toolbar-update)
   (run-hooks 'mew-message-mode-hook))
 
-(defun mew-message-mode-line (fld msg)
-  (unless msg
-    (setq msg (mew-current-get-msg (mew-frame-id))))
-  (when msg
-    (setq mode-line-process (format " %s/%s" fld msg))))
+(defun mew-message-mode-line (fld message)
+  (let* ((msg (or message 
+		  (mew-current-get-msg (mew-frame-id))))
+	 (status (mew-message-line-status))
+	 (flag (nth status '("N" "W" "L"))))
+    (when msg
+      (setq mode-line-process (format " %s %s/%s" flag fld msg)))))
 
 (defun mew-message-next-page (&optional lines)
   "Scroll up this message. Return 'nil' if more pages. Otherwise, return 't'."
   (interactive)
-  (move-to-window-line -1)
   (if (save-excursion
-        (end-of-line)
+	(goto-char (window-end))
         (and (pos-visible-in-window-p) (eobp)))
       ;; Nothing in this page.
       (if (or (null mew-break-pages)
 	      (save-excursion
+		(goto-char (window-end))
 		(save-restriction
 		  (widen) (forward-line) (eobp)))) ;; Real end of buffer?
           (if mew-summary-show-pause
@@ -87,7 +76,7 @@ The keys that are defined for this mode are:
     nil))
 
 (defun mew-message-prev-page (&optional lines)
-  "Back-scroll this message. Return 'nil' if more pages. 
+  "Back-scroll this message. Return 'nil' if more pages.
 Otherwise, return 't'."
   (interactive)
   (move-to-window-line 0)
@@ -152,21 +141,21 @@ Otherwise, return 't'."
 	(mew-summary-move-and-display msg))))))
 
 (defun mew-message-reply ()
-  "Answer to this message. A new draft is prepared in Draft mode. 
+  "Answer to this message. A new draft is prepared in Draft mode.
 Mew automatically decides To: and Cc:."
   (interactive)
   (mew-message-goto-summary)
   (call-interactively 'mew-summary-reply))
 
 (defun mew-message-reply-with-citation ()
-  "Answer to this message. A new draft is prepared in Draft mode. 
+  "Answer to this message. A new draft is prepared in Draft mode.
 Mew automatically decides To: and Cc: and cites the body."
   (interactive)
   (mew-message-goto-summary)
   (call-interactively 'mew-summary-reply-with-citation))
 
 (defun mew-message-forward ()
-  "Forward this message to a third person. A new draft is prepared in 
+  "Forward this message to a third person. A new draft is prepared in
 Draft mode and this message is automatically attached."
   (interactive)
   (mew-message-goto-summary)
@@ -174,13 +163,13 @@ Draft mode and this message is automatically attached."
 
 (defun mew-message-resend ()
   "\\<mew-message-mode-map>
-Resend this message with Resent-To:. It is strongly 
-discouraged to use this command since beginners are always 
+Resend this message with Resent-To:. It is strongly
+discouraged to use this command since beginners are always
 confused. Please use '\\[mew-message-forward]' instead."
   (interactive)
   (mew-message-goto-summary)
   (call-interactively 'mew-summary-resend))
- 
+
 (defun mew-message-next-msg (&optional arg)
   "Display a message or a part below."
   (interactive "p")
@@ -192,7 +181,7 @@ confused. Please use '\\[mew-message-forward]' instead."
       (mew-summary-display))
     ;; for window config
     (select-window (get-buffer-window mbuf))))
-      
+
 (defun mew-message-prev-msg (&optional arg)
   "Display a message or a part above."
   (interactive "p")
@@ -236,17 +225,96 @@ confused. Please use '\\[mew-message-forward]' instead."
 	(setq url (mew-puny-encode-url url)))
     (if url (browse-url url))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; lines
+;;;
+
+(defun mew-message-line-status ()
+  (cond
+   ((not (eq buffer-undo-list t)) 1) ;; wrapped
+   (truncate-lines 2) ;; long
+   (t 0))) ;; normal
+
+(defun mew-message-line ()
+  "Toggle normal lines, wrapped lines and long lines."
+  (interactive)
+  (let* ((N 3)
+	 (status (% (1+ (mew-message-line-status)) N)))
+    (mew-elet
+     (cond
+      ((= status 0)
+       (setq truncate-lines nil)
+       (message "Normal lines"))
+      ((= status 1)
+       (setq buffer-undo-list nil)
+       (mew-wrap-lines (if (mew-header-end) (1+ (mew-header-end)) (point-min)) (point-max))
+       (message "Wrapped lines"))
+      ((= status 2)
+       (save-excursion
+	 (primitive-undo (length buffer-undo-list) buffer-undo-list))
+       (buffer-disable-undo)
+       (setq truncate-lines t)
+       (message "Long lines")))
+     (set-window-hscroll (selected-window) 0)
+     (set-buffer-modified-p nil))))
+
+(defun mew-normal-line ()
+  (setq truncate-lines nil))
+
+(defcustom mew-wrap-lines-column fill-column
+  "*Number of fill column to wrap line."
+  :group 'mew-message
+  :type 'integer)
+
+(defun mew-wrap-lines (beg end)
+  (save-restriction
+    (narrow-to-region beg end)
+    (save-excursion
+      (goto-char (point-min))
+      (let ((adaptive-fill-mode nil)
+	    (winwidth (- (window-width) 2))
+	    (fill-column mew-wrap-lines-column)
+	    width start fill-prefix)
+	(while (not (eobp))
+	  (setq start (point))
+	  (setq fill-prefix (mew-fill-match-adaptive-prefix))
+	  (end-of-line)
+	  (setq width (current-column))
+	  (forward-line)
+	  (if fill-prefix
+	      (setq fill-column (+ mew-wrap-lines-column (string-width fill-prefix)))
+	    (setq fill-column mew-wrap-lines-column))
+	  (setq fill-column (min fill-column winwidth))
+	  (when (> width fill-column)
+	    (fill-region-as-paragraph start (1- (point)))))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Horizontal Scrolling
+;;;
+
+(defun mew-message-scroll-left ()
+  "Scroll the Message window to the right."
+  (interactive)
+  (call-interactively 'scroll-left))
+
+(defun mew-message-scroll-right ()
+  "Scroll the Message window to the left."
+  (interactive)
+  (call-interactively 'scroll-right))
+
 (provide 'mew-message)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -255,7 +323,7 @@ confused. Please use '\\[mew-message-forward]' instead."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
