@@ -34,7 +34,7 @@ This function are used to check validity of 'case' and sort key.")
                     (fboundp mew-input-exit-minibuffer-function)))
           (funcall mew-input-exit-minibuffer-function))
       (exit-minibuffer)))
- 
+
 (defvar mew-input-comma-function nil
   "A function to be called when ',' is typed in minibuffer.
 This function can be used to check validity of 'case'.")
@@ -55,8 +55,7 @@ This function can be used to check validity of 'case'.")
 (defun mew-input-clear ()
   "A function to clean up side effects of window configuration
 at completions."
-  (save-excursion
-    (set-buffer (window-buffer (minibuffer-window)))
+  (with-current-buffer (window-buffer (minibuffer-window))
     ;; (mew-ainfo-get-win-cfg) is shared by many functions
     ;; because minibuffer is just one!
     (mew-ainfo-set-win-cfg nil)))
@@ -93,8 +92,7 @@ at completions."
     (when (and win
 	       mew-input-folder-search-key
 	       mew-input-folder-search-match)
-      (save-excursion
-	(set-buffer mew-buffer-completions)
+      (with-current-buffer mew-buffer-completions
 	(mew-elet
 	 (goto-char (point-min))
 	 (when (looking-at "^Key: ")
@@ -107,10 +105,7 @@ at completions."
 	   (goto-char (match-beginning 1))
 	   (setq face (if (not window-system)
 			  'mew-face-header-from
-			(if (or (and mew-xemacs-p (find-face 'isearch))
-				(and (not mew-xemacs-p) (facep 'isearch)))
-			    'isearch
-			  'region)))
+			(if (facep 'isearch) 'isearch 'region)))
 	   (put-text-property (point) (match-end 1) 'face face)
 	   (unless (pos-visible-in-window-p (point) win)
 	     (set-window-start win (progn (forward-line -2) (point))))))))))
@@ -130,12 +125,11 @@ at completions."
     (mew-input-folder-search-setup1 min)))
 
 (defun mew-input-folder-search-setup-buffer (alist min)
-  (save-excursion
-    (set-buffer (get-buffer-create mew-input-folder-search-buf))
+  (with-current-buffer (get-buffer-create mew-input-folder-search-buf)
     (setq buffer-read-only t)
     (mew-elet
      (mew-erase-buffer)
-     (mapcar (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
+     (mapc (lambda (x) (if (stringp (car x)) (insert (car x) "\n"))) alist))
     (if min (goto-char (point-min)))))
 
 (defun mew-input-folder-search-setup1 (&optional min)
@@ -173,7 +167,7 @@ at completions."
   (let (do-search alist)
     (save-excursion
       (goto-char (point-max))
-      (if (search-backward "," nil t)
+      (if (search-backward "," (minibuffer-prompt-end) t)
 	  (progn
 	    (goto-char (match-end 0))
 	    (unless (looking-at mew-regex-case2)
@@ -204,8 +198,7 @@ at completions."
 
 (defun mew-input-folder-search-forward-1 (&optional again)
   (let (no-match pos)
-    (save-excursion
-      (set-buffer mew-input-folder-search-buf)
+    (with-current-buffer mew-input-folder-search-buf
       (setq pos (point))
       (if again
 	  (forward-line)
@@ -219,7 +212,7 @@ at completions."
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
 	(goto-char pos)))
-    (if no-match 
+    (if no-match
 	(mew-input-folder-display "no match")
       (mew-input-folder-display))))
 
@@ -245,8 +238,7 @@ at completions."
 
 (defun mew-input-folder-search-backward-1 (&optional again)
   (let (no-match pos)
-    (save-excursion
-      (set-buffer mew-input-folder-search-buf)
+    (with-current-buffer mew-input-folder-search-buf
       (setq pos (point))
       (if again
 	  (progn (forward-line -1) (end-of-line))
@@ -260,7 +252,7 @@ at completions."
 		   (save-excursion (end-of-line) (point)))))
 	(setq no-match t)
 	(goto-char pos)))
-    (if no-match 
+    (if no-match
 	(mew-input-folder-display "no match")
       (mew-input-folder-display))))
 
@@ -334,20 +326,12 @@ and does not insert anything on single folder input."
 When in folder search mode, this function searches a candidate
 folder and displays it in addition to its bound key."
   (interactive)
-  (let* ((key (if mew-xemacs-p
-		  (event-key last-command-event)
-		(this-command-keys)))
-	 last-str gfunc)
+  (let ((key (this-command-keys))
+	last-str gfunc)
     (cond
-     ((mew-characterp key) ;; XEmacs
-      (setq key (char-to-string key))
-      (setq last-str key)
-      (setq gfunc (lookup-key (current-global-map) key)))
      ((stringp key)
       (setq last-str key)
       (setq gfunc (lookup-key (current-global-map) key)))
-     ((symbolp key) ;; XEmacs
-      (setq gfunc (lookup-key (current-global-map) key)))
      ((vectorp key)
       (setq gfunc (lookup-key (current-global-map) key))
       (unless gfunc
@@ -369,8 +353,7 @@ folder and displays it in addition to its bound key."
 	    (when (string= mew-input-folder-search-key "")
 	      (setq mew-input-folder-search-key nil)
 	      (setq mew-input-folder-search-match nil)
-	      (save-excursion
-		(set-buffer mew-input-folder-search-buf)
+	      (with-current-buffer mew-input-folder-search-buf
 		(cond
 		 ((eq mew-input-folder-search-direction 'forward)
 		  (goto-char (point-min)))
@@ -391,7 +374,7 @@ folder and displays it in addition to its bound key."
        ((null gfunc)
 	())
        ((string-match "self-insert-command" (symbol-name gfunc))
-	(insert last-command-char))
+	(insert last-command-event))
        ((and (fboundp gfunc) (commandp gfunc))
 	(call-interactively gfunc))))))
 
@@ -409,7 +392,7 @@ folder and displays it in addition to its bound key."
 
 (defun mew-input-folder-prefix ()
   "A function to insert a folder prefix in minibuffer.
-If the previous character is another folder prefix, 
+If the previous character is another folder prefix,
 it is deleted automatically."
   (interactive)
   (cond
@@ -423,55 +406,55 @@ it is deleted automatically."
 	   (prefix (if (eq mew-input-complete-function
 			   'mew-complete-local-folder)
 		       mew-folder-local mew-folder-imap))
-	   (insert-ok (string= (char-to-string last-command-char) prefix)))
+	   (insert-ok (string= (char-to-string last-command-event) prefix)))
       (cond
        ((or (= pos 0)
 	    (save-excursion (forward-char -1) (looking-at ",")))
 	(if insert-ok (insert prefix)))
        ((or (and (= pos 1)
-		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]")))
 	    ;; excluding *
 	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
-	;; A wrong prefix might accidentally be here. 
+	;; A wrong prefix might accidentally be here.
 	;; So, replace it just in case
 	(when insert-ok
 	  (forward-char -1)
 	  (delete-char 1)
 	  (insert prefix)))
        (t
-	(insert last-command-char)))))
+	(insert last-command-event)))))
    (t
     (let ((pos (- (point) (mew-minibuf-point-min))))
       (cond
        ((or (= pos 0)
 	    (save-excursion (forward-char -1) (looking-at ",")))
-	(mew-input-folder-complete-case last-command-char)
-	(insert last-command-char))
+	(mew-input-folder-complete-case last-command-event)
+	(insert last-command-event))
        ;;
        ((or (and (= pos 1)
-		 (save-excursion (forward-char -1) (looking-at "[-+%$]"))) 
+		 (save-excursion (forward-char -1) (looking-at "[-+%$]")))
 	    ;; excluding *
 	    (save-excursion (forward-char -2) (looking-at ",[-+%$]")))
 	(forward-char -1)
 	(delete-char 1)
-	(mew-input-folder-complete-case last-command-char)
-	(insert last-command-char))
+	(mew-input-folder-complete-case last-command-event)
+	(insert last-command-event))
        ;;
        ((save-excursion
 	  (forward-char -2)
 	  (looking-at ":[-+%$*]"))
 	(forward-char -1)
 	(delete-char 1)
-	(when (memq last-command-char '(?+ ?*))
+	(when (memq last-command-event '(?+ ?*))
 	  ;; delete case:
 	  (delete-region (save-excursion (or (and (search-backward "," nil t)
 						  (match-end 0))
 					     (mew-minibuf-point-min)))
 			 (point)))
-	(insert last-command-char))
+	(insert last-command-event))
        ;;
        (t
-	(insert last-command-char)))))))
+	(insert last-command-event)))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -479,10 +462,8 @@ it is deleted automatically."
 ;;;
 
 (defun mew-input-folder-check (folders)
-  (let (case:folder folder ret)
-    (while folders
-      (setq case:folder (car folders))
-      (setq folders (cdr folders))
+  (let (folder ret)
+    (dolist (case:folder folders)
       (catch 'continue
 	(if (string-match mew-regex-case case:folder)
 	    (progn
@@ -495,9 +476,6 @@ it is deleted automatically."
 	(if (and (mew-folder-popp folder) ;; not $inbox
 		 (not (string= mew-pop-inbox-folder folder)))
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq case:folder (mew-canonicalize-case-folder case:folder))
 	(setq ret (cons case:folder ret))))
     (setq ret (delete nil ret))
@@ -529,10 +507,20 @@ it is deleted automatically."
 	 ;; mew-inherit-case must be nil
 	 (ret (read-from-minibuffer (format "Folder name (%s): " default)
 				    init mew-input-folder-map nil
-				    'mew-input-folder-hist)))
+				    'mew-input-folder-hist))
+	 (new-folder))
     (when (or (string= ret "") (string= ret init))
       (setq ret default))
-    (car (mew-input-folder-check (list ret)))))
+    ;; mew-input-folder-check requires an IMAP separator, 
+    ;; which requires C-uZ. C-uZ needs "g". So, chicken and egg.
+    ;; Skip the check for inbox to avoid the problem.
+    (setq new-folder (mew-case:folder-folder ret))
+    (if (member new-folder (list mew-inbox-folder
+				 mew-pop-inbox-folder
+				 mew-imap-inbox-folder
+				 mew-nntp-newsgroup))
+	ret
+      (car (mew-input-folder-check (list ret))))))
 
 (defun mew-input-folders (case:folder)
   "Input a folder from the minibuffer."
@@ -552,10 +540,8 @@ it is deleted automatically."
     (mew-input-folder-check ret)))
 
 (defun mew-input-refile-folder-check (folders proto)
-  (let (case:folder folder ret)
-    (while folders
-      (setq case:folder (car folders))
-      (setq folders (cdr folders))
+  (let (folder ret)
+    (dolist (case:folder folders)
       (catch 'continue
 	;; If case is specified, just remove it at this moment...
 	(if (string-match mew-regex-case case:folder)
@@ -573,9 +559,6 @@ it is deleted automatically."
 	(if (and (= (length folder) 1)
 		 (member folder mew-folder-prefixes)) ;; "prefix" only
 	    (throw 'continue nil))
-	(when (string-match " " folder)
-	  (mew-warn "SPC is not allowed!")
-	  (throw 'continue nil))
 	(setq folder (mew-canonicalize-case-folder folder))
 	(setq ret (cons folder ret))))
     (nreverse ret)))
@@ -626,7 +609,7 @@ it is deleted automatically."
   (mew-input-clear)
   (let ((mew-input-complete-function 'mew-complete-address)
 	val vals addrs ret)
-    (setq val (read-from-minibuffer 
+    (setq val (read-from-minibuffer
 	       (if default (format prompt default) prompt)
 	       ""
 	       mew-input-map
@@ -635,9 +618,7 @@ it is deleted automatically."
     (if (and default (string= val ""))
 	(setq val default))
     (setq vals (mapcar 'mew-chop (mew-split-quoted val ?,)))
-    (while vals
-      (setq val (car vals))
-      (setq vals (cdr vals))
+    (dolist (val vals)
       (setq addrs (mew-alias-expand val mew-addrbook-alist 0))
       (setq addrs (mapcar 'mew-addrstr-append-domain addrs))
       (setq ret (nconc ret addrs)))
@@ -719,16 +700,16 @@ it is deleted automatically."
 		      (format "Sort by (%s)?: " key)
 		      ""
 		      mew-input-map
-		      nil 
+		      nil
 		      'mew-input-sort-key-hist))
 	 field mode)
     (if (or (null field:mode) (string= field:mode ""))
 	(setq field:mode key))
     (setq field (car (mew-split field:mode ?:)))
-    (setq mode  (or (car (cdr (mew-split field:mode ?:)))
-		    (cdr (assoc field mew-sort-key-alist))
-		    "text"))
-    (cons field mode)))
+    (setq mode (or (car (cdr (mew-split field:mode ?:)))
+		   (mew-alist-get-value (assoc field mew-sort-key-alist))
+		   "text"))
+    (list field mode)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -754,8 +735,7 @@ it is deleted automatically."
 
 (defun mew-range-update (folder)
   (when (get-buffer folder)
-    (save-excursion
-      (set-buffer folder)
+    (with-current-buffer folder
       (goto-char (point-max))
       (if (bobp)
 	  mew-range-all ;; buffer is empty.
@@ -948,14 +928,12 @@ it is deleted automatically."
 ;;;
 
 (defun mew-input-case-check ()
-  (let* ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
-         (lst (mew-split case ?,))
-	 err)
+  (let ((case (mew-buffer-substring (mew-minibuf-point-min) (point-max)))
+	err)
     (catch 'nomatch
-      (while lst
-	(unless (member (car lst) mew-config-cases)
-	  (throw 'nomatch (setq err (car lst))))
-	(setq lst (cdr lst))))
+      (dolist (cs (mew-split case ?,))
+	(unless (member cs mew-config-cases)
+	  (throw 'nomatch (setq err cs)))))
     (if err
         (progn
           (mew-temp-minibuffer-message (format " [No match: %s]" err))
@@ -971,7 +949,7 @@ it is deleted automatically."
 	(mew-circular-complete-function 'mew-circular-complete-case)
 	(mew-input-exit-minibuffer-function 'mew-input-case-check)
 	(mew-input-comma-function 'mew-input-case-check)
-	case lst ret)
+	case ret)
     (if edit
 	(setq case (read-from-minibuffer
 		    "Case value: "
@@ -987,11 +965,9 @@ it is deleted automatically."
                   'mew-input-case-hist)))
     (if (string= case "")
 	default
-      (setq lst (mew-split case ?,))
-      (while lst
-	(if (member (car lst) mew-config-cases)
-	    (setq ret (cons (car lst) ret)))
-	(setq lst (cdr lst)))
+      (dolist (cs (mew-split case ?,))
+	(if (member cs mew-config-cases)
+	    (setq ret (cons cs ret))))
       (mapconcat 'identity (nreverse ret) ","))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -1014,24 +990,6 @@ it is deleted automatically."
      (t (message "Mark %c is not supported" char)
 	nil))))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Language name
-;;;
-
-;; read-language-name should be used. Unfortunately, that of
-;; XEmacs 21.1.14 is broken. This function should be obsoleted
-;; when Mew quit support of XEmacs 21.1.14
-
-(defun mew-input-language-name (prompt &optional default) 
-  (let* ((completion-ignore-case t)
-         (name (completing-read prompt
-                                language-info-alist
-                                nil
-                                t nil nil)))
-    (if (string= name "") (setq name default))
-    name))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Encoding
@@ -1040,13 +998,13 @@ it is deleted automatically."
 (defun mew-input-encoding ()
   (let* ((encoding-list (list mew-b64 mew-qp))
 	 (encoding-alist (mapcar (lambda (x) (list x)) encoding-list))
-	 (prompt (format "Lines are too long. Input encoding (%s): " 
+	 (prompt (format "Lines are too long. Input encoding (%s): "
 			 mew-default-encoding))
 	 (cte (completing-read prompt encoding-alist)))
     (if (string= cte "") (setq cte mew-default-encoding))
     (setq cte (downcase cte))
     (while (not (member cte encoding-list))
-      (setq prompt (format "'%s' is unknown. Input encoding (%s): " 
+      (setq prompt (format "'%s' is unknown. Input encoding (%s): "
 			   cte mew-default-encoding))
       (setq cte (completing-read prompt encoding-alist))
       (if (string= cte "") (setq cte mew-default-encoding))
@@ -1072,13 +1030,13 @@ it is deleted automatically."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1997-2006 Mew developing team.
+;; Copyright (C) 1997-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1087,7 +1045,7 @@ it is deleted automatically."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
