@@ -6,19 +6,16 @@
 ;;; Code:
 
 (require 'mew)
-(eval-when-compile
-  (if (mew-which-el "timer")
-      (require 'timer)))
 
 ;;;
-;;; PGP 2.6.x is supported. 
+;;; PGP 2.6.x is supported.
 ;;; PGP 5.x is supported. But very ad-hoc.
-;;; GNUPG 1.0.2 is supported. 
+;;; GNUPG 1.0.2 or later is supported.
 ;;; PGP 6.x is supported. But very ad-hoc.
 ;;;
 
 (defvar mew-pgp-ver nil
-  "Automatically set 0 if PGP version is 2. 
+  "Automatically set 0 if PGP version is 2.
 Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 
 (defconst mew-pgp-ver2 0)
@@ -120,7 +117,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     "Overwrite (y/N)\\? "
     "Overwrite (y/N)\\? "))
 
-(defconst mew-pgp-msg-enter-pass 
+(defconst mew-pgp-msg-enter-pass
   '("Enter pass phrase: "
     "Enter pass phrase: "
     "Enter pass phrase: "
@@ -168,6 +165,12 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     "Unsupported packet format" ;; including algorithms and packets
     "xxx"))
 
+(defconst mew-pgp-msg-nocross
+  '("xxx"
+    "xxx"
+    "xxx"
+    "is not cross-certified"))
+
 (defconst mew-pgp-verify-addr
   '(".* \\(signature from user\\) "
     "\\(   \\)"
@@ -223,7 +226,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 (defvar mew-pgp-result-pubring  "No public keyring")
 (defvar mew-pgp-result-secring  "No secret keyring")
 (defvar mew-pgp-result-pubkey   "No his/her public key")
-(defvar mew-pgp-result-expired  "His/her public key is expired")
+(defvar mew-pgp-result-expired  "His/her public key is expired or not signed by yourself")
 (defvar mew-pgp-result-invalid
   "His/her public key is invalid. Sign the key by yourself, first")
 (defvar mew-pgp-result-seckey   "No your secret key")
@@ -233,21 +236,21 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 (defvar mew-pgp-result-sec-succ "PGP decrypted")
 (defvar mew-pgp-result-dec-fail "PGP NOT decrypted for some reasons")
 (defvar mew-pgp-result-unsup    "PGP unsupported signature")
+(defvar mew-pgp-result-nocross  "His/her Public key is not cross-certified")
 
 (defvar mew-pgp-prompt-enter-pass   "Enter pass phrase (%s): ")
 (defvar mew-pgp-prompt-reenter-pass "Re-enter pass phrase (%s): ")
 
-(defsubst mew-pgp-get (list-or-vec)
+(defun mew-pgp-get (list-or-vec)
   (elt list-or-vec mew-pgp-ver))
 
-(defsubst mew-pgp-set (vec val)
+(defun mew-pgp-set (vec val)
   (aset vec mew-pgp-ver val))
 
 
 (defun mew-pgp-debug (label string)
   (when (mew-debug 'pgp)
-    (save-excursion
-      (set-buffer (get-buffer-create mew-buffer-debug))
+    (with-current-buffer (get-buffer-create mew-buffer-debug)
       (goto-char (point-max))
       (insert (format "\n<%s>\n%s\n" label string)))))
 
@@ -292,7 +295,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
   (mew-pgp-setup))
 
 ;; xxx how about multiple users on local machine?
-(defsubst mew-pgp-passtag ()
+(defun mew-pgp-passtag ()
   (mew-pgp-get mew-pgp-list))
 
 (defun mew-pgp-passphrase (&optional again)
@@ -322,17 +325,17 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 		    (setq keyid "not found")
 		  (setq keyid (format "0x%s"(mew-match-string 1))))
 		(setq ret (concat mew-pgp-result-pubkey ": ID = " keyid)))
-	    (goto-char (point-min))
 	    (if (search-forward (mew-pgp-get mew-pgp-msg-no-keyring) nil t)
 		(setq ret mew-pgp-result-pubring)
-	      (goto-char (point-min))
 	      (if (re-search-forward (mew-pgp-get mew-pgp-msg-unsupported) nil t)
 		  (setq ret mew-pgp-result-unsup)
-		;; this line must be nil since this function is used
-		;; by the decryption function, too, for signed-then-encrypted
-		;; messages. We cannot tell whether or not signatures exist
-		;; from the outside of the cipher.
-		))))
+		(if (re-search-forward (mew-pgp-get mew-pgp-msg-nocross) nil t)
+		    (setq ret mew-pgp-result-nocross)
+		  ;; this line must be nil since this function is used
+		  ;; by the decryption function, too, for signed-then-encrypted
+		  ;; messages. We cannot tell whether or not signatures exist
+		  ;; from the outside of the cipher.
+		  )))))
       ;; Signature result is found.
       (setq ret (concat (mew-match-string 1) " PGP sign "))
       (goto-char (point-max))
@@ -411,14 +414,15 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
       (insert "Version: 1\n")
       (write-region (point-min) (point-max) file2 nil 'no-msg))
     (setq file3 (concat (mew-make-temp-name) mew-pgp-ascii-suffix))
+    (if (and mew-encrypt-to-myself
+	     (not (member mew-inherit-encode-pgp-signer decrypters)))
+	(setq decrypters (cons mew-inherit-encode-pgp-signer decrypters)))
     (if (not roption)
 	(setq args (append (list ooption file3 file1) eoptions decrypters))
-      (while decrypters
-	(setq decs (cons (car decrypters) (cons roption decs)))
-	;; nreverse later, take care.
-	(setq decrypters (cdr decrypters)))
-      (setq decrypters (nreverse decs))
-      (setq args (append eoptions decrypters (list ooption file3 file1))))
+      (mapc
+       (lambda (x) (setq decs (cons roption (cons x decs))))
+       decrypters)
+      (setq args (append eoptions decs (list ooption file3 file1))))
     (with-temp-buffer
       (apply 'mew-call-process-lang pgpe nil t nil args)
       (setq check (mew-pgp-encrypt-check)))
@@ -460,7 +464,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (with-temp-buffer
 	    (insert mew-pgp-string)
 	    (setq verify (mew-pgp-verify-check)))
-	  (when verify 
+	  (when verify
 	    (setq mew-pgp-decrypt-msg (concat mew-pgp-decrypt-msg "\n\t" verify))))
       ;; unpredictable error
       (mew-passwd-set-passwd (mew-pgp-passtag) nil)
@@ -510,7 +514,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 		 "PGP sign"
 		 (current-buffer) ;; xxx nil?
 		 pgps
-		 (append soptions (list loption mew-inherit-encode-signer ooption file2 file1))))
+		 (append soptions (list loption mew-inherit-encode-pgp-signer ooption file2 file1))))
     (mew-set-process-cs process mew-cs-autoconv mew-cs-dummy)
     (set-process-filter process 'mew-pgp-process-filter1)
     (set-process-sentinel process 'mew-pgp-process-sentinel)
@@ -526,146 +530,150 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 ;;;
 
 (defun mew-pgp-process-sentinel (process event)
-  (let ((decrypted mew-pgp-result-sec-succ)
-	(msg ""))
-    (if (not mew-pgp-failure)
+  (save-excursion
+    (let ((decrypted mew-pgp-result-sec-succ)
+	  (msg ""))
+      (if (not mew-pgp-failure)
+	  (cond
+	   ((eq mew-pgp-running 'decrypting)
+	    (setq mew-pgp-decrypt-msg decrypted))
+	   ((eq mew-pgp-running 'signing)
+	    (setq mew-pgp-sign-msg nil)))
+	(cond
+	 ;; sign or decrypt
+	 ((eq mew-pgp-failure mew-pgp-err-pass)
+	  (setq msg mew-pgp-result-pass))
+	 ;; decrypt-then-verify
+	 ((eq mew-pgp-failure mew-pgp-err-pubring)
+	  (setq msg decrypted))
+	 ;; decrypt-then-verify
+	 ((eq mew-pgp-failure mew-pgp-err-pubkey)
+	  (setq msg decrypted))
+	 ;; sign
+	 ((eq mew-pgp-failure mew-pgp-err-secring)
+	  (setq msg mew-pgp-result-secring))
+	 ;; sign
+	 ((eq mew-pgp-failure mew-pgp-err-seckey)
+	  (setq msg mew-pgp-result-seckey))
+	 ;; decrypt
+	 ((eq mew-pgp-failure mew-pgp-err-seckey-or-secring)
+	  (setq msg mew-pgp-result-seckey-or-secring))
+	 ;; other
+	 (t ;; mew-pgp-err-other or nil
+	  (setq msg mew-pgp-result-other)))
 	(cond
 	 ((eq mew-pgp-running 'decrypting)
-	  (setq mew-pgp-decrypt-msg decrypted))
+	  (setq mew-pgp-decrypt-msg msg))
 	 ((eq mew-pgp-running 'signing)
-	  (setq mew-pgp-sign-msg nil)))
-      (cond
-       ;; sign or decrypt
-       ((eq mew-pgp-failure mew-pgp-err-pass)
-	(setq msg mew-pgp-result-pass))
-       ;; decrypt-then-verify
-       ((eq mew-pgp-failure mew-pgp-err-pubring)
-	(setq msg decrypted))
-       ;; decrypt-then-verify
-       ((eq mew-pgp-failure mew-pgp-err-pubkey)
-	(setq msg decrypted))
-       ;; sign
-       ((eq mew-pgp-failure mew-pgp-err-secring)
-	(setq msg mew-pgp-result-secring))
-       ;; sign
-       ((eq mew-pgp-failure mew-pgp-err-seckey)
-	(setq msg mew-pgp-result-seckey))
-       ;; decrypt
-       ((eq mew-pgp-failure mew-pgp-err-seckey-or-secring)
-	(setq msg mew-pgp-result-seckey-or-secring))
-       ;; other
-       (t ;; mew-pgp-err-other or nil
-	(setq msg mew-pgp-result-other)))
-      (cond
-       ((eq mew-pgp-running 'decrypting)
-	(setq mew-pgp-decrypt-msg msg))
-       ((eq mew-pgp-running 'signing)
-	(setq mew-pgp-sign-msg msg))))
-    (setq mew-pgp-running nil)))
+	  (setq mew-pgp-sign-msg msg))))
+      (setq mew-pgp-running nil))))
 
 (defun mew-pgp-process-filter1 (process string)
-  ;; sign or decrypt, not verify
-  (mew-pgp-debug "PGP filter1" string)
-  (setq mew-pgp-string (concat mew-pgp-string string))
-  (cond
-   ;; no secret key or no secring for decrypt
-   ((string-match (mew-pgp-get mew-pgp-msg-no-seckey-or-secring) string)
-    (setq mew-pgp-failure mew-pgp-err-seckey-or-secring)
-    (set-process-filter process 'mew-pgp-process-filter3))
+  (save-excursion
+    ;; sign or decrypt, not verify
+    (mew-pgp-debug "PGP filter1" string)
+    (setq mew-pgp-string (concat mew-pgp-string string))
+    (cond
+     ;; no secret key or no secring for decrypt
+     ((string-match (mew-pgp-get mew-pgp-msg-no-seckey-or-secring) string)
+      (setq mew-pgp-failure mew-pgp-err-seckey-or-secring)
+      (set-process-filter process 'mew-pgp-process-filter3))
 
-   ;; no secring for sign
-   ((string-match (mew-pgp-get mew-pgp-msg-no-keyring) string)
-    (setq mew-pgp-failure mew-pgp-err-secring)
-    ;; Enter secret key filename: 
-    (process-send-string process "\n")
-    (set-process-filter process 'mew-pgp-process-filter3))
+     ;; no secring for sign
+     ((string-match (mew-pgp-get mew-pgp-msg-no-keyring) string)
+      (setq mew-pgp-failure mew-pgp-err-secring)
+      ;; Enter secret key filename:
+      (process-send-string process "\n")
+      (set-process-filter process 'mew-pgp-process-filter3))
 
-   ;; no secret key for sign
-   ((string-match (mew-pgp-get mew-pgp-msg-no-enckey) string)
-    (setq mew-pgp-failure mew-pgp-err-seckey)
-    ;; Enter secret key filename: 
-    (process-send-string process "\n")
-    (set-process-filter process 'mew-pgp-process-filter3))
+     ;; no secret key for sign
+     ((string-match (mew-pgp-get mew-pgp-msg-no-enckey) string)
+      (setq mew-pgp-failure mew-pgp-err-seckey)
+      ;; Enter secret key filename:
+      (process-send-string process "\n")
+      (set-process-filter process 'mew-pgp-process-filter3))
 
-   ;; pass phrase for sign or decrypt
-   ((string-match (mew-pgp-get mew-pgp-msg-enter-pass) string)
-    (process-send-string process (format "%s\n" (mew-pgp-passphrase)))
-    (set-process-filter process 'mew-pgp-process-filter2))
+     ;; pass phrase for sign or decrypt
+     ((string-match (mew-pgp-get mew-pgp-msg-enter-pass) string)
+      (process-send-string process (format "%s\n" (mew-pgp-passphrase)))
+      (set-process-filter process 'mew-pgp-process-filter2))
 
-   ;; overwrite a previous data with a new data. (multiple data is unsupported)
-   ((string-match (mew-pgp-get mew-pgp-msg-overwrite) string)
-    (process-send-string process "y\n")
-    (set-process-filter process 'mew-pgp-process-filter1))
+     ;; overwrite a previous data with a new data. (multiple data is unsupported)
+     ((string-match (mew-pgp-get mew-pgp-msg-overwrite) string)
+      (process-send-string process "y\n")
+      (set-process-filter process 'mew-pgp-process-filter1))
 
-   ;; just in case
-   ((string-match (mew-pgp-get mew-pgp-msg-enter) string)
-    (setq mew-pgp-failure mew-pgp-err-other)
-    ;; Enter PSWD:
-    (process-send-string process "\n")
-    (set-process-filter process 'mew-pgp-process-filter3))))
+     ;; just in case
+     ((string-match (mew-pgp-get mew-pgp-msg-enter) string)
+      (setq mew-pgp-failure mew-pgp-err-other)
+      ;; Enter PSWD:
+      (process-send-string process "\n")
+      (set-process-filter process 'mew-pgp-process-filter3)))))
 
 (defun mew-pgp-process-filter2 (process string)
-  (mew-pgp-debug "PGP filter2" string)
-  (setq mew-pgp-string (concat mew-pgp-string string))
-  (cond
-   ;; re-enter pass phrase
-   ((string-match (mew-pgp-get mew-pgp-msg-reenter-pass) string)
-    (setq mew-pgp-string nil)
-    (mew-passwd-set-passwd (mew-pgp-passtag) nil) ;; cancel anyway
-    (process-send-string process (format "%s\n" (mew-pgp-passphrase 'again)))
-    (set-process-filter process 'mew-pgp-process-filter2))
+  (save-excursion
+    (mew-pgp-debug "PGP filter2" string)
+    (setq mew-pgp-string (concat mew-pgp-string string))
+    (cond
+     ;; re-enter pass phrase
+     ((string-match (mew-pgp-get mew-pgp-msg-reenter-pass) string)
+      (setq mew-pgp-string nil)
+      (mew-passwd-set-passwd (mew-pgp-passtag) nil) ;; cancel anyway
+      (process-send-string process (format "%s\n" (mew-pgp-passphrase 'again)))
+      (set-process-filter process 'mew-pgp-process-filter2))
 
-   ;; pass phrases were wrong three times
-   ((string-match (mew-pgp-get mew-pgp-msg-bad-pass) string)
-    (setq mew-pgp-failure mew-pgp-err-pass)
-    (mew-passwd-set-passwd (mew-pgp-passtag) nil) ;; cancel anyway
-    (set-process-filter process 'mew-pgp-process-filter3))
+     ;; pass phrases were wrong three times
+     ((string-match (mew-pgp-get mew-pgp-msg-bad-pass) string)
+      (setq mew-pgp-failure mew-pgp-err-pass)
+      (mew-passwd-set-passwd (mew-pgp-passtag) nil) ;; cancel anyway
+      (set-process-filter process 'mew-pgp-process-filter3))
 
-   ;; no pubring for verify
-   ((string-match (mew-pgp-get mew-pgp-msg-no-keyring) string)
-    (setq mew-pgp-failure mew-pgp-err-pubring)
-    ;; Enter public key filename: 
-    (process-send-string process "\n")
-    (set-process-filter process 'mew-pgp-process-filter3))
+     ;; no pubring for verify
+     ((string-match (mew-pgp-get mew-pgp-msg-no-keyring) string)
+      (setq mew-pgp-failure mew-pgp-err-pubring)
+      ;; Enter public key filename:
+      (process-send-string process "\n")
+      (set-process-filter process 'mew-pgp-process-filter3))
 
-   ;; no public key for verify
-   ((string-match (mew-pgp-get mew-pgp-msg-no-vrfkey) string)
-    (setq mew-pgp-failure mew-pgp-err-pubkey)
-    ;; Enter public key filename: 
-    (process-send-string process "\n")
-    (set-process-filter process 'mew-pgp-process-filter3))
+     ;; no public key for verify
+     ((string-match (mew-pgp-get mew-pgp-msg-no-vrfkey) string)
+      (setq mew-pgp-failure mew-pgp-err-pubkey)
+      ;; Enter public key filename:
+      (process-send-string process "\n")
+      (set-process-filter process 'mew-pgp-process-filter3))
 
-   ;; after decrypted secret key, symmetric key is not unknown...
-   ;; gpg: unknown cipher algorithm
-   ;; no secret key or no secring for decrypt
-   ((string-match (mew-pgp-get mew-pgp-msg-no-seckey-or-secring) string)
-    (setq mew-pgp-failure mew-pgp-err-seckey-or-secring)
-    (set-process-filter process 'mew-pgp-process-filter3))
+     ;; after decrypted secret key, symmetric key is not unknown...
+     ;; gpg: unknown cipher algorithm
+     ;; no secret key or no secring for decrypt
+     ((string-match (mew-pgp-get mew-pgp-msg-no-seckey-or-secring) string)
+      (setq mew-pgp-failure mew-pgp-err-seckey-or-secring)
+      (set-process-filter process 'mew-pgp-process-filter3))
 
-   ;; overwrite a previous data with a new data. (multiple data is unsupported)
-   ((string-match (mew-pgp-get mew-pgp-msg-overwrite) string)
-    (process-send-string process "y\n")
-    (set-process-filter process 'mew-pgp-process-filter2))
+     ;; overwrite a previous data with a new data. (multiple data is unsupported)
+     ((string-match (mew-pgp-get mew-pgp-msg-overwrite) string)
+      (process-send-string process "y\n")
+      (set-process-filter process 'mew-pgp-process-filter2))
 
-   ;; just in case
-   ((string-match (mew-pgp-get mew-pgp-msg-enter) string)
-    (setq mew-pgp-failure mew-pgp-err-other)
-    ;; Enter PSWD:
-    (process-send-string process "\n")
-    (set-process-filter process 'mew-pgp-process-filter3))))
+     ;; just in case
+     ((string-match (mew-pgp-get mew-pgp-msg-enter) string)
+      (setq mew-pgp-failure mew-pgp-err-other)
+      ;; Enter PSWD:
+      (process-send-string process "\n")
+      (set-process-filter process 'mew-pgp-process-filter3)))))
 
 (defun mew-pgp-process-filter3 (process string)
-  (mew-pgp-debug "PGP filter3" string)
-  ;; ending or error
-  (setq mew-pgp-string (concat mew-pgp-string string))
-  ;; string may contain old "Enter"
-  (cond
-   ;; just in case
-   ((string-match (mew-pgp-get mew-pgp-msg-enter) string)
-    ;; (setq mew-pgp-failure mew-pgp-err-other) ;; this is wrong
-    ;; Enter PSWD:
-    (process-send-string process "\n")
-    (set-process-filter process 'mew-pgp-process-filter3))))
+  (save-excursion
+    (mew-pgp-debug "PGP filter3" string)
+    ;; ending or error
+    (setq mew-pgp-string (concat mew-pgp-string string))
+    ;; string may contain old "Enter"
+    (cond
+     ;; just in case
+     ((string-match (mew-pgp-get mew-pgp-msg-enter) string)
+      ;; (setq mew-pgp-failure mew-pgp-err-other) ;; this is wrong
+      ;; Enter PSWD:
+      (process-send-string process "\n")
+      (set-process-filter process 'mew-pgp-process-filter3)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -726,7 +734,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     ("--encrypt" "--armor" "--textmode" "--batch")))
 
 (defun mew-old-pgp-encode (type &optional signer)
-  (let (mew-inherit-encode-signer doit syntax ctl ct charset cs)
+  (let (mew-inherit-encode-pgp-signer doit syntax ctl ct charset cs)
     (if (mew-attach-p)
 	(if (mew-encode-syntax-have-one-part)
 	    (progn
@@ -743,7 +751,9 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (message "Old PGP cannot be used if multipart exists"))
       (setq doit t))
     (when doit
-      (setq mew-inherit-encode-signer (or signer (mew-get-my-address)))
+      (setq mew-inherit-encode-pgp-signer (or signer
+					      (mew-pgp-signer (mew-tinfo-get-case))
+					      (mew-get-my-address)))
       (cond
        ((eq type 'pgp-signature)
 	(mew-old-pgp-sign cs syntax))
@@ -763,15 +773,14 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (unless cs
       (setq charset (mew-charset-guess-region (point) (point-max)))
       (setq cs (mew-charset-to-cs charset)))
-    (mew-frwlet
-     mew-cs-dummy cs
-     (write-region (point) (point-max) file1 nil 'no-msg))
+    (mew-frwlet mew-cs-dummy cs
+      (write-region (point) (point-max) file1 nil 'no-msg))
     (condition-case nil
 	(setq fmc (mew-pgp-sign file1))
       (error
        (mew-delete-file file1)
        (mew-encode-error
-	(format "unknown error for %s. Check %s, anyway" 
+	(format "unknown error for %s. Check %s, anyway"
 		mew-ct-mls mew-temp-dir))))
     (mew-set '(file2 nil nil errmsg) fmc)
     (if errmsg
@@ -780,9 +789,8 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (mew-delete-file file2)
 	  (error errmsg))
       (delete-region (point) (point-max))
-      (mew-frwlet
-       cs mew-cs-dummy 
-       (insert-file-contents file2))
+      (mew-frwlet cs mew-cs-dummy
+	(insert-file-contents file2))
       (mew-delete-file file1)
       (mew-delete-file file2)
       (setq mew-encode-syntax syntax)
@@ -798,18 +806,14 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (unless cs
       (setq charset (mew-charset-guess-region (point) (point-max)))
       (setq cs (mew-charset-to-cs charset)))
-    (mew-frwlet
-     mew-cs-dummy cs
-     (write-region (point) (point-max) file1 nil 'no-msg))
-    (if (and mew-encrypt-to-myself
-	     (not (member mew-inherit-encode-signer decrypters)))
-	(setq decrypters (cons mew-inherit-encode-signer decrypters)))
+    (mew-frwlet mew-cs-dummy cs
+      (write-region (point) (point-max) file1 nil 'no-msg))
     (condition-case nil
 	(setq fc (mew-pgp-encrypt file1 decrypters))
       (error
        (mew-delete-file file1)
        (mew-encode-error
-	(format "unknown error for %s. Check %s, anyway" 
+	(format "unknown error for %s. Check %s, anyway"
 		mew-ct-mle mew-temp-dir))))
     (mew-set '(file2 nil file3 nil errmsg) fc)
     (if errmsg
@@ -820,9 +824,8 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (error errmsg))
       ;; Create multipart content-header
       (delete-region (point) (point-max))
-      (mew-frwlet
-       cs mew-cs-dummy 
-       (insert-file-contents file3))
+      (mew-frwlet cs mew-cs-dummy
+	(insert-file-contents file3))
       (mew-delete-file file1)
       (mew-delete-file file2)
       (mew-delete-file file3)
@@ -844,20 +847,18 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
     (unless cs
       (setq charset (mew-charset-guess-region (point) (point-max)))
       (setq cs (mew-charset-to-cs charset)))
-    (mew-frwlet
-     mew-cs-dummy cs
-     (write-region (point) (point-max) file1 nil 'no-msg))
+    (mew-frwlet mew-cs-dummy cs
+      (write-region (point) (point-max) file1 nil 'no-msg))
     (if (and mew-encrypt-to-myself
-	     (not (member mew-inherit-encode-signer decrypters)))
-	(setq decrypters (cons mew-inherit-encode-signer decrypters)))
+	     (not (member mew-inherit-encode-pgp-signer decrypters)))
+	(setq decrypters (cons mew-inherit-encode-pgp-signer decrypters)))
     (setq file2 (mew-make-temp-name))
-    (setq args (list loption mew-inherit-encode-signer ooption file2 file1))
+    (setq args (list loption mew-inherit-encode-pgp-signer ooption file2 file1))
     (if (not roption)
 	(setq args (append args eoptions decrypters))
-      (while decrypters
-	(setq decs (cons (car decrypters) (cons roption decs)))
+      (dolist (decrypter decrypters)
 	;; nreverse later, take care.
-	(setq decrypters (cdr decrypters)))
+	(setq decs (cons decrypter (cons roption decs))))
       (setq decrypters (nreverse decs))
       (setq args (append eoptions decrypters args)))
     (setq mew-pgp-running 'signing)
@@ -883,9 +884,8 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	  (mew-delete-file file2)
 	  (error (or mew-pgp-sign-msg check)))
       (delete-region (point) (point-max))
-      (mew-frwlet
-       cs mew-cs-dummy 
-       (insert-file-contents file2))
+      (mew-frwlet cs mew-cs-dummy
+	(insert-file-contents file2))
       (mew-delete-file file1)
       (mew-delete-file file2)
       (setq mew-encode-syntax nil)
@@ -929,8 +929,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
      (let ((fld (mew-summary-folder-name))
 	   (msg (mew-summary-message-number2))
 	   type)
-       (save-excursion
-	 (set-buffer (mew-buffer-message))
+       (with-current-buffer (mew-buffer-message)
 	 ;; We need MIME-decoded buffer to check PGP boundaries.
 	 (setq type (mew-old-pgp-check))
 	 (if type
@@ -945,17 +944,16 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	 mew-inherit-decode-signer
 	 file1 file2 result xmew win start cte)
     (with-temp-buffer
-      (mew-frwlet
-       mew-cs-text-for-read mew-cs-text-for-write
-       (mew-insert-file-contents file)
-       (re-search-forward mew-eoh nil t)
-       (forward-line)
-       (delete-region (point-min) (point))
-       (when (eq type 'pgp-signature)
-	 (setq cte (mew-syntax-get-cte (mew-syntax-get-part syntax)))
-	 (mew-decode-mime-body nil mew-ct-txt cte))
-       (setq file1 (mew-make-temp-name))
-       (write-region (point-min) (point-max) file1 nil 'no-msg)))
+      (mew-frwlet mew-cs-text-for-read mew-cs-text-for-write
+	(mew-insert-file-contents file)
+	(re-search-forward mew-eoh nil t)
+	(forward-line)
+	(delete-region (point-min) (point))
+	(when (eq type 'pgp-signature)
+	  (setq cte (mew-syntax-get-cte (mew-syntax-get-part syntax)))
+	  (mew-decode-mime-body nil mew-ct-txt cte))
+	(setq file1 (mew-make-temp-name))
+	(write-region (point-min) (point-max) file1 nil 'no-msg)))
     (mew-elet
      (widen)
      (setq win (get-buffer-window (current-buffer)))
@@ -982,21 +980,19 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
      (when (and file1 (file-exists-p file1))
        (mew-delete-file file1))
      (when (and file2 (file-exists-p file2))
-       (mew-frwlet
-	mew-cs-text-for-read mew-cs-dummy
-	(mew-insert-file-contents file2))
+       (mew-frwlet mew-cs-text-for-read mew-cs-dummy
+	 (mew-insert-file-contents file2))
        (mew-delete-file file2))
-     (if mew-mule-p
-	 (mew-cs-decode-region
-	  (point) (point-max)
-	  ;; This is valid for PGP signature only.
-	  ;; Since cipher text is encoded with Radix 64, charset should
-	  ;; be "US-ASCII". I don't know this code is valid for
-	  ;; cipher text in the case that a wrong charset is specified.
-	  (or (mew-charset-to-cs 
-	       (mew-syntax-get-param
-		(mew-syntax-get-ct (mew-syntax-get-part syntax)) "charset"))
-	      mew-cs-autoconv)))
+     (mew-cs-decode-region
+      (point) (point-max)
+      ;; This is valid for PGP signature only.
+      ;; Since cipher text is encoded with Radix 64, charset should
+      ;; be "US-ASCII". I don't know this code is valid for
+      ;; cipher text in the case that a wrong charset is specified.
+      (or (mew-charset-to-cs
+	   (mew-syntax-get-param
+	    (mew-syntax-get-ct (mew-syntax-get-part syntax)) "charset"))
+	  mew-cs-autoconv))
      (mew-message-set-end-of)
      (set-window-start win start)
      (set-buffer-modified-p nil))))
@@ -1051,7 +1047,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 	    (message "Cannot extract pgp key for %s" user)
 	  (setq mew-encode-syntax
 		(mew-syntax-insert-entry
-		 mew-encode-syntax 
+		 mew-encode-syntax
 		 nums
 		 (mew-encode-syntax-single file mew-type-apk nil user)))
 	  (mew-encode-syntax-print mew-encode-syntax))))))
@@ -1059,7 +1055,7 @@ Set 1 if 5. Set 2 if 6. Set 3 if GNUPG.")
 (defvar mew-pgp-tmp-file nil)
 
 (defun mew-mime-pgp-keys (cache begin end &optional params)
-  "A function to add PGP keys in Application/PGP-Keys to your 
+  "A function to add PGP keys in Application/PGP-Keys to your
 public keyring."
   (mew-elet
    (insert " ######   #####  ######  #     # ####### #     #\n"
@@ -1080,17 +1076,15 @@ public keyring."
       (message "PGP not found")
     (when (y-or-n-p "Add this PGP key onto your public keyring? ")
       (setq mew-pgp-tmp-file (mew-make-temp-name))
-      (save-excursion
-	(set-buffer cache)
-	(mew-frwlet
-	 mew-cs-dummy mew-cs-autoconv
-	 (write-region begin end mew-pgp-tmp-file nil 'no-msg))
+      (with-current-buffer cache
+	(mew-frwlet mew-cs-dummy mew-cs-autoconv
+	  (write-region begin end mew-pgp-tmp-file nil 'no-msg))
 	(set-buffer (mew-buffer-message))
 	(mew-elet
 	 (message "Adding PGP keys...")
 	 (apply 'mew-call-process-lang
 		(mew-pgp-get mew-prog-pgpk)
-		nil t nil 
+		nil t nil
 		(append (mew-pgp-get mew-prog-pgpk-add-arg)
 			(list mew-pgp-tmp-file)))
 	 (message "Adding PGP keys...done")
@@ -1120,14 +1114,10 @@ public keyring."
 ;;;
 
 (defun mew-pgp-fetch-key-by-xuri ()
-  (let ((key-list mew-x-pgp-key-list)
-	key val uri)
-    (save-excursion
-      (set-buffer (mew-buffer-message))
+  (let (val uri)
+    (with-current-buffer (mew-buffer-message)
       (catch 'loop
-	(while key-list
-	  (setq key (car key-list))
-	  (setq key-list (cdr key-list))
+	(dolist (key mew-x-pgp-key-list)
 	  (setq val (mew-header-get-value key))
 	  (when (and val (string-match "http:[^ \t\n]*" val))
 	    (setq val (mew-match-string 0 val))
@@ -1138,20 +1128,19 @@ public keyring."
       (message "A PGP key is not fetched"))))
 
 (defun mew-pgp-fetch-key (arg)
-  "Fetch a PGP public key. 
+  "Fetch a PGP public key.
 
 A PGP public key is fetch from one of the PGP PKS servers with a
 key ID which is extracted from the X-Mew: field. The list of the
 PGP PKS servers is specified by 'mew-pgp-pks-servers'.
 
-If called with \\[universal-argument], a PGP public key is fetched 
+If called with \\[universal-argument], a PGP public key is fetched
 according to a URL in a field specified by 'mew-x-pgp-key-list'."
   (interactive "P")
   (if arg
       (mew-pgp-fetch-key-by-xuri)
     (let (keyid userid xmew)
-      (save-excursion
-	(set-buffer (mew-buffer-message))
+      (with-current-buffer (mew-buffer-message)
 	(setq userid (mew-header-parse-address mew-from:))
 	(setq xmew (mew-header-get-value mew-x-mew:)))
       (cond
@@ -1172,29 +1161,27 @@ according to a URL in a field specified by 'mew-x-pgp-key-list'."
 	 uri)
     (unless id
       (setq id (read-string "PGP key id or e-mail address: ")))
-    (setq server (completing-read "PGP key server: " alist nil nil server))
+    (setq server (mew-input-general "PGP key server" alist nil server))
     (unless (mew-pgp-pks-http-port-p server alist)
-      (setq server (concat server ":" mew-pgp-pks-port)))
+      (setq server (concat server ":" (mew-*-to-string mew-pgp-pks-port))))
     (setq uri (concat "http://" server mew-pgp-pks-lang id))
     (mew-pgp-fetch uri)))
 
 (defun mew-pgp-fetch (uri)
   (message "PGP key fetching...")
   (let (pro)
-    (mew-piolet
-     mew-cs-autoconv mew-cs-autoconv
-     (setq pro (apply 'start-process
-		      "Mew PGP fetch"
-		      (generate-new-buffer mew-buffer-prefix)
-		      mew-prog-pgpkey
-		      (append mew-prog-pgpkey-args (list uri)))))
+    (mew-piolet mew-cs-autoconv mew-cs-autoconv
+      (setq pro (apply 'start-process
+		       "Mew PGP fetch"
+		       (generate-new-buffer mew-buffer-prefix)
+		       mew-prog-pgpkey
+		       (append mew-prog-pgpkey-args (list uri)))))
     (set-process-sentinel pro 'mew-pgp-fetch-process-sentinel)))
 
 (defun mew-pgp-fetch-process-sentinel (process event)
   (message "PGP key fetching...done")
-  (let* ((buf (process-buffer process)))
-    (save-excursion
-      (set-buffer buf)
+  (let ((buf (process-buffer process)))
+    (with-current-buffer buf
       (goto-char (point-min))
       (if (and (search-forward mew-pgp-key-begin nil t)
 	       (search-forward mew-pgp-key-end nil t))
@@ -1204,49 +1191,49 @@ according to a URL in a field specified by 'mew-x-pgp-key-list'."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; <Decrypt> or <Decrypt-then-Verify>
-;; 
+;;
 ;; no seckey no secring:: <1>
 ;; You do not have the secret key needed to decrypt this file.
-;; 
+;;
 ;;    <Verify> <2>
-;; 	Keyring file 'pubring.pgp' does not exist. 
+;; 	Keyring file 'pubring.pgp' does not exist.
 ;; 	Enter public key filename:
-;; 
+;;
 ;; 	Key matching expected Key ID 1B8BF431 not found in file 'pubring.pgp'.
 ;; 	Enter public key filename:
-;; 
+;;
 ;; <Sign> <1>
-;; 
-;; A secret key is required to make a signature. 
-;; Keyring file 'secring.pgp' does not exist. 
-;; Enter secret key filename: 
-;; 
-;; A secret key is required to make a signature. 
+;;
+;; A secret key is required to make a signature.
+;; Keyring file 'secring.pgp' does not exist.
+;; Enter secret key filename:
+;;
+;; A secret key is required to make a signature.
 ;; Key matching userid 'foo' not found in file 'secring.pgp'.
-;; Enter secret key filename: 
-;; 
+;; Enter secret key filename:
+;;
 ;; <Encrypt>
-;; 
-;; Keyring file 'pubring.pgp' does not exist. 
-;; 
+;;
+;; Keyring file 'pubring.pgp' does not exist.
+;;
 ;; Key matching userid 'foo' not found in file 'pubring.pgp'.
-;; 
+;;
 ;; <Verify>
-;; Keyring file 'pubring.pgp' does not exist. 
-;; 
+;; Keyring file 'pubring.pgp' does not exist.
+;;
 ;; Key matching expected Key ID 1B8BF431 not found in file 'pubring.pgp'.
 
 (provide 'mew-pgp)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2007 Mew developing team.
+;; Copyright (C) 1994-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1255,7 +1242,7 @@ according to a URL in a field specified by 'mew-x-pgp-key-list'."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
