@@ -43,7 +43,7 @@ target is the region."
   (interactive "P")
   (mew-pickable
    (mew-summary-with-mewl
-    (let* ((folder (mew-pickable-folder))
+    (let* ((folder (mew-summary-physical-folder))
 	   (msgs (mew-summary-pick-msgs folder regionp))
 	   (prompt (format "%s/%s pick" mew-prog-mewl mew-prog-grep))
 	   (prog mew-prog-grep)
@@ -75,58 +75,74 @@ target is the region."
 	    (setq msgs (mew-summary-pick-with-mewl pattern folder msgs))
 	    (message "Picking messages in %s...done" folder)))
 	  (mew-summary-pick-ls folder msgs)))))))
-  
+
+(defmacro mew-dolist-eob (spec &rest body)
+  ;; (declare (indent 1))
+  `(let ((--mew-temp-- ,(nth 1 spec)) ,(nth 0 spec))
+     (while (and --mew-temp-- (not (eobp)))
+       (setq ,(nth 0 spec) (car --mew-temp--))
+       (setq --mew-temp-- (cdr --mew-temp--))
+       ,@body)))
+(put 'mew-dolist-eob 'lisp-indent-function 1)
+
+(defun mew-summary-pick-ls-thread (msgs)
+  (let* ((preline 0)
+	 (threadmsgs (mew-summary-thread-get-msglst
+		      (mew-vinfo-get-top) 'separator))
+	 (total (length threadmsgs))
+	 (msgs2 msgs)
+	 linenum)
+    (goto-char (point-min))
+    (mew-dolist-eob (msg msgs)
+      (setq linenum (member msg threadmsgs))
+      (if (null linenum)
+	  (setq msgs2 (delete (car msgs) msgs2))
+	(setq linenum (- total (length linenum)))
+	(forward-line (- linenum preline))
+	(setq preline linenum)
+	(mew-summary-mark-as mew-mark-review)))
+    (set-buffer-modified-p nil)
+    msgs2))
+
+(defun mew-summary-pick-ls-selection (msgs)
+  ;; messages are not displayed in order
+  (goto-char (point-min))
+  (let (msgs2)
+    (mew-dolist-eob (msg msgs)
+      (when (mew-summary-search-msg msg)
+	(setq msgs2 (cons (car msgs) msgs2))
+	(mew-summary-mark-as mew-mark-review))
+      (forward-line))
+    (set-buffer-modified-p nil)
+    (nreverse msgs2)))
+
+(defun mew-summary-pick-ls-summary (folder msgs)
+  (when (get-buffer folder)
+    (set-buffer folder)
+    (save-excursion
+      (goto-char (point-min))
+      (mew-dolist-eob (msg msgs)
+	(when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
+	  (mew-summary-mark-as mew-mark-review)
+	  (forward-line))))
+    (set-buffer-modified-p nil)))
+
 (defun mew-summary-pick-ls (folder msgs)
-  (let ((preline 0) (msgsback msgs)
-	threadmsgs total linenum n)
-    (if (null msgs)
-	(message "No message to be marked")
-      (setq n (length msgs))
+  (if (null msgs)
+      (message "No message to be marked")
+    (let ((n (length msgs)) msgs2)
       (if (= n 1)
 	  (message "Marking 1 message...")
 	(message "Marking %d messages..." n))
       (save-excursion
 	(cond
 	 ((mew-thread-p)
-	  ;; Thread mode
-	  (setq threadmsgs (mew-summary-thread-get-msglst
-			    (mew-vinfo-get-top) 'separator))
-	  (setq total (length threadmsgs))
-	  (goto-char (point-min))
-	  (while (and msgs (not (eobp)))
-	    (setq linenum (member (car msgs) threadmsgs))
-	    (if (null linenum)
-		(setq msgsback (delete (car msgs) msgsback))
-	      (setq linenum (- total (length linenum)))
-	      (forward-line (- linenum preline))
-	      (setq preline linenum)
-	      (mew-summary-mark-as mew-mark-review))
-	    (setq msgs (cdr msgs)))
-	  (set-buffer-modified-p nil))
-	 ((mew-virtual-p)
-	  ;; Virtual mode
-	  (goto-char (point-min))
-	  (setq msgsback nil)
-	  (while (and msgs (not (eobp)))
-	    (when (re-search-forward (mew-regex-sumsyn-msg (car msgs)) nil t)
-	      (setq msgsback (cons (car msgs) msgsback))
-	      (mew-summary-mark-as mew-mark-review)
-	      (forward-line))
-	    (setq msgs (cdr msgs)))
-	  (setq msgsback (nreverse msgsback))
-	  (set-buffer-modified-p nil)))
-	;; Summary mode (physical)
-	(when (get-buffer folder)
-	  (set-buffer folder)
-	  (save-excursion
-	    (goto-char (point-min))
-	    (setq msgs msgsback)
-	    (while (and msgs (not (eobp)))
-	      (when (re-search-forward (mew-regex-sumsyn-msg (car msgs)) nil t)
-		(mew-summary-mark-as mew-mark-review)
-		(forward-line))
-	      (setq msgs (cdr msgs)))
-	    (set-buffer-modified-p nil))))
+	  (setq msgs2 (mew-summary-pick-ls-thread msgs)))
+	 ((mew-selection-p)
+	  (setq msgs2 (mew-summary-pick-ls-selection msgs)))
+	 (t ;; Summary mode
+	  (setq msgs2 msgs)))
+	(mew-summary-pick-ls-summary folder msgs2))
       (if (= n 1)
 	  (message "Marking 1 message...done")
 	(message "Marking %d messages...done" n)))))
@@ -142,16 +158,15 @@ target is the region."
 	   (max min)
 	   i)
       (setq src-msgs (cdr src-msgs))
-      (while src-msgs
-	(setq i (string-to-number (car src-msgs)))
+      (dolist (src-msg src-msgs)
+	(setq i (string-to-number src-msg))
 	(if (< i min) (setq min i))
-	(if (> i max) (setq max i))
-	(setq src-msgs (cdr src-msgs)))
+	(if (> i max) (setq max i)))
       (format "%d-%d" min max))))
 
 (defun mew-summary-pick-with-mewl (pattern folder src-msgs)
   "A function to pick messages matching PATTERN with 'mewl'"
-  (let ((pfolder (mew-expand-folder2 folder))
+  (let ((pfolder (mew-scan-mewl-folder (mew-expand-folder2 folder)))
 	(range (mew-summary-pick-range src-msgs))
 	(opts (list "-b" mew-mail-path
 		    "-l" (number-to-string mew-scan-max-field-length)
@@ -163,9 +178,8 @@ target is the region."
       (setq opts (nconc opts (list pfolder))))
     (with-temp-buffer
       (mew-set-buffer-multibyte t)
-      (mew-piolet
-       mew-cs-text-for-read mew-cs-text-for-write
-       (apply 'call-process mew-prog-mewl nil t nil opts))
+      (mew-piolet mew-cs-text-for-read mew-cs-text-for-write
+	(apply 'call-process mew-prog-mewl nil t nil opts))
       (goto-char (point-min))
       (while (re-search-forward mew-regex-message-files2 nil t)
 	(setq msgs (cons (mew-match-string 1) msgs))
@@ -188,15 +202,14 @@ target is the region."
     (with-temp-buffer
       (mew-set-buffer-multibyte t)
       (cd dir)
-      (mew-piolet
-       mew-cs-text-for-read mew-cs-text-for-write
-       (while src-msgs
-	 (goto-char (point-max))
-	 (setq nxt (nthcdr mew-prog-grep-max-msgs src-msgs))
-	 (if nxt (setcdr (nthcdr (1- mew-prog-grep-max-msgs) src-msgs) nil))
-	 (apply 'call-process prog nil t nil
-		(append opts (and pattern (list pattern)) src-msgs))
-	 (setq src-msgs nxt)))
+      (mew-piolet mew-cs-text-for-read mew-cs-text-for-write
+	(while src-msgs
+	  (goto-char (point-max))
+	  (setq nxt (nthcdr mew-prog-grep-max-msgs src-msgs))
+	  (if nxt (mew-ntake mew-prog-grep-max-msgs src-msgs))
+	  (apply 'call-process prog nil t nil
+		 (append opts (and pattern (list pattern)) src-msgs))
+	  (setq src-msgs nxt)))
       (goto-char (point-min))
       (while (re-search-forward mew-regex-message-files2 nil t)
 	(setq msgs (cons (mew-match-string 1) msgs))
@@ -212,13 +225,13 @@ target is the region."
 ;;;
 
 (defun mew-summary-find-keyword-down (&optional arg)
-  "Display a message in the forward direction, find a keyword 
-and highlight it. 
+  "Display a message in the forward direction, find a keyword
+and highlight it.
 In Summary mode, the target is messages marked with '*'.
 In Virtual mode, the target is all messages in a Virtual folder.
-The keyword is stored in a buffer local variable in Summary mode. 
-If no keyword is set to the variable, this command first asks you 
-for a keyword. 
+The keyword is stored in a buffer local variable in Summary mode.
+If no keyword is set to the variable, this command first asks you
+for a keyword.
 If you want to change the stored keyword, execute this command with '\\[universal-argument]'."
   (interactive "P")
   (let* ((fld (mew-summary-folder-name))
@@ -239,14 +252,14 @@ If you want to change the stored keyword, execute this command with '\\[universa
 		 "Keyword: " (or (car mew-input-pick-pattern-hist) key)))
       (mew-sinfo-set-find-key key))
     (cond
-     ((and (or virtualp (equal mew-mark-review mark))
+     ((and (or virtualp (equal mew-mark-review mark)) ;; MUST be equal
 	   (or (not (string= fld ofld)) (not (string= msg omsg))))
       (mew-summary-display-asis)
       (setq top t))
      ((or (null mwin)
-	  (not (or virtualp (equal mew-mark-review mark)))
+	  (not (or virtualp (equal mew-mark-review mark))) ;; MUST be equal
 	  (or (not (string= fld ofld)) (not (string= msg omsg)))
-	  (save-excursion (set-buffer mbuf) (eobp)))
+	  (with-current-buffer mbuf (eobp)))
       (cond
        (virtualp
 	(forward-line)
@@ -274,13 +287,13 @@ If you want to change the stored keyword, execute this command with '\\[universa
 	(select-window cwin)))))
 
 (defun mew-summary-find-keyword-up (&optional arg)
-  "Display a message in the backward direction, find a keyword 
-and highlight it. 
+  "Display a message in the backward direction, find a keyword
+and highlight it.
 In Summary mode, the target is messages marked with '*'.
 In Virtual mode, the target is all messages in a Virtual folder.
-The keyword is stored in a buffer local variable in Summary mode. 
-If no keyword is set to the variable, this command first asks you 
-for a keyword. 
+The keyword is stored in a buffer local variable in Summary mode.
+If no keyword is set to the variable, this command first asks you
+for a keyword.
 If you want to change the stored keyword, execute this command with '\\[universal-argument]'."
   (interactive "P")
   (let* ((fld (mew-summary-folder-name))
@@ -301,20 +314,20 @@ If you want to change the stored keyword, execute this command with '\\[universa
 		 "Keyword: " (or (car mew-input-pick-pattern-hist) key)))
       (mew-sinfo-set-find-key key))
     (cond
-     ((and (or virtualp (equal mew-mark-review mark))
+     ((and (or virtualp (equal mew-mark-review mark)) ;; MUST be equal
 	   (or (not (string= fld ofld)) (not (string= msg omsg))))
       (mew-summary-display-asis)
       (setq bottom t))
      ((or (null mwin)
-	  (not (or virtualp (equal mew-mark-review mark)))
+	  (not (or virtualp (equal mew-mark-review mark))) ;; MUST be equal
 	  (or (not (string= fld ofld)) (not (string= msg omsg)))
-	  (save-excursion (set-buffer mbuf) (bobp)))
+	  (with-current-buffer mbuf (bobp)))
       (cond
        (virtualp
 	(forward-line -1)
 	(mew-summary-display-asis)
 	(setq bottom t))
-       ((mew-summary-down-mark mew-mark-review)
+       ((mew-summary-up-mark mew-mark-review)
 	(mew-summary-display-asis)
 	(setq bottom t))
        (t
@@ -364,7 +377,7 @@ If you want to change the stored keyword, execute this command with '\\[universa
 	(setcdr asc value)
       (setq mew-pick-macro-alist
 	    (cons (cons str1 value) mew-pick-macro-alist)))))
-    
+
 (defun mew-pick-macro-expand (name args)
   (let ((asc (assoc name mew-pick-macro-alist))
 	alist args2 body body-copy assq)
@@ -376,19 +389,18 @@ If you want to change the stored keyword, execute this command with '\\[universa
 	(setq alist (cons (cons (car args2) (car args)) alist))
 	(setq args (cdr args))
 	(setq args2 (cdr args2)))
-      (while body
-	(if (stringp (car body))
-	    (setq body-copy (cons (car body) body-copy))
-	  (setq assq (assq (car body) alist))
-	  (if assq (setq body-copy (cons (cdr assq) body-copy))))
-	(setq body (cdr body)))
+      (dolist (bd body)
+	(if (stringp bd)
+	    (setq body-copy (cons bd body-copy))
+	  (setq assq (assq bd alist))
+	  (if assq (setq body-copy (cons (cdr assq) body-copy)))))
       (concat "("
 	      (mew-pick-macro-expand-string
 	       (apply 'concat (nreverse body-copy)))
 	      ")"))))
 
 (defun mew-pick-macro-expand-string (str)
-  (if (string= str "") 
+  (if (string= str "")
       ""
     (let ((first (string-to-char str))
 	  asc key rest eq-flag val args)
@@ -439,7 +451,7 @@ If you want to change the stored keyword, execute this command with '\\[universa
 (defmacro mew-pick-lex-cut ()
   `(when (and start (< start i))
      (let ((word (substring pattern start i)) key op val)
-       ;; Allowing "to,cc" 
+       ;; Allowing "to,cc"
        (if (string-match "^\\([-a-z0-9,]+\\)\\(!?==?\\)\\(.+\\)$" word)
 	   (progn
 	     (setq key (mew-match-string 1 word))
@@ -454,7 +466,7 @@ If you want to change the stored keyword, execute this command with '\\[universa
 (defun mew-pick-lex (pattern)
   (let ((len (length pattern))
 	(i 0) ret start c dq)
-    (while (< i len)
+    (while (< i len) ;; cannot use dotimes
       (setq c (aref pattern i))
       (cond
        (dq
@@ -657,7 +669,7 @@ If you want to change the stored keyword, execute this command with '\\[universa
 
 (defun mew-pick-filter-est-kyvl (kyvl)
   (unless (and (listp kyvl) (= (length kyvl) 3))
-    (error "Filter is invalid" kyvl))
+    (error "Filter %s is invalid" kyvl))
   (let ((op (nth 0 kyvl))
  	(ky (nth 1 kyvl))
 	(vl (nth 2 kyvl)))
@@ -676,31 +688,6 @@ If you want to change the stored keyword, execute this command with '\\[universa
       (error "'%s' is not supported" op)))
     (format "%s %s %s" ky op vl)))
 
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;
-;;; Namazu
-;;;
-
-(defun mew-pick-canonicalize-pattern-namazu (pattern)
-  (let ((mew-inherit-pick-omit-and t))
-    (mapconcat
-     'mew-pick-native-text-namazu
-     (mew-pick-parse (mew-pick-lex pattern))
-     " ")))
-
-(defun mew-pick-native-text-namazu (token)
-  (mew-pick-native-text "mew-pick-pattern-namazu-" token))
-
-(defun mew-pick-pattern-namazu-and   (sym) "and")
-(defun mew-pick-pattern-namazu-or    (sym) "or")
-(defun mew-pick-pattern-namazu-open  (sym) "(")
-(defun mew-pick-pattern-namazu-close (sym) ")")
-(defun mew-pick-pattern-namazu-not   (sym) "not")
-(defun mew-pick-pattern-namazu-key   (key) key)
-(defun mew-pick-pattern-namazu-kyvl  (kyvl)
-  (when (string= (nth 0 kyvl) "=")
-    (format "+%s:%s" (nth 1 kyvl) (nth 2 kyvl))))
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Google
@@ -709,21 +696,18 @@ If you want to change the stored keyword, execute this command with '\\[universa
 (defun mew-pick-canonicalize-pattern-google (pattern)
   (let ((mew-inherit-pick-omit-and t)
 	(tokens (mew-pick-parse (mew-pick-lex pattern)))
-	tkn ret)
-    (while tokens
-      (setq tkn (car tokens))
-      (setq tokens (cdr tokens))
-      (if (eq tkn 'not)
+	ret)
+    (dolist (token tokens)
+      (if (eq token 'not)
 	  (if (stringp (car tokens))
 	      (progn
 		(setq ret (cons (concat "-" (car tokens)) ret))
 		(setq tokens (cdr tokens)))
 	    nil) ;; xxx
-	(setq ret (cons tkn ret))))
-    (setq tokens (nreverse ret))
+	(setq ret (cons token ret))))
     (mapconcat
      'mew-pick-native-text-google
-     tokens
+     (nreverse ret)
      " ")))
 
 (defun mew-pick-native-text-google (token)
@@ -747,30 +731,27 @@ If you want to change the stored keyword, execute this command with '\\[universa
 (defun mew-pick-canonicalize-pattern-wds (pattern)
   (let ((mew-inherit-pick-omit-and t)
 	(tokens (mew-pick-parse (mew-pick-lex pattern)))
-	tkn ret)
-    (while tokens
-      (setq tkn (car tokens))
-      (setq tokens (cdr tokens))
-      (if (eq tkn 'not)
+	ret)
+    (dolist (token tokens)
+      (if (eq token 'not)
 	  (if (stringp (car tokens))
 	      (progn
 		(setq ret (cons (concat "-" (car tokens)) ret))
 		(setq tokens (cdr tokens)))
 	    nil) ;; xxx
-	(setq ret (cons tkn ret))))
-    (setq tokens (nreverse ret))
+	(setq ret (cons token ret))))
     (mapconcat
      'mew-pick-native-text-wds
-     tokens
+     (nreverse ret)
      " ")))
 
 (defun mew-pick-native-text-wds (token)
   (mew-pick-native-text "mew-pick-pattern-wds-" token))
 
-(defun mew-pick-pattern-wds-and   (sym) "and")
-(defun mew-pick-pattern-wds-or    (sym) "or")
-(defun mew-pick-pattern-wds-open  (sym) (error "'(' is not supported"))
-(defun mew-pick-pattern-wds-close (sym) (error "')' is not supported"))
+(defun mew-pick-pattern-wds-and   (sym) "")
+(defun mew-pick-pattern-wds-or    (sym) "OR")
+(defun mew-pick-pattern-wds-open  (sym) "(")
+(defun mew-pick-pattern-wds-close (sym) ")")
 (defun mew-pick-pattern-wds-not   (sym) "-")
 (defun mew-pick-pattern-wds-key   (key) key)
 (defun mew-pick-pattern-wds-kyvl  (kyvl)
@@ -784,23 +765,10 @@ If you want to change the stored keyword, execute this command with '\\[universa
 
 (defun mew-pick-canonicalize-pattern-spotlight (pattern)
   (let* ((tokens (mew-pick-parse (mew-pick-lex pattern)))
-;;	 tkn ret
-	 str)
-;;     (while tokens
-;;       (setq tkn (car tokens))
-;;       (setq tokens (cdr tokens))
-;;       (if (eq tkn 'not)
-;; 	  (if (stringp (car tokens))
-;; 	      (progn
-;; 		(setq ret (cons (list "!=" "body" (car tokens)) ret))
-;; 		(setq tokens (cdr tokens)))
-;; 	    nil) ;; xxx
-;; 	(setq ret (cons tkn ret))))
-;;     (setq tokens (nreverse ret))
-    (setq str (mapconcat
+	 (str (mapconcat
 	       'mew-pick-native-text-spotlight
 	       tokens
-	       " "))
+	       " ")))
     (concat "kMDItemContentType == \"mew\"w && " str)))
 
 (defun mew-pick-native-text-spotlight (token)
@@ -838,7 +806,7 @@ If you want to change the stored keyword, execute this command with '\\[universa
 ;;       (format "%s != \"%s\"c" kmd vl))
      (t
       (error "'%s' is not supported" op)))))
-    
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
 ;;; Old
@@ -854,13 +822,13 @@ If you want to change the stored keyword, execute this command with '\\[universa
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2006 Mew developing team.
+;; Copyright (C) 1996-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -869,7 +837,7 @@ If you want to change the stored keyword, execute this command with '\\[universa
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
