@@ -657,9 +657,10 @@ with a search method."
   (mew-msgid-check
    (mew-summary-msg
     (let* ((ofolder (mew-summary-folder-name 'ext))
-	   (subj (mew-summary-get-subject))
+	   (subj (or (mew-summary-get-subject) mew-error-no-subject))
 	   (str (mew-subject-simplify2 subj))
-	   (vfolder (mew-folder-to-selection str))
+	   (vfolder (mew-folder-to-selection
+		     (if (string= str "") mew-error-no-subject subj)))
 	   (regex (format "\\(.*\\)/\\([0-9]+\\)\\(%s\\)?$" (regexp-quote mew-suffix)))
 	   (rttl 0)
 	   (file (mew-make-temp-name))
@@ -685,7 +686,7 @@ with a search method."
 	(with-temp-buffer
 	  (mew-set-buffer-multibyte t)
 	  (mew-piolet mew-cs-text-for-read mew-cs-text-for-write
-	    (call-process mew-prog-smew nil t nil db myid mydir)
+	    (call-process mew-prog-smew nil t nil myid db mydir)
 	    (goto-char (point-min))
 	    (setq start (point))
 	    (while (re-search-forward regex nil t)
@@ -711,8 +712,7 @@ with a search method."
 	 (cache (mew-cache-hit fld msg))
 	 subj)
     (if (and cache (get-buffer cache))
-	(save-excursion
-	  (set-buffer cache)
+	(with-current-buffer cache
 	  (setq subj (mew-header-get-value mew-subj:)))
       (with-temp-buffer
 	(mew-insert-message
@@ -747,7 +747,7 @@ with a search method."
 
 (defun mew-summary-make-id-index-all (&optional full-update)
   "Make ID database for all folders.
-If called with '\\[universal-argument]', the database is created 
+If called with '\\[universal-argument]', the database is created
 from scratch."
   (interactive "P")
   (if (not (mew-which-exec mew-prog-cmew))
@@ -771,8 +771,7 @@ from scratch."
 
 (defun mew-summary-make-id-index-sentinel (process event)
   (let ((buf (process-buffer process)))
-    (save-excursion
-      (set-buffer buf)
+    (with-current-buffer buf
       (goto-char (point-max))
       (forward-line -1)
       (let ((str (mew-buffer-substring (line-beginning-position)
@@ -784,7 +783,7 @@ from scratch."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2005-2008 Mew developing team.
+;; Copyright (C) 2005-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
