@@ -252,7 +252,7 @@
 (defun mew-smtp-auth-get-func (auth)
   (nth 1 (mew-assoc-case-equal auth mew-smtp-auth-alist 0)))
 
-;;
+;; CRAM MD5
 
 (defun mew-smtp-command-auth-cram-md5 (pro pnm)
   (mew-smtp-process-send-string pro "AUTH CRAM-MD5")
@@ -269,12 +269,7 @@
     (setq cram-md5 (mew-cram-md5 user passwd challenge))
     (mew-smtp-process-send-string pro cram-md5)))
 
-;; There are two variations for SMTP AUTH LOGIN.
-;; 2) AUTH PLAIN then id then password
-;; 1) AUTH PLAIN id then password
-;;
-;; 1) is popular however.
-;; And a server of an ISP supports 1) only.
+;; LOGIN
 
 (defun mew-smtp-command-auth-login (pro pnm)
   (mew-smtp-process-send-string pro "AUTH LOGIN")
@@ -317,14 +312,14 @@
 
 ;; (defun mew-smtp-command-pwd-plain (pro pnm)
 ;;   (let* ((case (mew-smtp-get-case pnm))
-;; 	 (user (mew-smtp-get-auth-user pnm))
-;; 	 (authorize-id (mew-smtp-auth-plain-authorize-id case))
-;; 	 (prompt (format "SMTP PLAIN password (%s): " user))
-;;          (passwd (mew-smtp-input-passwd prompt pnm))
-;; 	 (plain (mew-base64-encode-string
-;; 		 (if authorize-id
-;; 		     (format "%s\0%s\0%s" user user passwd)
-;; 		   (format "\0%s\0%s" user passwd)))))
+;;	 (user (mew-smtp-get-auth-user pnm))
+;;	 (authorize-id (mew-smtp-auth-plain-authorize-id case))
+;;	 (prompt (format "SMTP PLAIN password (%s): " user))
+;;	 (passwd (mew-smtp-input-passwd prompt pnm))
+;;	 (plain (mew-base64-encode-string
+;;		 (if authorize-id
+;;		     (format "%s\0%s\0%s" user user passwd)
+;;		   (format "\0%s\0%s" user passwd)))))
 ;;     (mew-smtp-process-send-string pro "%s" plain)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -398,7 +393,7 @@
 	(setq pro (make-network-process :name name :buffer buf
 					:host server :service port
 					:family family :nowait nowait))
- 	(if nowait
+	(if nowait
 	    (run-at-time mew-smtp-submission-timeout nil 'mew-smtp-submission-timeout pro))
 	pro))
   (defalias 'mew-open-network-stream 'open-network-stream))
@@ -538,8 +533,7 @@
 
 (defun mew-smtp-debug (label string)
   (when (mew-debug 'net)
-    (save-excursion
-      (set-buffer (get-buffer-create mew-buffer-debug))
+    (with-current-buffer (get-buffer-create mew-buffer-debug)
       (goto-char (point-max))
       (insert (format "\n<%s>\n%s\n" label string)))))
 
@@ -618,13 +612,13 @@
     (save-excursion
       (cond
        (error
-	(message "%s  This mail has been queued to %s" error qfld)
 	(when buf
 	  ;; A message file is not inserted at the beginning of the SMTP
 	  ;; session.
 	  (set-buffer buf)
 	  (mew-smtp-queue case error pnm))
-	(mew-smtp-log pnm error))
+	(mew-smtp-log pnm error)
+	(message-box (format "%s  This mail has been queued to %s" error qfld)))
        (done
 	(message "Sending in background...done"))
        (t
@@ -699,7 +693,8 @@
      (mew-draft-header-insert mew-organization: (mew-organization case))
      (mew-draft-header-insert-alist (mew-header-alist case))
      ;; X-Mailer: must be the last
-     (mew-draft-header-insert mew-x-mailer: mew-x-mailer)
+     (if (mew-use-x-mailer case)
+	 (mew-draft-header-insert mew-x-mailer: mew-x-mailer))
      (mew-header-set "\n")
      (insert mew-bcc-body)
      (goto-char (mew-header-end))
@@ -756,7 +751,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1999-2008 Mew developing team.
+;; Copyright (C) 1999-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
