@@ -28,6 +28,7 @@ A file name of a certificate should be 'cert-hash.0'.
 (defvar mew-prog-ssl-arg nil) ;; xxx what about v4?
 
 (defvar mew-ssl-ver nil)
+(defvar mew-ssl-minor-ver nil)
 
 (defconst mew-ssl-process-exec-cnt 3)
 
@@ -89,7 +90,7 @@ A file name of a certificate should be 'cert-hash.0'.
 		"-d" ,(format "%s:%d" mew-ssl-localhost localport)
 		"-v" ,(number-to-string (mew-ssl-verify-level case))
 		"-D" "debug"
-		"-P" "none"
+		"-P" ""
 		"-r" ,(format "%s:%d" server remoteport)
 		,@mew-prog-ssl-arg))
 	(if tls (setq args (cons "-n" (cons tls args))))
@@ -101,6 +102,8 @@ A file name of a certificate should be 'cert-hash.0'.
 	(insert (format "verify=%d\n" (mew-ssl-verify-level case)))
 	(insert "foreground=yes\n")
 	(insert "debug=debug\n")
+	(if (>= mew-ssl-minor-ver 22)
+	    (insert "syslog=no\n"))
 	(insert "CApath=" (expand-file-name (mew-ssl-cert-directory case)) "\n")
 	(insert (format "[%d]\n" localport))
 	(insert (format "accept=%s:%d\n" mew-ssl-localhost localport))
@@ -199,8 +202,7 @@ A local port number can be obtained the process name after ':'. "
 
 (defun mew-ssl-debug (label string)
   (when (mew-debug 'net)
-    (save-excursion
-      (set-buffer (get-buffer-create mew-buffer-debug))
+    (with-current-buffer (get-buffer-create mew-buffer-debug)
       (goto-char (point-max))
       (insert (format "\n<%s>\n%s\n" label string)))))
 
@@ -254,16 +256,20 @@ A local port number can be obtained the process name after ':'. "
 (defun mew-ssl-setup ()
   (if (not (mew-which-exec mew-prog-ssl))
       (setq mew-ssl-ver nil)
-    (let ((status (call-process mew-prog-ssl nil nil nil "-V")))
-      (if (= status 0)
-	  (setq mew-ssl-ver 3)
-	(setq mew-ssl-ver 4)))))
+    (with-temp-buffer
+      (call-process mew-prog-ssl nil t nil "-version")
+      (goto-char (point-min))
+      (if (looking-at "stunnel 4\\.\\([0-9]+\\)")
+	  (progn
+	    (setq mew-ssl-ver 4)
+	    (setq mew-ssl-minor-ver (string-to-number (mew-match-string 1))))
+	(setq mew-ssl-ver 3)))))
 
 (provide 'mew-ssl)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2002-2008 Mew developing team.
+;; Copyright (C) 2002-2011 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
