@@ -142,6 +142,7 @@ If nodisplay is non-nil, displays the cached message."
   (mew-highlight-cursor-line)
   (when (and mew-use-unread-mark
 	     (mew-summary-message-number)
+	     ;; mark would be nil. MUST be equal
 	     (equal (mew-summary-get-mark) mew-mark-unread))
     (unless (and mew-inherit-after-marking
 		 (not mew-delete-unread-mark-by-mark))
@@ -446,8 +447,7 @@ a decrypted/verified body is displayed."
 		   (mew-cache-dinfo-get-use-alt cbuf)
 		 mew-use-alternative))
 	  syntax ct begin end)
-     (save-excursion
-       (set-buffer buf)
+     (with-current-buffer buf
        (mew-erase-buffer)
        (mew-insert-message fld msg mew-cs-text-for-read nil)
        (mew-dinfo-set nil t nil alt)
@@ -613,7 +613,7 @@ ad-hoc solution for putting the review mark on a thread.")
     t)
    (t
     (mew-decode-syntax-delete)
-    (previous-line 1) ;; for invisible
+    (forward-line -1) ;; for invisible
     (unless (pos-visible-in-window-p)
       ;; This ensures that some messages are displayed in Summary.
       (recenter))
@@ -763,8 +763,9 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 	(setq col (current-column))
 	(mew-sinfo-set-column col))
       (while (> arg 0)
-	(previous-line 1) ;; for invisible
-	(while (looking-at mew-regex-thread-separator)
+	(forward-line -1) ;; for invisible
+	(while (or (looking-at mew-regex-thread-separator)
+		   (eq (get-text-property (point) 'invisible) t))
 	  (forward-line -1))
 	(setq arg (1- arg)))
       (move-to-column col))))
@@ -785,7 +786,7 @@ over the window. Type '\\[mew-summary-prev-page]' to see them when a message is
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2008 Mew developing team.
+;; Copyright (C) 1996-2011 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
