@@ -366,12 +366,13 @@ system."
 	      (size (mew-sumsyn-message-size))
 	      (uid (mew-sumsyn-message-uid))
 	      (file (concat (file-name-as-directory fld) msg))
-	      (path (mew-expand-folder file)))
+	      (path (mew-expand-folder file))
+	      (rpath (mew-msg-get-filename path)))
 	 (if (mew-virtual-p)
 	     (setq file (mew-folder-path-to-folder file 'has-proto)))
 	 (message "%s size=%s uid=%s" file size uid)
 	 (when mew-use-cut-file-name
-	   (kill-new path)))))))
+	   (kill-new rpath)))))))
 
 (defun mew-sumsyn-match (regex)
   (save-excursion
@@ -385,7 +386,7 @@ system."
   (let ((fn (mew-match-string 1))
 	(bn (buffer-name)))
     (cond
-     ((not (string= fn "")) fn)
+     ((not (string= fn "")) (mew-sumsym-decode-folder fn))
      ((mew-virtual-for-one-summary)
       (mew-vinfo-get-physical-folder))
      (t bn))))
@@ -572,7 +573,7 @@ system."
        (mew-xinfo-set-multi-form nil)
        (mew-xinfo-set-icon-spec nil)
        (mew-syntax-multipart syntax nil nil 'mew-draft-button)
-       (mapcar 'insert-and-inherit (nreverse (mew-xinfo-get-multi-form)))
+       (mapc 'insert-and-inherit (nreverse (mew-xinfo-get-multi-form)))
        (put-text-property (mew-attach-begin) (point-max) 'read-only t)
        (mew-front-nonsticky (mew-attach-begin) (1+ (mew-attach-begin)))
        (mew-rear-sticky (1- (point-max)) (point-max))
@@ -591,14 +592,13 @@ system."
 
 (defun mew-decode-syntax-print (sumbuf syntax form icon)
   (let* ((stx (mew-syntax-get-part syntax)))
-    (save-excursion
-      (set-buffer sumbuf)
+    (with-current-buffer sumbuf
       (when (mew-syntax-multipart-p stx)
 	(forward-line)
 	(mew-elet
 	 (let ((pos (point)))
 	   (mew-decode-syntax-begin-set)
-	   (mapcar 'mew-decode-syntax-insert form)
+	   (mapc 'mew-decode-syntax-insert form)
 	   (if (= pos (point))
 	       ;; Nothing was printed.
 	       (mew-decode-syntax-remove)
@@ -797,8 +797,7 @@ system."
 
 (defun mew-decode-syntax-delete ()
   (when (mew-decode-syntax-p)
-    (save-excursion
-      (set-buffer (mew-decode-syntax-buffer))
+    (with-current-buffer (mew-decode-syntax-buffer)
       (mew-xinfo-set-icon-spec nil)
       (mew-summary-toolbar-update)
       (save-excursion
@@ -933,8 +932,7 @@ system."
 
 (defun mew-summary-end-of-message-p ()
   (let (pos beg end)
-    (save-excursion
-      (set-buffer (mew-decode-syntax-buffer))
+    (with-current-buffer (mew-decode-syntax-buffer)
       (save-excursion
 	(setq pos (point))
 	(setq end (mew-decode-syntax-end))
@@ -1028,7 +1026,7 @@ system."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2008 Mew developing team.
+;; Copyright (C) 1996-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
