@@ -40,8 +40,8 @@ Second pass - Repeat the following procedure for top nodes linearly:
 	      node list.
 
 If you have bogus messages and the second pass is carried out, thread
-structure MAY loop. This results in an infinite loop of visualizing 
-threads (not making threads). 
+structure MAY loop. This results in an infinite loop of visualizing
+threads (not making threads).
 
 Mew does not provide any loop detection/avoidance mechanism. So, you
 should understand this risk."
@@ -52,8 +52,8 @@ should understand this risk."
   "*Vector of strings to be used for indentation of thread.
 
 This consists of four members; 1st member for prefixing to a child
-message that is not the last one, 2nd member is for prefixing to the 
-last child, 3rd and 4th members are for prefixing to grand-child thread trees, 
+message that is not the last one, 2nd member is for prefixing to the
+last child, 3rd and 4th members are for prefixing to grand-child thread trees,
 4th member is for the child tree of the last child message.
 
 Example1: [\" +\" \" +\" \" |\" \"  \"] makes thread view below.
@@ -85,7 +85,7 @@ All members must have the same length."
   "*If non-nil, the specified string is inserted between threads.")
 (defvar mew-thread-separator "--")
 
-(defsubst mew-thread-insert-separator ()
+(defun mew-thread-insert-separator ()
   (if (and mew-use-thread-separator
 	   (/= (save-excursion (beginning-of-line) (point)) 1))
       (insert mew-thread-separator "\n")))
@@ -98,34 +98,34 @@ All members must have the same length."
 (defun mew-thread-make-entry ()
   (make-vector 5 nil))
 
-(defsubst mew-thread-get-myid (entry)
+(defun mew-thread-get-myid (entry)
   (aref entry 0))
 
-(defsubst mew-thread-get-prntid (entry)
+(defun mew-thread-get-prntid (entry)
   (aref entry 1))
 
-(defsubst mew-thread-get-child (entry)
+(defun mew-thread-get-child (entry)
   (aref entry 2))
 
-(defsubst mew-thread-get-msg (entry)
+(defun mew-thread-get-msg (entry)
   (aref entry 3))
 
-(defsubst mew-thread-get-line (entry)
+(defun mew-thread-get-line (entry)
   (aref entry 4))
 
-(defsubst mew-thread-set-myid (entry myid)
+(defun mew-thread-set-myid (entry myid)
   (aset entry 0 myid))
 
-(defsubst mew-thread-set-prntid (entry prntid)
+(defun mew-thread-set-prntid (entry prntid)
   (aset entry 1 prntid))
 
-(defsubst mew-thread-set-child (entry child)
+(defun mew-thread-set-child (entry child)
   (aset entry 2 child))
 
-(defsubst mew-thread-set-msg (entry msg)
+(defun mew-thread-set-msg (entry msg)
   (aset entry 3 msg))
 
-(defsubst mew-thread-set-line (entry line)
+(defun mew-thread-set-line (entry line)
   (aset entry 4 line))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -151,7 +151,7 @@ All members must have the same length."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
-;;; Making thread
+;;; Commands
 ;;;
 
 (defun mew-summary-mark-thread ()
@@ -159,30 +159,26 @@ All members must have the same length."
   (interactive)
   (mew-summary-thread-region (point-min) (point-max) 'mark))
 
-(defsubst mew-thread-cache-valid-p (vfolder)
+(defun mew-thread-cache-valid-p (vfolder)
   (let ((cfolder (mew-summary-folder-name 'ext))
 	ofld)
-    (save-excursion
-      (when (get-buffer vfolder)
-	(set-buffer vfolder)
-	(setq ofld (mew-vinfo-get-parent-folder))
+    (when (get-buffer vfolder)
+      (with-current-buffer vfolder
+	(setq ofld (mew-vinfo-get-original-folder))
 	(and (equal ofld cfolder)
 	     (get-buffer ofld)
 	     (equal (mew-sinfo-get-cache-time)
 		    (progn (set-buffer ofld) (mew-sinfo-get-cache-time))))))))
 
 (defun mew-summary-make-thread (&optional arg)
-  "If called in Summary mode or in Virtual mode made of a single
-physical folder, this command makes threads for the Summary mode
-as Virtual mode, then the cursor jump onto the current message in
-the Virtual mode. If a corresponding Virtual mode exists, this
-command just visits the Virtual mode.
+  "If called in Summary mode or Selection, make threads for
+all messages.
 
-If called with '\\[universal-argument]' in Summary mode, make
-threads for messages in the region.
+If called with '\\[universal-argument]', make threads for
+messages in the region.
 
-If called in Thread (Virtual) mode, switch back to the
-corresponding Summary mode and move to the current message."
+If called in Thread, switch back to the corresponding Summary
+mode or Selection."
   (interactive "P")
   (if (mew-mark-active-p) (setq arg t))
   (if arg
@@ -195,21 +191,15 @@ corresponding Summary mode and move to the current message."
 	   (folder (mew-summary-folder-name 'ext)) ;; xxx
 	   fld vfolder)
       (cond
-       ((mew-virtual-p)
-	(cond
-	 ((mew-thread-p)
-	  (setq fld (mew-vinfo-get-parent-folder))
-	  (if (not (and fld (get-buffer fld)))
-	      (message "No original folder")
-	    (mew-summary-visit-folder fld nil 'no-ls)
-	    (mew-summary-toggle-disp-msg (if disp 'on 'off))
-	    (if (not msg)
-		(goto-char (point-max))
-	      (mew-summary-move-and-display msg))))
-	 ((mew-pickable-p)
-	  (mew-summary-thread-region (point-min) (point-max) nil msg))
-	 (t
-	  (message "No physical folder"))))
+       ((mew-thread-p)
+	(setq fld (mew-vinfo-get-original-folder))
+	(if (not (and fld (get-buffer fld)))
+	    (message "No original folder")
+	  (mew-summary-visit-folder fld nil 'no-ls)
+	  (mew-summary-toggle-disp-msg (if disp 'on 'off))
+	  (if (not msg)
+	      (goto-char (point-max))
+	    (mew-summary-move-and-display msg))))
        ((and (setq vfolder (mew-folder-to-thread folder))
 	     (mew-thread-cache-valid-p vfolder))
 	(mew-summary-visit-folder vfolder)
@@ -217,6 +207,8 @@ corresponding Summary mode and move to the current message."
 	(when msg
 	  (mew-summary-move-and-display msg)
 	  (mew-thread-move-cursor)))
+       ((mew-selection-p)
+	(mew-summary-thread-region (point-min) (point-max) nil msg))
        (t
 	(mew-summary-thread-region (point-min) (point-max) nil msg))))))
 
@@ -232,135 +224,158 @@ corresponding Summary mode and move to the current message."
       (setq iter (lambda () (mew-summary-search-regexp-visible regex))))
     (mew-summary-thread-region (point-min) (point-max) nil nil iter)))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Making thread
+;;;
+
+(defun mew-thread-get-iter (mark iter)
+  (cond
+   (iter iter)
+   (mark (lambda () (re-search-forward mew-regex-msg-review nil t)))
+   (t    (lambda () (not (eobp))))))
+
+(defun mew-thread-create-db (size)
+  (let ((dbsize
+	 (cond
+	  ((<= size 211) 211)
+	  ((<= size 1511) 1511)
+	  ((<= size 7211) 7211)
+	  (t 18211))))
+    (make-vector dbsize 0))) ;; hash
+
+(defun mew-thread-pass-1 (db func)
+  (let (start msg my-id prnt-id prnt-cld me top line prnt)
+    (save-excursion
+      (goto-char (point-min))
+      (while (funcall func)
+	(beginning-of-line)
+	(setq start (point))
+	(if (not (mew-sumsyn-match mew-regex-sumsyn-long))
+	    (forward-line)
+	  (setq msg     (mew-sumsyn-message-number))
+	  (setq my-id   (mew-sumsyn-my-id))
+	  (setq prnt-id (mew-sumsyn-parent-id))
+	  (forward-line)
+	  ;; Throw away properties here and give properties later.
+	  ;; This is faster than inheriting properties.
+	  (setq line (mew-buffer-substring start (point)))
+	  (setq me (mew-thread-make-entry))
+	  (mew-thread-set-msg me msg)
+	  (mew-thread-set-line me line)
+	  (if (string= my-id "")
+	      (setq top (cons me top))
+	    ;; some broken messages refer themselves
+	    ;; don't register me here so that his parent
+	    ;; will not be found.
+	    (if (or (string= prnt-id "") (string= my-id prnt-id))
+		(setq top (cons me top))
+	      (mew-thread-set-prntid me prnt-id)
+	      (setq prnt (symbol-value (intern-soft prnt-id db)))
+	      (if (null prnt)
+		  (setq top (cons me top))
+		(setq prnt-cld (mew-thread-get-child prnt))
+		(if prnt-cld
+		    (nconc prnt-cld (list me))
+		  (mew-thread-set-child prnt (list me)))))
+	    (mew-thread-set-myid me my-id)
+	    (set (intern my-id db) me)))))
+    top))
+
+(defun mew-summary-setup-vfolder (db top column)
+  (let* ((ofolder (mew-summary-folder-name 'ext))
+	 (vfolder (mew-folder-to-thread ofolder))
+	 (pfolder (mew-summary-physical-folder))
+	 (disp (mew-sinfo-get-disp-msg))
+	 (ctime (mew-sinfo-get-cache-time))
+	 (case (mew-sinfo-get-case)))
+    (mew-summary-switch-to-folder vfolder)
+    (mew-vinfo-set-mode 'thread)
+    (mew-vinfo-set-physical-folder pfolder)
+    (mew-vinfo-set-original-folder ofolder)
+    (mew-erase-buffer)
+    (mew-hscroll)
+    (mew-summary-toggle-disp-msg (if disp 'on 'off))
+    (mew-sinfo-set-cache-time ctime)
+    (mew-sinfo-set-case case)
+    (setq mew-summary-buffer-raw t)
+    (mew-vinfo-set-db db)
+    (mew-vinfo-set-top top)
+    (mew-vinfo-set-column column)))
+
+(defun mew-thread-pass-2 (db top)
+  (if (null mew-use-complete-thread)
+      (nreverse top)
+    ;; This may create looped thread.
+    ;; See mew-use-complete-thread for more information.
+    (let (prnt prnt-id prnt-cld ret)
+      (dolist (me top)
+	(if (not (and (mew-thread-get-myid me)
+		      (setq prnt-id (mew-thread-get-prntid me))))
+	    (setq ret (cons me ret))
+	  (setq prnt (symbol-value (intern-soft prnt-id db)))
+	  (if (null prnt)
+	      (setq ret (cons me ret))
+	    (setq prnt-cld (mew-thread-get-child prnt))
+	    (if prnt-cld
+		(setq prnt-cld (nconc prnt-cld (list me)))
+	      (mew-thread-set-child prnt (list me))))))
+      ret)))
+
+(defun mew-thread-postscript (mark disp-msg)
+  (when mark (mew-mark-undo-mark mew-mark-review))
+  (jit-lock-register 'mew-summary-cook-region)
+  (mew-summary-set-count-line)
+  (set-buffer-modified-p nil)
+  (if disp-msg
+      (mew-summary-move-and-display disp-msg)
+    (goto-char (point-max)))
+  (mew-thread-move-cursor))
+
+(defun mew-thread-debug-info (tm1 tm2 tm3 tm4 tm5 tm6)
+  (when (mew-debug 'thread)
+    (let* ((t1 (mew-time-calc tm2 tm1))
+	   (t2 (mew-time-calc tm4 tm3))
+	   (t3 (mew-time-calc tm6 tm5)))
+      (message "pass1 %f, pass2 %f, visual %f" t1 t2 t3))))
+
 (defun mew-summary-thread-region (beg end &optional mark disp-msg iter)
   "Make threads for messages in a region.  If you want to know how
 threads are created, see 'mew-use-complete-thread'."
   (interactive "r")
-  (mew-pickable
-   (when (mew-summary-exclusive-p)
-     (let* ((ofolder (mew-summary-folder-name 'ext))
-	    (vfolder (mew-folder-to-thread (mew-physical-folder ofolder)))
-	    (folder (mew-summary-physical-folder))
-	    (disp (mew-sinfo-get-disp-msg))
-	    (ctime (mew-sinfo-get-cache-time))
-	    (case (mew-sinfo-get-case))
-	    (column (mew-get-summary-form folder 'column))
-	    db size top start me prnt prnt-cld my-id prnt-id func msg line
-	    tm1 tm2 tm3 tm4 tm5 tm6)
-       (message "Making thread (first pass)...")
-       (save-restriction
-	 (narrow-to-region beg end)
-	 (setq size (count-lines beg end))
-	 (cond
-	  ((<= size 211)
-	   (setq size 211))
-	  ((<= size 1511)
-	   (setq size 1511))
-	  ((<= size 7211)
-	   (setq size 7211))
-	  (t
-	   (setq size 18211)))
-	 (setq db (make-vector size 0)) ;; hash
-	 (save-excursion
-	   (goto-char (point-min))
-	   (if mark
-	       (setq func (lambda ()
-			    (re-search-forward mew-regex-msg-review nil t)))
-	     (setq func (lambda () (not (eobp)))))
-           (if iter
-               (setq func iter))
-	   (setq tm1 (current-time))
-	   (while (funcall func)
-	     (beginning-of-line)
-	     (setq start (point))
-	     (if (not (mew-sumsyn-match mew-regex-sumsyn-long))
-		 (forward-line)
-	       (setq msg     (mew-sumsyn-message-number))
-	       (setq my-id   (mew-sumsyn-my-id))
-	       (setq prnt-id (mew-sumsyn-parent-id))
-	       (forward-line)
-	       ;; Throw away properties here and give properties later.
-	       ;; This is faster than inheriting properties.
-	       (setq line (mew-buffer-substring start (point)))
-	       (setq me (mew-thread-make-entry))
-	       (mew-thread-set-msg me msg)
-	       (mew-thread-set-line me line)
-	       (if (string= my-id "")
-		   (setq top (cons me top))
-		 ;; some broken messages refer themselves
-		 ;; don't register me here so that his parent
-		 ;; will not be found.
-		 (if (string= prnt-id "")
-		     (setq top (cons me top))
-		   (mew-thread-set-prntid me prnt-id)
-		   (setq prnt (symbol-value (intern-soft prnt-id db)))
-		   (if (null prnt)
-		       (setq top (cons me top))
-		     (setq prnt-cld (mew-thread-get-child prnt))
-		     (if prnt-cld
-			 (nconc prnt-cld (list me))
-		       (mew-thread-set-child prnt (list me)))))
-		 (mew-thread-set-myid me my-id)
-		 (set (intern my-id db) me))))))
-       ;;
-       (if (null me)
- 	   (message "No target messages")
-	 (setq tm2 (current-time))
-	 (mew-summary-switch-to-folder vfolder ofolder)
-	 (mew-erase-buffer)
-	 (mew-hscroll)
-	 (mew-summary-toggle-disp-msg (if disp 'on 'off))
-	 (mew-sinfo-set-cache-time ctime)
-	 (mew-sinfo-set-case case)
-	 (setq mew-summary-buffer-raw t)
-	 (mew-vinfo-set-top nil)
-	 (mew-vinfo-set-db db)
- 	 (mew-vinfo-set-column column)
-	 ;;
-	 (message "Making thread (second pass)...")
-	 (setq tm3 (current-time))
-	 (if (null mew-use-complete-thread)
-	     (mew-vinfo-set-top (nreverse top))
-	   ;; This may create looped thread.
-	   ;; See mew-use-complete-thread for more information.
-	   (while top
-	     (setq me (car top))
-	     (if (not (and (mew-thread-get-myid me)
-			   (setq prnt-id (mew-thread-get-prntid me))))
-		 (mew-vinfo-set-top (cons me (mew-vinfo-get-top)))
-	       (setq prnt (symbol-value (intern-soft prnt-id db)))
-	       (if (null prnt)
-		   (mew-vinfo-set-top (cons me (mew-vinfo-get-top)))
-		 (setq prnt-cld (mew-thread-get-child prnt))
-		 (if prnt-cld
-		     (nconc prnt-cld (list me))
-		   (mew-thread-set-child prnt (list me)))))
-	     (setq top (cdr top))))
-	 ;;
-	 (setq tm4 (current-time))
-	 (message "Displaying thread...")
-	 (setq tm5 (current-time))
-	 (mew-summary-thread-print-top
-	  (mew-vinfo-get-top) folder column)
-	 ;;
-	 ;; Unmarking in both Summary and Thread
-	 (when mark (mew-mark-undo-mark mew-mark-review))
-	 (setq tm6 (current-time))
-	 (when mew-gemacs-p
-	   (jit-lock-register 'mew-summary-cook-region))
-	 (mew-summary-set-count-line)
-	 (set-buffer-modified-p nil)
-	 (if disp-msg
-	     (mew-summary-move-and-display disp-msg)
-	   (goto-char (point-max)))
-	 (mew-thread-move-cursor)
-	 (message "Displaying thread...done")
-	 (run-hooks 'mew-thread-display-hook)
-	 (when (mew-debug 'thread)
-	   (let* ((t1 (mew-time-calc tm2 tm1))
-		  (t2 (mew-time-calc tm4 tm3))
-		  (t3 (mew-time-calc tm6 tm5)))
-	     (message "pass1 %f, pass2 %f, visual %f" t1 t2 t3))))))))
+  (when (mew-summary-exclusive-p)
+    (let* ((column (or (mew-sinfo-get-summary-column) ;; scanned
+		       ;; Summary only
+		       (mew-get-summary-column (mew-summary-folder-name 'ext))))
+	   db top tm1 tm2 tm3 tm4 tm5 tm6)
+      (save-restriction
+	(narrow-to-region beg end)
+	(setq db (mew-thread-create-db (count-lines beg end)))
+	;;
+	(message "Making thread (first pass)...")
+	(setq tm1 (current-time))
+	(setq top (mew-thread-pass-1 db (mew-thread-get-iter mark iter)))
+	(setq tm2 (current-time)))
+      ;;
+      (if (null top)
+	  (message "No target messages")
+	(message "Making thread (second pass)...")
+	(setq tm3 (current-time))
+	(setq top (mew-thread-pass-2 db top))
+	(setq tm4 (current-time))
+	;;
+	(mew-summary-setup-vfolder db top column)
+	;;
+	(message "Displaying thread...")
+	(setq tm5 (current-time))
+	(mew-summary-thread-print-top (mew-vinfo-get-top) column)
+	(setq tm6 (current-time))
+	;;
+	(mew-thread-postscript mark disp-msg)	
+	;;
+	(message "Displaying thread...done")
+	(run-hooks 'mew-thread-display-hook)
+	(mew-thread-debug-info tm1 tm2 tm3 tm4 tm5 tm6)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -400,10 +415,10 @@ threads are created, see 'mew-use-complete-thread'."
 ;;; Visualizing thread
 ;;;
 
-(defun mew-summary-thread-print-top (top folder column)
-  (while top
-    (let* ((me (car top))
-	   (cld (mew-thread-get-child me)))
+(defun mew-summary-thread-print-top (top column)
+  (let (cld)
+    (dolist (me top)
+      (setq cld (mew-thread-get-child me))
       (mew-elet
        (mew-thread-insert-separator)
        (insert (mew-thread-get-line me))
@@ -411,10 +426,9 @@ threads are created, see 'mew-use-complete-thread'."
        (move-to-column column)
        (mew-thread-put-property (point) (1+ (point)) 0)
        (forward-line))
-      (if cld (mew-summary-thread-print-tree cld folder column))
-      (setq top (cdr top)))))
+      (if cld (mew-summary-thread-print-tree cld column)))))
 
-(defun mew-summary-thread-print-tree (tree folder column)
+(defun mew-summary-thread-print-tree (tree column)
   (let ((tree-stack nil)
 	(prefix "")
 	(level 1) pos)
@@ -482,8 +496,7 @@ threads are created, see 'mew-use-complete-thread'."
   "Put the 'o' mark on all messages of the current sub-thread."
   (interactive)
   (mew-thread-only
-   (let* ((fld (mew-physical-folder (mew-summary-folder-name 'ext)))
-	  ;; mew-vinfo-get-parent-folder cannot be used here
+   (let* ((fld (mew-folder-basename (mew-summary-folder-name 'ext)))
 	  (folders (mew-summary-refile-body nil nil nil 'no-mark))
 	  (folders-str (mew-join "," folders))
 	  (func (lambda ()
@@ -510,18 +523,13 @@ with the current folder as a candidate in addition to guessed folders."
        (mew-refile-set-from-alist alist folders)))))
 
 (defun mew-refile-set-from-alist (alist folders)
-  (let (ent fld msg msgs)
-    (while alist
-      (setq ent (car alist))
-      (setq alist (cdr alist))
+  (let (fld)
+    (dolist (ent alist)
       (setq fld (car ent))
-      (setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
-      (while msgs
-	(setq msg (number-to-string (car msgs)))
-	(setq msgs (cdr msgs))
+      (dolist (msg (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
+	(setq msg (number-to-string msg))
 	(when (get-buffer fld)
-	  (save-excursion
-	    (set-buffer fld)
+	  (with-current-buffer fld
 	    (mew-refile-reset msg)
 	    (mew-refile-set msg folders)))))))
 
@@ -619,19 +627,16 @@ with the current folder as a candidate in addition to guessed folders."
 	(mew-thread-unmark-physical-from-alist alist))))))
 
 (defun mew-thread-unmark-physical-from-alist (alist)
-  (let (ent fld msg msgs)
-    (while alist
-      (setq ent (car alist))
-      (setq alist (cdr alist))
+  (let (fld msgs)
+    (dolist (ent alist)
       (setq fld (car ent))
       (setq msgs (sort (copy-sequence (cdr ent)) '<)) ;; sort has side effect
       (when (get-buffer fld)
 	(set-buffer fld)
 	(save-excursion
 	  (goto-char (point-min))
-	  (while msgs
-	    (setq msg (number-to-string (car msgs)))
-	    (setq msgs (cdr msgs))
+	  (dolist (msg msgs)
+	    (setq msg (number-to-string msg))
 	    (when (re-search-forward (mew-regex-sumsyn-msg msg) nil t)
 	      (mew-thread-undo fld msg)
 	      (mew-mark-unmark))))))))
@@ -680,58 +685,125 @@ a top node, move onto the top of the previous thread."
        (mew-thread-move-cursor)
        (mew-summary-display)))))
 
-(defun mew-summary-thread-parent ()
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Diag
+;;;
+
+(defun mew-summary-parent-global (par-id)
+  (mew-summary-diag-global par-id "-p" "Parent"))
+
+(defun mew-summary-child-global (my-id)
+  (mew-summary-diag-global my-id "-c" "Child"))
+
+(defun mew-summary-diag-global (id opt who)
+  (mew-msgid-check
+   (let ((db (mew-expand-file "+" mew-id-db-file))
+	 (regex (format "\\(.*\\)/\\([0-9]+\\)\\(%s\\)?$" (regexp-quote mew-suffix)))
+	 path msg folder)
+     (with-temp-buffer
+       (mew-piolet mew-cs-text-for-read mew-cs-text-for-write
+	 (call-process mew-prog-smew nil t nil opt id db "")
+	 (goto-char (point-min))
+	 (when (looking-at regex)
+	   (setq path (mew-match-string 1))
+	   (setq msg (mew-match-string 2)))))
+     (if (not msg)
+	 nil
+       (setq folder (mew-folder-path-to-folder path))
+       (when folder
+	 (mew-summary-visit-folder folder nil 'no-ls)
+	 (if (mew-summary-search-msg msg)
+	     (progn
+	       (mew-summary-display)
+	       t)
+	   (concat who " not found. Scan 'update would be necessary")))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Parent
+;;;
+
+(defun mew-summary-parent ()
   "Move onto the parent message of the current message."
   (interactive)
   (mew-summary-goto-message)
   (mew-decode-syntax-delete)
-  (let ((pos (point))
-	(par-id (mew-summary-parent-id))
-	key)
-    (if (or (null par-id) (string= par-id ""))
-	(message "No required info")
-      (setq key (mew-regex-sumsyn-my-id par-id))
-      (if (or (re-search-backward key nil t)
-	      (progn
-		(goto-char (point-max))
-		(re-search-backward key nil t)))
-	  (progn
-	    (mew-thread-move-cursor)
-	    (mew-summary-display)
-	    (message "Parent found"))
-	(goto-char pos)
-	(message "No parent")))))
+  (let ((par-id (mew-summary-parent-id)) result)
+    (cond
+     ((or (null par-id) (string= par-id ""))
+      (message "No parent"))
+     ((mew-summary-parent-local par-id)
+      (message "Parent found"))
+     ((and (y-or-n-p "No parent in this folder. Find in others? ")
+	   (setq result (mew-summary-parent-global par-id)))
+      (if (eq result t)
+	  (message "Parent found")
+	(message "%s" result)))
+     (t
+      (message "Parent not found")))))
 
-(defun mew-summary-thread-child ()
-  "Move onto the child message of the current message."
+(defun mew-summary-parent-local (par-id)
+  (let ((pos (point))
+	(key (mew-regex-sumsyn-my-id par-id)))
+    (if (or (re-search-backward key nil t)
+	    (re-search-forward  key nil t))
+	(progn
+	  (mew-thread-move-cursor)
+	  (mew-summary-display)
+	  t)
+      (goto-char pos)
+      nil)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Child
+;;;
+
+(defun mew-summary-child ()
+  "Move onto the first child message of the current message."
   (interactive)
   (mew-summary-goto-message)
   (mew-decode-syntax-delete)
+  (let ((my-id (mew-summary-my-id)) result)
+    (cond
+     ((or (null my-id) (string= my-id ""))
+      (message "No child"))
+     ((mew-summary-child-local my-id)
+      (message "Child found"))
+     ((and (y-or-n-p "No child in this folder. Find in others? ")
+           (setq result (mew-summary-child-global my-id)))
+      (if (eq result t)
+	  (message "Child found")
+	(message "%s" result)))
+     (t
+      (message "Child not found")))))
+
+(defun mew-summary-child-local (my-id)
   (let ((pos (point))
-	(my-id (mew-summary-my-id))
-	key)
-    (if (or (null my-id) (string= my-id ""))
-	(message "No required info")
-      (setq key (mew-regex-sumsyn-par-id my-id))
-      (if (or (re-search-forward key nil t)
-	      (progn
-		(goto-char (point-min))
-		(re-search-forward key nil t)))
-	  (progn
-	    (mew-thread-move-cursor)
-	    (mew-summary-display)
-	    (message "Child found"))
-	(goto-char pos)
-	(message "No child")))))
+        (key (mew-regex-sumsyn-par-id my-id)))
+  (if (or (re-search-forward  key nil t)
+          (re-search-backward key nil t))
+      (progn
+        (mew-thread-move-cursor)
+        (mew-summary-display)
+        t)
+    (goto-char pos)
+    nil)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;
+;;; Sibling
+;;;
 
 (defun mew-summary-thread-sibling-up ()
   "Search backward by one sibling message of the current message."
   (interactive)
-  (let ((pos (point)) 
+  (let ((pos (point))
 	(par-id (mew-summary-parent-id))
 	key)
     (if (or (null par-id) (string= par-id ""))
-	(message "No required info")
+	(message "No sibling")
       (setq key (mew-regex-sumsyn-par-id par-id))
       (if (re-search-backward key nil t)
 	  (progn
@@ -739,7 +811,7 @@ a top node, move onto the top of the previous thread."
 	    (mew-summary-display)
 	    (message "Sibling found"))
 	(goto-char pos)
-	(message "No sibling")))))
+	(message "Sibling not found")))))
 
 (defun mew-summary-thread-sibling-down ()
   "Search forward by one sibling message of the current message."
@@ -748,7 +820,7 @@ a top node, move onto the top of the previous thread."
 	(par-id (mew-summary-parent-id))
 	key)
     (if (or (null par-id) (string= par-id ""))
-	(message "No required info")
+	(message "No sibling")
       (setq key (mew-regex-sumsyn-par-id par-id))
       (forward-line)
       (if (re-search-forward key nil t)
@@ -757,7 +829,7 @@ a top node, move onto the top of the previous thread."
 	    (mew-summary-display)
 	    (message "Sibling found"))
 	(goto-char pos)
-	(message "No sibling")))))
+	(message "Sibling not found")))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;;
@@ -809,7 +881,7 @@ a top node, move onto the top of the previous thread."
 
 (defun mew-thread-toggle ()
   "If children of a message are displayed, they will hide and
-\"+\" is displayed on the parent. 
+\"+\" is displayed on the parent.
 If the children are hidden, they will appear."
   (interactive)
   (mew-thread-only
@@ -948,7 +1020,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
   (save-excursion
     (mew-summary-goto-message)
     (beginning-of-line)
-    ;; parent    
+    ;; parent
     (let ((id (mew-summary-my-id))
 	  fld msg)
       (goto-char (mark-marker));; user's mark
@@ -1079,13 +1151,13 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 2000-2007 Mew developing team.
+;; Copyright (C) 2000-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -1094,7 +1166,7 @@ The thread/message is specified with the mark(\\[set-mark-command])."
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
