@@ -49,6 +49,7 @@
 (defvar mew-prog-plain    'mew-mime-text/plain)
 (defvar mew-prog-html     '(mew-mime-text/html mew-mime-text/html-ext))
 (defvar mew-prog-xml      '(mew-mime-text/xml  mew-mime-text/xml-ext))
+(defvar mew-prog-patch    '(mew-mime-text/plain mew-mime-text/patch-ext))
 (defvar mew-prog-enriched 'mew-mime-text/enriched)
 (defvar mew-prog-text     'mew-mime-text/plain)
 (defvar mew-prog-audio
@@ -84,6 +85,8 @@
 				   mew-mime-application-ms-tnef-ext))
 (defvar mew-prog-rtf             '(mew-mime-application/rtf
 				   mew-mime-application/rtf-ext))
+(defvar mew-prog-unzip           'mew-mime-application/octet-stream)
+
 ;;;
 ;;; Misc
 ;;;
@@ -104,7 +107,7 @@
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2008 Mew developing team.
+;; Copyright (C) 1996-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
