@@ -17,9 +17,9 @@
  'mew-basic-folders
  (list mew-inbox-folder mew-draft-folder mew-queue-folder mew-postq-folder))
 
-(mew-defvar 
+(mew-defvar
  'mew-scan-fields
- (list "Folder:" "Filename:" 
+ (list "Folder:" "Filename:"
        mew-subj: mew-date: mew-from: mew-to: mew-cc:
        mew-ct: mew-cte: mew-x-mew-uidl:
        mew-message-id: mew-in-reply-to: mew-references: mew-x-mew-ref:
@@ -51,7 +51,7 @@
  'mew-regex-message-files3
  (format "^\\([0-9]+\\)\\(%s\\)?$" (regexp-quote mew-suffix)))
 
-;; mewl and grep 
+;; mewl and grep
 (mew-defvar
  'mew-regex-message-files2
  (format "^\\([0-9]+\\)\\(%s\\)?" (regexp-quote mew-suffix)))
@@ -101,6 +101,7 @@
    ("image/gif"   "\\.gif$"   mew-b64 mew-prog-image mew-icon-image gif)
    ("image/tiff"  "\\.tif?f$" mew-b64 mew-prog-image mew-icon-image tiff)
    ("image/jpeg"  "\\.jpe?g$" mew-b64 mew-prog-image mew-icon-image jpeg)
+   ("image/pjpeg" "\\.jfif$"  mew-b64 mew-prog-image mew-icon-image jpeg) ;; MS
    ("image/png"   "\\.png$"   mew-b64 mew-prog-image mew-icon-image png)
    ("image/x-xwd" "\\.xwd$"   mew-b64 mew-prog-image mew-icon-image xwd)
    ("image/x-xbm" "\\.xbm$"   mew-b64 mew-prog-image mew-icon-image xbm)
@@ -137,6 +138,22 @@
     mew-prog-msexcel                mew-icon-text)
    ("application/vnd.ms-powerpoint" "\\.ppt$"  mew-b64
     mew-prog-mspowerpoint           mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.wordprocessingml.document" "\\.docx$" mew-b64
+    mew-prog-msword                 mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.wordprocessingml.template" "\\.dotx$" mew-b64
+    mew-prog-msword                 mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet" "\\.xlsx$" mew-b64
+    mew-prog-msexcel                mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.spreadsheetml.template" "\\.xltx$" mew-b64
+    mew-prog-msexcel                mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.presentationml.presentation" "\\.pptx$" mew-b64
+    mew-prog-mspowerpoint           mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.presentationml.slideshow" "\\.ppsx$" mew-b64
+    mew-prog-mspowerpoint           mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.presentationml.template" "\\.potx$" mew-b64
+    mew-prog-mspowerpoint           mew-icon-text)
+   ("application/vnd.openxmlformats-officedocument.presentationml.slide" "\\.sldx$" mew-b64
+    mew-prog-mspowerpoint           mew-icon-text)
    ("application/vnd.visio"         "\\.vsd$"  mew-b64
     mew-prog-visio                  mew-icon-text)
    ("application/ms-tnef"           "\\.dat$" mew-b64
@@ -153,6 +170,10 @@
     mew-prog-oasys                  mew-icon-text)
    ("application/vnd.fujitsu.oasys3" "\\.oa3$" mew-b64
     mew-prog-oasys                  mew-icon-text)
+   ("application/zip"               "\\.zip$" mew-b64
+    mew-prog-zip                    mew-icon-application/octet-stream)
+   ("application/x-zip-compressed"  "\\.zip$" mew-b64
+    mew-prog-zip                    mew-icon-application/octet-stream)
    ("application/octet-stream"
     "\\.tar$\\|\\.tar\\.\\|\\.gz$\\|\\.Z$\\|\\.taz$\\|\\.tgz$\\|\\.tbz$\\|\\.bz2?$\\|\\.lzh$\\|\\.zip$\\|\\.bin$\\|\\.pgp$\\|\\.gpg$\\|\\.exe$\\|\\.dll$"
     mew-b64 mew-prog-octet-stream mew-icon-application/octet-stream)
@@ -161,17 +182,20 @@
    ("text/enriched" "\\.rtf$"   nil     mew-prog-enriched  mew-icon-text)
    ("text/css"      "\\.css$"   nil     mew-prog-text      mew-icon-text)
    ("text/sgml"     "\\.sgml$"  nil     mew-prog-text      mew-icon-text)
+   ("text/xml"      "\\.xml$"   nil     mew-prog-xml       mew-icon-text)
+   ("text/calendar" "\\.ics$"   nil     mew-prog-text      mew-icon-text)
+   ("text/x-patch"  "\\.diff$\\|\\.patch$"
+    nil     mew-prog-patch      mew-icon-text)
    ("text/plain"    "\\.txt$\\|\\.c$\\|\\.h$\\|\\.el$\\|\\.diff$\\|\\.patch$"
     nil     mew-prog-plain     mew-icon-text)
-   ("text/xml"      "\\.xml$"   nil     mew-prog-xml       mew-icon-text)
    ("text/rfc822-headers" "\\.hdr$" nil
-    mew-prog-rfc822-headers     mew-icon-message/rfc822)
+    mew-prog-rfc822-headers    mew-icon-message/rfc822)
    ("text"         "^$"        nil     mew-prog-text      mew-icon-text)
    ;; Unknown CT: matches here.
    ("application/xml"               "\\.xml$"  mew-b64
-    mew-prog-xml2                   mew-icon-text)
+    mew-prog-xml2              mew-icon-text)
    (t               "^$"       nil   mew-prog-octet-stream mew-icon-unknown)
-   ;; Unknown suffix matches here and return the entry specified 
+   ;; Unknown suffix matches here and return the entry specified
    ;; by mew-content-type.
    (nil             ".*")))
 
@@ -206,24 +230,24 @@
    ("application/vnd.sun.xml.math"             "\\.sxm$")))
 
 (when mew-mime-content-type-for-ooffice
-  (let (ret)
-    (mapcar
-     (lambda (x) 
-       (setq ret (cons (list (nth 0 x) (nth 1 x) 'mew-b64 'mew-prog-ooffice 'mew-icon-text) ret)))
-     mew-mime-content-type-for-ooffice)
-    (setq mew-mime-content-type (nconc (nreverse ret) mew-mime-content-type))))
+  (setq mew-mime-content-type
+	(nconc
+	 (mapcar
+	  (lambda (x) (list (nth 0 x) (nth 1 x) 'mew-b64 'mew-prog-ooffice 'mew-icon-text))
+	  mew-mime-content-type-for-ooffice)
+	 mew-mime-content-type)))
 
 (provide 'mew-varsx)
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1996-2007 Mew developing team.
+;; Copyright (C) 1996-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
 ;; modification, are permitted provided that the following conditions
 ;; are met:
-;; 
+;;
 ;; 1. Redistributions of source code must retain the above copyright
 ;;    notice, this list of conditions and the following disclaimer.
 ;; 2. Redistributions in binary form must reproduce the above copyright
@@ -232,7 +256,7 @@
 ;; 3. Neither the name of the team nor the names of its contributors
 ;;    may be used to endorse or promote products derived from this software
 ;;    without specific prior written permission.
-;; 
+;;
 ;; THIS SOFTWARE IS PROVIDED BY THE TEAM AND CONTRIBUTORS ``AS IS'' AND
 ;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 ;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
