@@ -2,7 +2,7 @@
 
 ;; Author:  Kazu Yamamoto <Kazu@Mew.org>
 ;; Created: Mar 23, 1994
-;; Revised: Feb 16, 2009
+;; Revised: Aug 31, 2010
 
 ;;; Commentary:
 
@@ -18,7 +18,7 @@
 ;;; Mew version
 ;;;
 
-(defconst mew-version-number "6.2.51"
+(defconst mew-version-number "6.3.50"
   "Version number for this version of Mew.")
 (defconst mew-version (format "Mew version %s" mew-version-number)
   "Version string for this version of Mew.")
@@ -215,7 +215,7 @@ CONTINUE, YANK-ACTION and SEND-ACTIONS are ignored."
 
 (defun mew-status-update (&optional arg)
   "Read Addrbook and update its information.
-If executed with '\\[universal-argument]', information about folders 
+If executed with '\\[universal-argument]', information about folders
 of the current world is also updated."
   (interactive "P")
   (cond
@@ -388,8 +388,7 @@ the lower window is not zero, switch to the buffer."
     ;;
     (if (get-buffer msgbuf)
 	(delete-windows-on msgbuf)
-      (save-excursion
-	(set-buffer (get-buffer-create msgbuf))
+      (with-current-buffer (get-buffer-create msgbuf)
 	;; "truncate?" is asked in Message mode.
 	;; so set the same toolbar as Summary mode
 	(mew-summary-toolbar-update)
@@ -421,8 +420,7 @@ the lower window is not zero, switch to the buffer."
 	(i 1))
     (walk-windows
      (lambda (win)
-       (when (save-excursion
-	       (set-buffer (window-buffer win))
+       (when (with-current-buffer (window-buffer win)
 	       (mew-summary-or-virtual-p))
 	 (if (< (car (window-edges win)) x)
 	     (setq i (1+ i)))))
@@ -591,8 +589,7 @@ It is typically called by kill-emacs-hook."
 	 (folder (if (bufferp buf) (buffer-name buf) buf))
 	 obuf)
     (if (get-buffer buf)
-	(save-excursion
-	  (set-buffer buf)
+	(with-current-buffer buf
 	  (when (mew-summary-or-virtual-p)
 	    (mew-summary-kill-subprocess)
 	    (setq obuf (mew-local-buffer-name folder))
@@ -743,7 +740,7 @@ Mew remain, so you can resume with buffer operations."
 
 ;;; Copyright Notice:
 
-;; Copyright (C) 1994-2009 Mew developing team.
+;; Copyright (C) 1994-2010 Mew developing team.
 ;; All rights reserved.
 
 ;; Redistribution and use in source and binary forms, with or without
